#!/bin/bash
# -----------------------------------------------------------
#
#  Script for building ROHS firmware
#
#
# -----------------------------------------------------------
export NSP_HOME=$PWD
export TARGET_HOME=$NSP_HOME
export KERNEL_DIR=$TARGET_HOME/src/kernel/linux-2.4.17_mvl21
export PATH=$PATH:/opt/hardhat/previewkit/mips/mipsel-linux-uclibc/bin:/opt/hardhat/previewkit/mips/fp_le/bin
export LD_LIBRARY_PATH=/opt/hardhat/previewkit/mips/mipsel-linux-uclibc/lib
export FILESYS_DIR=$TARGET_HOME/filesystem
export DOT_PROFILE=$TARGET_HOME/.PROFILE
export PROFILE_ROOT=$TARGET_HOME/PROFILE

# profile: what platform are we building for?
PROFILE=AR7WRD

# save profile
echo "$PROFILE" > $DOT_PROFILE
PROFILE_DIR=$PROFILE_ROOT/$PROFILE

echo "profile=$PROFILE"
echo "profile_dir=$PROFILE_DIR"
echo "profile_root=$PROFILE_ROOT"
if [ ! -d "$PROFILE_DIR" ]; then
   echo "I cannot find profile directory: $PROFILE_DIR"
   exit 1
fi

# export it
export PROFILE
export PROFILE_DIR

# config file
CONF=$TARGET_HOME/.config
IS_RT=$TARGET_HOME/.isrt_tmp
BASE=$PROFILE_DIR/config_generic

# which switch? (default = realtek)
SWITCH=$PROFILE_DIR/realtek
if test "$1" = "marvell" || test "$2" = "marvell" || test "$3" = "marvell" || test "$4" = "marvell" || test "$5" = "marvell"; then
   SWITCH=$PROFILE_DIR/marvell
else
  if test "$1" = "noswitch" || test "$2" = "noswitch" || test "$3" = "noswitch" || test "$4" = "noswitch" || test "$5" = "noswitch"; then
     SWITCH=$PROFILE_DIR/noswitch
  fi
fi

# set the configuration for the switch
cat $SWITCH.txt > $CONF
cat $BASE >> $CONF
cat $SWITCH.txt >> $CONF
cat $IS_RT >> $CONF

# build
if test "$1" = "no-clean" || test "$2" = "no-clean" || test "$3" = "no-clean" || test "$4" = "no-clean" || test "$5" = "no-clean"; then
  echo -n ""
else
  make clean
fi

make nsp

# cleanup
rm -f $DOT_PROFILE
rm -f $IS_RT

