/****************************************************************

        bwb_fnc.c       Interpretation Routines
			for Predefined Functions
                        for Bywater BASIC Interpreter

    bwBASIC version 3.00 beta
    Copyright (c) 1992-2000, Ted A. Campbell
    Copyright (c) 1995-2000, Jon B. Volkoff

    http://www.bwBASIC.com
    email: bwBASIC@bwBASIC.com

    Copyright and Permissions Information:

    All U.S. and international rights are claimed by the authors.

    This software is released under the terms of the GNU General
    Public License (GPL), which is distributed with this software
    in the file "COPYING".  The GPL specifies the terms under
    which users may copy and use the software in this distribution.
    Please note that it rules out the distribution of proprietary
    versions of the software, and it rules out the use of altered
    versions of the software. This means that you cannot remove
    the Bywater BASIC name or copyright information.

    A separate license is available for proprietary or commercial
    distribution, for information on which you should contact
    bwBASIC@bwBASIC.com.

****************************************************************/

/*---------------------------------------------------------------*/
/* NOTE: Modifications marked "JBV" were made by Jon B. Volkoff, */
/* 11/1995 (eidetics@cerf.net).                                  */
/*---------------------------------------------------------------*/

#define FSTACKSIZE      32

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#include "bwbasic.h"
#include "bwb_mes.h"

#if UNIX_CMDS
#include <sys/stat.h>
#endif

#ifndef RAND_MAX			/* added in v1.11 */
#define RAND_MAX	32767
#endif

static time_t t;
static struct tm *lt;

/***************************************************************

        FUNCTION:       fnc_init()

        DESCRIPTION:    This command initializes the function
                        linked list, placing all predefined functions
                        in the list.

***************************************************************/

int
fnc_init( int task )
   {
   register int n;
   struct bwb_function *f;

   strcpy( fnc_start.name, "FNC_START" );
   fnc_start.type = 'X';
   fnc_start.vector = fnc_null;
   strcpy( fnc_end.name, "FNC_END" );
   fnc_end.type = 'x';
   fnc_end.vector = fnc_null;
   fnc_end.next = &fnc_end;

   f = &fnc_start;

   /* now go through each of the preestablished functions and set up
      links between them; from this point the program address the functions
      only as a linked list (not as an array) */

   for ( n = 0; n < FUNCTIONS; ++n )
      {
      f->next = &( bwb_prefuncs[ n ] );
      f = f->next;
      }

   /* link the last pointer to the end; this completes the list */

   f->next = &fnc_end;

   return TRUE;
   }

/***************************************************************

        FUNCTION:       fnc_find()

        DESCRIPTION:    This C function attempts to locate
                        a BASIC function with the specified name.
                        If successful, it returns a pointer to
                        the C structure for the BASIC function,
                        if not successful, it returns NULL.

***************************************************************/

struct bwb_function *
fnc_find( char *buffer )
   {
   struct bwb_function * f;
   register int n;
   static char *tbuf;
   static int init = FALSE;

   if ( strlen( buffer ) == 0 )
      {
      return NULL;
      }

   /* get memory for temporary buffer if necessary */

   if ( init == FALSE )
      {
      init = TRUE;

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_find" )) == NULL )
         {
#if PROG_ERRORS
	 bwb_error( "in fnc_find(): failed to find memory for tbuf" );
#else
	 bwb_error( err_getmem );
#endif
	 }
      }

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_find(): called for <%s> ", buffer );
   bwb_debug( bwb_ebuf );
#endif

   strcpy( tbuf, buffer );
   bwb_strtoupper( tbuf );

   for ( f = fnc_start.next; f != &fnc_end; f = f->next )
      {
      if ( strcmp( f->name, tbuf ) == 0 )
         {
#if INTENSIVE_DEBUG
	 sprintf( bwb_ebuf, "in fnc_find(): found <%s> ", f->name );
	 bwb_debug( bwb_ebuf );
#endif
         return f;
         }
      }

   /* search has failed: return NULL */

   return NULL;

   }

/***************************************************************

        FUNCTION:       fnc_null()

        DESCRIPTION:    This is a null function that can be used
                        to fill in a required function-structure
                        pointer when needed.

***************************************************************/

struct bwb_variable *
fnc_null( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_tab()

	DESCRIPTION:    This C function implements the BASIC TAB()
			function, adding tab spaces to a specified
			column.

			TAB is a core function, i.e., required
			for ANSI Minimal BASIC.

	SYNTAX:		TAB( number )

***************************************************************/

struct bwb_variable *
fnc_tab( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   static char t_string[ 4 ];
   bstring *b;

   /* initialize nvar if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) STRING );
      }

   /* check for correct number of parameters */

   if ( argc < 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function TAB().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      break_handler();
      return NULL;
      }
   else if ( argc > 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Too many parameters (%d) to function TAB().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      break_handler();
      return NULL;
      }

   t_string[ 0 ] = PRN_TAB;
   t_string[ 1 ] = (char) var_getnval( &( argv[ 0 ] ));
   t_string[ 2 ] = '\0';

   b = var_getsval( &nvar );
   str_ctob( b, t_string );

   return &nvar;
   }

#if COMMON_FUNCS

/***************************************************************


        FUNCTION:       fnc_date()

        DESCRIPTION:    This C function implements the BASIC
                        predefined DATE$ function, returning
                        a string containing the year, month,
                        and day of the month. 

			Revised by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)
                        Added optional argument for time string
                        formatting using strftime().  Format
                        specifications may vary depending on
                        your platform's implementation of the
                        strftime() function.

	SYNTAX:		DATE$([format-string])

***************************************************************/

struct bwb_variable *
fnc_date( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   char fmtstr[ MAXSTRINGSIZE + 1 ];

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function Date$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 0, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   time( &t );
   lt = localtime( &t );

   if (argc == 0) 
      {
      sprintf( tbuf, "%02d-%02d-%04d", lt->tm_mon + 1, lt->tm_mday,
         1900 + lt->tm_year );
      }
   else 
      {
      /* resolve the argument and place string value in tbuf */
      str_btoc( fmtstr, var_getsval( &( argv[ 0 ] )));
      strftime( tbuf, sizeof(tbuf), fmtstr, lt ); 
      }

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_time()

        DESCRIPTION:    This C function implements the BASIC
                        predefined TIME$ function, returning a
                        string containing the hour, minute, and
                        second count.

	SYNTAX:		TIME$

***************************************************************/

struct bwb_variable *
fnc_time( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static char *tbuf;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_time" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_time(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   time( &t );
   lt = localtime( &t );

   sprintf( tbuf, "%02d:%02d:%02d", lt->tm_hour, lt->tm_min,
      lt->tm_sec );
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_chr()

        DESCRIPTION:    This C function implements the BASIC
                        predefined CHR$ function, returning a
                        string containing the single character
                        whose ASCII value is the argument to
                        this function.

	SYNTAX:		CHR$( number )

***************************************************************/

struct bwb_variable *
fnc_chr( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static int init = FALSE;
#if TEST_BSTRING
   bstring *b;
#endif

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_chr(): entered function, argc <%d>",
      argc );
   bwb_debug( bwb_ebuf );
#endif

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
#if INTENSIVE_DEBUG
      sprintf( bwb_ebuf, "in fnc_chr(): entered function, initialized nvar" );
      bwb_debug( bwb_ebuf );
#endif
      }

   /* check arguments */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough arguments to function CHR$()" );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function CHR$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_chr(): entered function, checkargs ok" );
   bwb_debug( bwb_ebuf );
#endif

   tbuf[ 0 ] = (char) var_getnval( &( argv[ 0 ] ) );
   tbuf[ 1 ] = '\0';
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

#if TEST_BSTRING
   b = var_findsval( &nvar, nvar.array_pos );
   sprintf( bwb_ebuf, "in fnc_chr(): bstring name is <%s>", b->name );
   bwb_debug( bwb_ebuf );
#endif
#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_chr(): tbuf[ 0 ] is <%c>", tbuf[ 0 ] );
   bwb_debug( bwb_ebuf );
#endif

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_len()

	DESCRIPTION:    This C function implements the BASIC LEN()
			function, returning the length of a
			specified string in bytes.

	SYNTAX:		LEN( string$ )

***************************************************************/

struct bwb_variable *
fnc_len( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   static char *tbuf;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_len" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_len(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   /* check parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function LEN().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function LEN().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* return length as an integer */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )) );
   * var_findnval( &nvar, nvar.array_pos )
      = (bnumber) strlen( tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_pos()

	DESCRIPTION:    This C function implements the BASIC
			POS() function, returning the current
			column position for the output device.

	SYNTAX:		POS

***************************************************************/

struct bwb_variable *
fnc_pos( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize nvar if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) prn_col;

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_lcase()

        DESCRIPTION:    This C function implements the BASIC
			LCASE$() function, converting the
			characters of a specified string to
			lower case.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		LCASE$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_lcase( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function LCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function LCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   bwb_strtolower( tbuf );

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_ucase()

        DESCRIPTION:    This C function implements the BASIC
			UCASE$() function, converting the
			characters of a specified string to
			upper case.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		UCASE$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_ucase( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function UCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function UCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   bwb_strtoupper( tbuf );

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   /* return address of nvar */

   return &nvar;

   }

#endif					/* COMMON_FUNCS */

#if MS_FUNCS

/***************************************************************

        FUNCTION:       fnc_timer()

        DESCRIPTION:    This C function implements the BASIC
                        predefined TIMER function

	SYNTAX:		TIMER

***************************************************************/

struct bwb_variable *
fnc_timer( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   static time_t now;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   time( &now );
   /* Following statement was (bnumber) (JBV) */
   * var_findnval( &nvar, nvar.array_pos )
      = (float) fmod( (double) now, (double) (60*60*24));

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_mid()

        DESCRIPTION:    This C function implements the BASIC
                        predefined MID$ function

	SYNTAX:		MID$( string$, start-position-in-string[, number-of-spaces ] )

***************************************************************/

struct bwb_variable *
fnc_mid( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   register int c;
   char target_string[ MAXSTRINGSIZE + 1 ];
   int target_counter, num_spaces;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check arguments */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough arguments to function MID$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

   if ( argc > 3 )
      {
      sprintf( bwb_ebuf, "Two many arguments to function MID$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

#else
   if ( fnc_checkargs( argc, argv, 2, 3 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* get arguments */

   str_btoc( target_string, var_getsval( &( argv[ 0 ] ) ));
   target_counter = (int) var_getnval( &( argv[ 1 ] ) ) - 1;
   if ( target_counter > (int) strlen( target_string ))
      {
      tbuf[ 0 ] = '\0';
      str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );
      return &nvar;
      }

   if ( argc == 3 )
      {
      num_spaces = (int) var_getnval( &( argv[ 2 ] ));
      }
   else
      {
      num_spaces = MAXSTRINGSIZE;
      }

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_mid() string <%s> startpos <%d> spaces <%d>",
      target_string, target_counter, num_spaces );
   bwb_debug( bwb_ebuf );
#endif

   c = 0;
   tbuf[ c ] = '\0';
   while ( ( c < num_spaces ) && ( target_string[ target_counter ] != '\0' ))
      {
      tbuf[ c ] = target_string[ target_counter ];
      ++c;
      tbuf[ c ] = '\0';
      ++target_counter;
      }
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_left()

        DESCRIPTION:    This C function implements the BASIC
                        predefined LEFT$ function

	SYNTAX:		LEFT$( string$, number-of-spaces )

***************************************************************/

struct bwb_variable *
fnc_left( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   register int c;
   char target_string[ MAXSTRINGSIZE + 1 ];
   int target_counter, num_spaces;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check arguments */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough arguments to function LEFT$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

   if ( argc > 2 )
      {
      sprintf( bwb_ebuf, "Two many arguments to function LEFT$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

#else
   if ( fnc_checkargs( argc, argv, 2, 2 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* get arguments */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] ) ));
   target_counter = 0;
   num_spaces = (int) var_getnval( &( argv[ 1 ] ));

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_left() string <%s> startpos <%d> spaces <%d>",
      tbuf, target_counter, num_spaces );
   bwb_debug( bwb_ebuf );
#endif

   c = 0;
   target_string[ 0 ] = '\0';
   while (( c < num_spaces ) && ( tbuf[ c ] != '\0' ))
      {
      target_string[ target_counter ] = tbuf[ c ];
      ++target_counter;
      target_string[ target_counter ] = '\0';
      ++c;
      }
   str_ctob( var_findsval( &nvar, nvar.array_pos ), target_string );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_right()

        DESCRIPTION:    This C function implements the BASIC
                        predefined RIGHT$ function

	SYNTAX:		RIGHT$( string$, number-of-spaces )

***************************************************************/

struct bwb_variable *
fnc_right( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   register int c;
   char target_string[ MAXSTRINGSIZE + 1 ];
   int target_counter, num_spaces;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check arguments */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough arguments to function RIGHT$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

   if ( argc > 2 )
      {
      sprintf( bwb_ebuf, "Two many arguments to function RIGHT$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

#else
   if ( fnc_checkargs( argc, argv, 2, 2 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* get arguments */

   str_btoc( target_string, var_getsval( &( argv[ 0 ] ) ));
   target_counter = strlen( target_string ) - (int) var_getnval( &( argv[ 1 ] ));
   num_spaces = MAXSTRINGSIZE;

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_right() string <%s> startpos <%d> spaces <%d>",
      target_string, target_counter, num_spaces );
   bwb_debug( bwb_ebuf );
#endif

   c = 0;
   tbuf[ c ] = '\0';
   while ( ( c < num_spaces ) && ( target_string[ target_counter ] != '\0' ))
      {
      tbuf[ c ] = target_string[ target_counter ];
      ++c;
      tbuf[ c ] = '\0';
      ++target_counter;
      }
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_asc()

        DESCRIPTION:    This function implements the predefined
			BASIC ASC() function, returning the ASCII
			number associated with the first character
			in the string argument.

	SYNTAX:		ASC( string$ )

***************************************************************/

struct bwb_variable *
fnc_asc( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static char *tbuf;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_asc" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_asc(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   /* check parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ASC().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ASC().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   if ( argv[ 0 ].type != STRING )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Argument to function ASC() must be a string." );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_mismatch );
#endif
      return NULL;
      }

   /* assign ASCII value of first character in the buffer */

   str_btoc( tbuf, var_findsval( &( argv[ 0 ] ), argv[ 0 ].array_pos ) );
   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) tbuf[ 0 ];

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_asc(): string is <%s>",
      tbuf );
   bwb_debug( bwb_ebuf );
#endif

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_string()

        DESCRIPTION:    This C function implements the BASIC
			STRING$() function.

	SYNTAX:		STRING$( number, ascii-value|string$ )

***************************************************************/

struct bwb_variable *
fnc_string( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   int length;
   register int i;
   char c;
   static char *tbuf;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_string" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_string(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function STRING$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 2 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function STRING$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 2, 2 ) == FALSE )
      {
      return NULL;
      }
#endif

   strcpy( nvar.name, "(string$)!" );
   nvar.type = STRING;
   tbuf[ 0 ] = '\0';
   length = (int) var_getnval( &( argv[ 0 ] ));

   if ( argv[ 1 ].type == STRING )
      {
      str_btoc( tbuf, var_getsval( &( argv[ 1 ] )));
      c = tbuf[ 0 ];
      }
   else
      {
      c = (char) var_getnval( &( argv[ 1 ] ) );
      }

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_string(): argument <%s> arg type <%c>, length <%d>",
      tbuf, argv[ 1 ].type, length );
   bwb_debug( bwb_ebuf );
   sprintf( bwb_ebuf, "in fnc_string(): type <%c>, c <0x%x>=<%c>",
      argv[ 1 ].type, c, c );
   bwb_debug( bwb_ebuf );
#endif

   /* add characters to the string */

   for ( i = 0; i < length; ++i )
      {
      tbuf[ i ] = c;
      tbuf[ i + 1 ] = '\0';
      }
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_instr()

        DESCRIPTION:	This C function implements the BASIC 
			INSTR() function, returning the position
			in string string-searched$ at which
			string-pattern$ occurs.

	SYNTAX:		INSTR( [start-position,] string-searched$, string-pattern$ )

***************************************************************/

struct bwb_variable *
fnc_instr( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   int n_pos, x_pos, y_pos;
   int start_pos;
   register int n;
   char xbuf[ MAXSTRINGSIZE + 1 ];
   char ybuf[ MAXSTRINGSIZE + 1 ];

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function INSTR().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 3 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function INSTR().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 2, 3 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* determine argument positions */

   if ( argc == 3 )
      {
      n_pos = 0;
      x_pos = 1;
      y_pos = 2;
      }
   else
      {
      n_pos = -1;
      x_pos = 0;
      y_pos = 1;
      }

   /* determine starting position */

   if ( n_pos == 0 )
      {
      start_pos = (int) var_getnval( &( argv[ n_pos ] ) ) - 1;
      }
   else
      {
      start_pos = 0;
      }

   /* get x and y strings */

   str_btoc( xbuf, var_getsval( &( argv[ x_pos ] ) ) );
   str_btoc( ybuf, var_getsval( &( argv[ y_pos ] ) ) );

   /* now search for match */

   for ( n = start_pos; n < (int) strlen( xbuf ); ++n )
      {
      if ( strncmp( &( xbuf[ n ] ), ybuf, strlen( ybuf ) ) == 0 )
         {
         * var_findnval( &nvar, nvar.array_pos ) = (bnumber) n + 1;
         return &nvar;
         }
      }

   /* match not found */

   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) 0;
   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_spc()

	DESCRIPTION:    This C function implements the BASIC
			SPC() function, returning a string
			containing a specified number of
			(blank) spaces.

	SYNTAX:		SPC( number )

***************************************************************/

struct bwb_variable *
fnc_spc( int argc, struct bwb_variable *argv, int unique_id )
   {
   return fnc_space( argc, argv, unique_id );
   }

/***************************************************************

        FUNCTION:       fnc_space()

	DESCRIPTION:    This C function implements the BASIC
			SPACE() function, returning a string
			containing a specified number of
			(blank) spaces.

	SYNTAX:		SPACE$( number )

***************************************************************/

struct bwb_variable *
fnc_space( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static char *tbuf;
   static int init = FALSE;
   int spaces;
   register int i;
   bstring *b;

   /* check for correct number of parameters */

   if ( argc < 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function SPACE$().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      break_handler();
      return NULL;
      }
   else if ( argc > 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Too many parameters (%d) to function SPACE$().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      break_handler();
      return NULL;
      }

   /* initialize nvar if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) STRING );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_space" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_space(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   tbuf[ 0 ] = '\0';
   spaces = (int) var_getnval( &( argv[ 0 ] ));

   /* add spaces to the string */

   for ( i = 0; i < spaces; ++i )
      {
      tbuf[ i ] = ' ';
      tbuf[ i + 1 ] = '\0';
      }

   b = var_getsval( &nvar );
   str_ctob( b, tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_environ()

        DESCRIPTION:    This C function implements the BASIC
			ENVIRON$() function, returning the value
			of a specified environment string.

	SYNTAX:		ENVIRON$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_environ( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   char tmp[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ENVIRON$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ENVIRON$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   /* call getenv() then write value to string */

   /*--------------------------------------------------------------------*/
   /* Added check for getenv return value to prevent segmentation faults */
   /* JBV 3/15/96                                                        */
   /*--------------------------------------------------------------------*/
   if (getenv( tbuf ) != NULL) strcpy( tmp, getenv( tbuf ));
   else strcpy( tmp, "" );

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tmp );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_err()

	DESCRIPTION:    This C function implements the BASIC
			ERR function, returning the error number
			for the most recent error.

			Please note that as of revision level
			2.10, bwBASIC does not utilize a standard
			list of error numbers, so numbers returned
			by this function will not be those found
			in either ANSI or Microsoft or other
			BASIC error tables.

	SYNTAX:		ERR

***************************************************************/

struct bwb_variable *
fnc_err( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize nvar if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) err_number;

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_erl()

	DESCRIPTION:    This C function implements the BASIC
			ERL function, returning the line number
			for the most recent error.

	SYNTAX:		ERL

***************************************************************/

struct bwb_variable *
fnc_erl( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize nvar if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) err_line;

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_loc()

        DESCRIPTION:    This C function implements the BASIC
			LOC() function. As implemented here,
			this only works for random-acess files.

	SYNTAX:		LOC( device-number )

***************************************************************/

struct bwb_variable *
fnc_loc( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   int dev_number;

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_loc(): received f_arg <%f> ",
      var_getnval( &( argv[ 0 ] ) ) );
   bwb_debug( bwb_ebuf );
#endif

   if ( argc < 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function LOC().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }
   else if ( argc > 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Too many parameters (%d) to function LOC().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }

   dev_number = (int) var_getnval( &( argv[ 0 ] ) );

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   /* note if this is the very beginning of the file */

   if ( dev_table[ dev_number ].loc == 0 )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) 0;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) =
         (bnumber) dev_table[ dev_number ].next_record;
      }

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_eof()

        DESCRIPTION:    This C function implements the BASIC
			EOF() function.

	SYNTAX:		EOF( device-number )

***************************************************************/

struct bwb_variable *
fnc_eof( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   int dev_number;
   int cur_pos, end_pos; /* JBV */

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_loc(): received f_arg <%f> ",
      var_getnval( &( argv[ 0 ] ) ) );
   bwb_debug( bwb_ebuf );
#endif

   if ( argc < 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function EOF().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }
   else if ( argc > 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Too many parameters (%d) to function EOF().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }

   dev_number = (int) var_getnval( &( argv[ 0 ] ) );

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   /* note if this is the very beginning of the file */

   if ( dev_table[ dev_number ].mode == DEVMODE_AVAILABLE )
      {
      bwb_error( err_devnum );
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else if ( dev_table[ dev_number ].mode == DEVMODE_CLOSED )
      {
      bwb_error( err_devnum );
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }

   /*------------------------------------------------------*/
   /* feof() finds EOF when you read past the end of file. */
   /* This is not how BASIC works, at least not GWBASIC.   */
   /* The EOF function should return an EOF indication     */
   /* when you are <at> the end of the file, not past it.  */
   /* This routine was modified to reflect this.           */
   /* (JBV, 10/15/95)                                      */
   /*------------------------------------------------------*/

   /*  else if ( feof( dev_table[ dev_number ].cfp ) == 0 ) */
   else if ( ftell( dev_table[ dev_number ].cfp ) !=
   dev_table [ dev_number ].lof )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_lof()

        DESCRIPTION:    This C function implements the BASIC
			LOF() function.

	SYNTAX:		LOF( device-number )

***************************************************************/

struct bwb_variable *
fnc_lof( int argc, struct bwb_variable *argv, int unique_id )
   {
   static struct bwb_variable nvar;
   static int init = FALSE;
   int dev_number;

/* Following section no longer needed, removed by JBV */
/* #if UNIX_CMDS
   static struct stat statbuf;
   int r;
#endif */

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_lof(): received f_arg <%f> ",
      var_getnval( &( argv[ 0 ] ) ) );
   bwb_debug( bwb_ebuf );
#endif

   if ( argc < 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function LOF().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }
   else if ( argc > 1 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "Too many parameters (%d) to function LOF().",
         argc );
      bwb_error( bwb_ebuf );
#else
      bwb_error( err_syntax );
#endif
      return NULL;
      }

   dev_number = (int) var_getnval( &( argv[ 0 ] ) );

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, NUMBER );
      }

   /* stat the file */

/* Following section no longer needed, removed by JBV */
/* #if UNIX_CMDS

   r = stat( dev_table[ dev_number ].filename, &statbuf );

   if ( r != 0 )
      {
#if PROG_ERRORS
      sprintf( bwb_ebuf, "in fnc_lof(): failed to find file <%s>",
         dev_table[ dev_number ].filename );
      bwb_error( bwb_ebuf );
#else
      sprintf( bwb_ebuf, ERR_OPENFILE,
         dev_table[ dev_number ].filename );
      bwb_error( bwb_ebuf );
#endif
      return NULL;
      }

   * var_findnval( &nvar, nvar.array_pos ) = (bnumber) statbuf.st_size; */

/* #else */  /* Removed by JBV, no longer needed */

   /* * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE; */
   * var_findnval( &nvar, nvar.array_pos ) =
      (bnumber) dev_table[ dev_number ].lof; /* JBV */

/* #endif */  /* Removed by JBV, no longer needed */

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_trim()

        DESCRIPTION:    This C function implements the BASIC
			TRIM$() function, removing leading and
			trailing white space characters from a
			specified string.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		TRIM$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_trim( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   char tmp[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;
   int i;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function TRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function TRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   for( i = 0; i < strlen(tbuf) && isspace( tbuf[i] ); i++) ;
   strcpy( tmp, &tbuf[i] );
   for( i = strlen( tmp ) - 1; i > 0 && isspace( tmp[i] ); i--)
        tmp[i] = '\0';

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tmp );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_ltrim()

        DESCRIPTION:    This C function implements the BASIC
			LTRIM$() function, removing leading white
			space characters from a specified string.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		LTRIM$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_ltrim( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   char tmp[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;
   int i;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function LTRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function LTRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   for( i = 0; i < strlen(tbuf) && isspace( tbuf[i] ); i++) ;
   strcpy( tmp, &tbuf[i] );

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tmp );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_rtrim()

        DESCRIPTION:    This C function implements the BASIC
			RTRIM$() function, removing trailing
			white space characters from a specified
			string.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		RTRIM$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_rtrim( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   char tmp[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;
   int i;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function RTRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function RTRIM$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   for( i = strlen( tbuf ) - 1; i > 0 && isspace( tbuf[i] ); i--)
        tbuf[i] = '\0';

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_token()

        DESCRIPTION:    This C function implements the BASIC
                        predefined TOKEN$ function, which extracts
                        the nth token from the given string.
                        A token delimiter is optionally specified
                        in the third argument; the default is a
                        space.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		TOKEN$( string$, token-number[, token-characters$ ] )

***************************************************************/

struct bwb_variable *
fnc_token( int argc, struct bwb_variable *argv, int unique_id  )
   {
   static struct bwb_variable nvar;
   register int c;
   char target_string[ MAXSTRINGSIZE + 1 ];
   int target_counter;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static int init = FALSE;

   char token_string[ MAXSTRINGSIZE + 1 ];
   char *found_string;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check arguments */

#if PROG_ERRORS
   if ( argc < 2 )
      {
      sprintf( bwb_ebuf, "Not enough arguments to function TOKEN$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }

   if ( argc > 3 )
      {
      sprintf( bwb_ebuf, "Two many arguments to function TOKEN$()" );
      bwb_error( bwb_ebuf );
      return &nvar;
      }
#else
   if ( fnc_checkargs( argc, argv, 2, 3 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* get arguments */

   str_btoc( target_string, var_getsval( &( argv[ 0 ] ) ));
   target_counter = (int) var_getnval( &( argv[ 1 ] ) ) - 1;

   if ( argc == 3 )
      {
      str_btoc( token_string, var_getsval( &( argv[ 2 ] ) ));
      }
   else
      {
      strcpy( token_string, " " );
      }

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in fnc_token() string <%s> token-no <%d> token-str <%d>",
      target_string, target_counter, token_string );
   bwb_debug( bwb_ebuf );
#endif

   c = 1;
   found_string = strtok( target_string, token_string );
   while ( c++ <= target_counter && found_string != NULL )
       found_string = strtok( NULL, token_string );

   if ( found_string != NULL )
      strcpy( tbuf, found_string );
   else
      strcpy( tbuf, "\0" );

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   return &nvar;
   }

/***************************************************************

        FUNCTION:       fnc_pcase()

        DESCRIPTION:    This C function implements the BASIC
			PCASE$() function, converting the
			first character of every word in a
			specified string to upper case and the
			rest to lower case.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		PCASE$( variable-string )

***************************************************************/

struct bwb_variable *
fnc_pcase( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   char *p;
   int make_upper;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function PCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function PCASE$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));


   p = tbuf;
   make_upper = TRUE;
   while ( *p != '\0' )
      {
      if ( isspace( *p ) == FALSE && make_upper == TRUE )
         {
         *p = (char) toupper( *p );
         make_upper = FALSE;
         }
      else if ( isspace( *p ) )
         {
         make_upper = TRUE;
         }
      else
         {
         *p = (char) tolower( *p );
         }
      ++p;
      }

   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isalnum()

        DESCRIPTION:    This C function implements the BASIC
			ISALNUM() function, returning TRUE
			if the first character of the specified
			string is alphanumeric, otherwise
			returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISALNUM( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isalnum( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISALNUM().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISALNUM().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0') 
      {
      if ( isalnum(*p) == FALSE ) 
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isalpha()

        DESCRIPTION:    This C function implements the BASIC
			ISALPHA() function, returning TRUE
			if the first character of the specified
			string is a letter of the alphabet,
			otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISALPHA( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isalpha( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISALPHA().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISALPHA().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0') 
      {
         if ( isalpha(*p) == FALSE ) 
         {
            return_value = FALSE;
            break;
         }
         ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }


/***************************************************************

        FUNCTION:       fnc_isascii()

        DESCRIPTION:    This C function implements the BASIC
			ISASCII() function, returning TRUE
			if the first character of the specified
			string is lower ASCII (less than 128
			decimal), otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISASCII( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isascii( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISASCII().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISASCII().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0') 
      {
#ifdef isascii
         if ( isascii(*p) == FALSE )
#else
         if ( (*p & ~0177) == FALSE )       
#endif
            {
            return_value = FALSE;
            break;
            }
         ++p;
   }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isblank()

        DESCRIPTION:    This C function implements the BASIC
			ISBLANK() function, returning TRUE
			if the first character of the specified
			string is blank (space or tab), otherwise
			returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISBLANK( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isblank( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISBLANK().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISBLANK().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0') 
      {
#ifdef isblank
      if ( isblank(tbuf[0]) == FALSE)
#else
      if ( (*p == (char) 32 || *p == (char) 9) == FALSE )
#endif
         {
         return_value = FALSE;
         break;
         }
      ++p;
   }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_iscntrl()

        DESCRIPTION:    This C function implements the BASIC
			ISCNTRL() function, returning TRUE
			if the first character of the specified
			string is control (decimal 0 to 31, or
			127), otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISCNTRL( variable-string )

***************************************************************/

struct bwb_variable *
fnc_iscntrl( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISCNTRL().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISCNTRL().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0') 
      {
      if ( iscntrl(*p) == FALSE ) 
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isdigit()

        DESCRIPTION:    This C function implements the BASIC
			ISDIGIT() function, returning TRUE
			if the first character of the specified
			string is a numeric digit, otherwise
			returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISDIGIT( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isdigit( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISDIGIT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISDIGIT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isdigit(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isgraph()

        DESCRIPTION:    This C function implements the BASIC
			ISGRAPH() function, returning TRUE
			if the first character of the specified
			string is a printable character,
			excluding the space character (decimal
			32), otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISGRAPH( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isgraph( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISGRAPH().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISGRAPH().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isgraph(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_islower()

        DESCRIPTION:    This C function implements the BASIC
			ISLOWER() function, returning TRUE
			if the first character of the specified
			string is a lowercase letter (a-z),
			otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISLOWER( variable-string )

***************************************************************/

struct bwb_variable *
fnc_islower( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISLOWER().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISLOWER().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( islower(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isprint()

        DESCRIPTION:    This C function implements the BASIC
			ISPRINT() function, returning TRUE
			if the first character of the specified
			string is printable (decimal 32-126),
			otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISPRINT( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isprint( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISPRINT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISPRINT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isprint(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_ispunct()

        DESCRIPTION:    This C function implements the BASIC
			ISPUNCT() function, returning TRUE
			if the first character of the specified
			string is a punctuation character.
			Here this will mean any printable character
			that is neither a control character nor
			an alphanumeric character (decimal 32-47,
			58-63, 91-96, and 123-126).  Otherwise,
			FALSE is returned.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISPUNCT( variable-string )

***************************************************************/

struct bwb_variable *
fnc_ispunct( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISPUNCT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISPUNCT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( ispunct(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isspace()

        DESCRIPTION:    This C function implements the BASIC
			ISSPACE() function, returning TRUE
			if the first character of the specified
			string is whitespace (space, carriage
			return, horizontal or vertical tab, new
			line, or form feed), otherwise returning
			FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISSPACE( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isspace( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISSPACE().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISSPACE().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isspace(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isupper()

        DESCRIPTION:    This C function implements the BASIC
			ISUPPER() function, returning TRUE
			if the first character of the specified
			string is an uppercase letter (A-Z),
			otherwise returning FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISUPPER( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isupper( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISUPPER().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISUPPER().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isupper(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/***************************************************************

        FUNCTION:       fnc_isxdigit()

        DESCRIPTION:    This C function implements the BASIC
			ISXDIGIT() function, returning TRUE
			if the first character of the specified
			string is a valid hexadecimal digit
			(0-9, A-F, a-f), otherwise returning
			FALSE.

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		ISXDIGIT( variable-string )

***************************************************************/

struct bwb_variable *
fnc_isxdigit( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   int return_value = TRUE;
   char *p;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) NUMBER );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc < 1 )
      {
      sprintf( bwb_ebuf, "Not enough parameters (%d) to function ISXDIGIT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
   else if ( argc > 1 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function ISXDIGIT().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 1, 1 ) == FALSE )
      {
      return NULL;
      }
#endif

   /* resolve the argument and place string value in tbuf */

   str_btoc( tbuf, var_getsval( &( argv[ 0 ] )));

   p = tbuf;
   while (*p != '\0')
      {
      if ( isxdigit(*p) == FALSE )
         {
            return_value = FALSE;
            break;
         }
      ++p;
      }

   if ( return_value == TRUE )
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) TRUE;
      }
   else
      {
      * var_findnval( &nvar, nvar.array_pos ) = (bnumber) FALSE;
      }

   /* return address of nvar */

   return &nvar;

   }

/* October 15 */

/***************************************************************

        FUNCTION:       fnc_curdir()

        DESCRIPTION:    This C function implements the BASIC
			CurDir$() function, returning the current
			directory. 

			Added by Hitesh K. Soneji 10/98
			(hitesh.soneji@industry.net)

	SYNTAX:		CurDir$()

***************************************************************/

struct bwb_variable *
fnc_curdir( int argc, struct bwb_variable *argv, int unique_id )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   static struct bwb_variable nvar;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &nvar, (int) STRING );
      }

   /* check for correct number of parameters */

#if PROG_ERRORS
   if ( argc > 0 )
      {
      sprintf( bwb_ebuf, "Too many parameters (%d) to function CURDIR$().",
         argc );
      bwb_error( bwb_ebuf );
      return NULL;
      }
#else
   if ( fnc_checkargs( argc, argv, 0, 0 ) == FALSE )
      {
      return NULL;
      }
#endif

   getcwd( tbuf, sizeof(tbuf) );
   str_ctob( var_findsval( &nvar, nvar.array_pos ), tbuf );

   /* return address of nvar */
   return &nvar;
   }


#endif					/* MS_FUNCS */

/***************************************************************

        FUNCTION:       fnc_test()

        DESCRIPTION:    This is a test function, developed in
                        order to test argument passing to
                        BASIC functions.

***************************************************************/

#if INTENSIVE_DEBUG
struct bwb_variable *
fnc_test( int argc, struct bwb_variable *argv, int unique_id )
   {
   register int c;
   static struct bwb_variable rvar;
   static char *tbuf;
   static int init = FALSE;

   /* initialize the variable if necessary */

   if ( init == FALSE )
      {
      init = TRUE;
      var_make( &rvar, NUMBER );

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( tbuf = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "fnc_test" )) == NULL )
         {
#if PROG_ERRORS
         bwb_error( "in fnc_test(): failed to get memory for tbuf" );
#else
         bwb_error( err_getmem );
#endif
         }
      }

   sprintf( bwb_ebuf, "TEST function: received %d arguments: \n", argc );
   prn_xprintf( stderr, bwb_ebuf );

   for ( c = 0; c < argc; ++c )
      {
      str_btoc( tbuf, var_getsval( &argv[ c ] ) );
      sprintf( bwb_ebuf, "                  arg %d (%c): <%s> \n", c,
	 argv[ c ].type, tbuf );
      prn_xprintf( stderr, bwb_ebuf );
      }

   return &rvar;

   }
#endif

/***************************************************************

        FUNCTION:       fnc_checkargs()

        DESCRIPTION:    This C function checks the arguments to
			functions.

***************************************************************/

#if PROG_ERRORS
#else
int
fnc_checkargs( int argc, struct bwb_variable *argv, int min, int max )
   {

   if ( argc < min )
      {
      bwb_error( err_syntax );
      return FALSE;
      }
   if ( argc > max )
      {
      bwb_error( err_syntax );
      return FALSE;
      }

   return TRUE;

   }
#endif

/***************************************************************

        FUNCTION:       fnc_fncs()

        DESCRIPTION:    This C function is used for debugging
                        purposes; it prints a list of all defined
                        functions.

	SYNTAX:		FNCS

***************************************************************/

#if PERMANENT_DEBUG

struct bwb_line *
bwb_fncs( struct bwb_line *l )
   {
   struct bwb_function *f;

   for ( f = fnc_start.next; f != &fnc_end; f = f->next )
      {
      sprintf( bwb_ebuf, "%s\t%c \n", f->name, f->type );
      prn_xprintf( stderr, bwb_ebuf );
      }

   return bwb_zline( l );

   }
#endif

