#include <stdlib.h>
#include <fcntl.h>
#include <sys/select.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>

#include "input.h"
#include "cm_message.h"
#include "cm_comm.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif /* DMALLOC */

fd_set	master_readfds;
int	max_readfd = 0;
int	input_fd, evtmgr_fd, queue_fd;
int	input_timeout=10;
int (*input_handler)(struct msg_base *msg);
int (*evtmsg_handler)(struct raw_event *msg);
int (*qcmd_handler)(unsigned char cmd);

//char	msg_buffer[CM_MAX_MESSAGE_LEN];
char	*msg_buffer=NULL;

void	setup_input(int in_fd, int evt_fd, int q_fd,
                    int (*in_handler)(struct msg_base *msg),
                    int (*evt_handler)(struct raw_event *msg), 
                    int (*q_handler)(unsigned char cmd), 
                    int timeout)
{
    int max_fd;
    
	msg_buffer=(char*)malloc(CM_MAX_MESSAGE_LEN > EVENT_MAX_MESSAGE_LEN ? 
                             CM_MAX_MESSAGE_LEN : 
                             EVENT_MAX_MESSAGE_LEN );

	FD_ZERO(&master_readfds);
	FD_SET(in_fd, &master_readfds);
    FD_SET(evt_fd, &master_readfds);
    FD_SET(q_fd, &master_readfds);

    max_fd = in_fd > evt_fd ?
        (in_fd > q_fd ? in_fd : q_fd) :
        (evt_fd > q_fd ? evt_fd : q_fd);
    
	max_readfd = max_fd + 1;
	input_fd=in_fd;
    evtmgr_fd = evt_fd;
    queue_fd = q_fd;
    input_handler = in_handler;
    evtmsg_handler = evt_handler;
    qcmd_handler = q_handler;

	/* Set the timeout */
	input_timeout=timeout;
}

int get_input( void )
{
	fd_set		readfds;
	ssize_t		recvlen;
	struct msg_base	*msg;
	int		active_fds;
/*	struct timeval	select_tv;  */
    struct sockaddr_un	from;
    socklen_t fromlen;
    
	/* Copy the master fd sets into our local fd sets */
	memcpy(&readfds, &master_readfds, sizeof(fd_set));


	active_fds=select(max_readfd, &readfds, NULL, NULL, (struct timeval *)0);
	if(active_fds<0){
            syslog(LOG_WARNING,"Select failed: Error:[%d]\n",active_fds);
            return -1;
	}

	/* Check for new message */
	if( FD_ISSET(input_fd, &readfds) )
    {

        fromlen=sizeof(struct sockaddr_un);

        recvlen = comm_recvfrom(input_fd, msg_buffer, CM_MAX_MESSAGE_LEN -1, MSG_DONTWAIT,
                                (struct sockaddr *)&from, &fromlen);
        if( recvlen > 0)
        {
            msg_buffer[recvlen]=0;
            msg = msg_parse(msg_buffer, recvlen);
            (*input_handler)(msg);
        } 
        else 
        {
            /* Error */
            if(recvlen == EAGAIN)
                syslog(LOG_WARNING, "No data after select call waiting for comm message\n");
            else 
                syslog(LOG_WARNING, "unknown error in comm_recvfrom after select call\n");
        }
	}

    if (FD_ISSET(evtmgr_fd, &readfds))
    {
        
        fromlen=sizeof(struct sockaddr_un);

        recvlen = sys_events_comm_recvfrom(evtmgr_fd, msg_buffer,
                                           EVENT_MAX_MESSAGE_LEN, MSG_DONTWAIT,
                                           (struct sockaddr *)&from, &fromlen);

        if( recvlen > 0)
        {
            (*evtmsg_handler)((struct raw_event *)msg_buffer);
        }
        else
        {
            /* Error */
            if(recvlen == EAGAIN)
                syslog(LOG_WARNING, "No data after select call waiting for event message\n");
            else 
                syslog(LOG_WARNING, "unknown error in event_recvfrom after select call\n");
        }
    }
    
    if (FD_ISSET(queue_fd, &readfds))
    {
        unsigned char cmd;
        /* read the single byte command from pipe and call qcmd processor */
        
        if (read(queue_fd, &cmd, 1) > 0)
        {
            (*qcmd_handler)(cmd);
        }
    }
    return 0;
}
