/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

#include <pal.h>

static int avalanche_init_titan_variant( void );
int avalanche_get_titan_variant( void );


/*******************************************************************************
 * FUNCTION:    PAL_sysProbeAndPrep
 *******************************************************************************
 * DESCRIPTION: Function to get availablity of hardware in run-time
 *
 * RETURNS:     0: Yes, -1: No.
 *
 * NOTES: This function can be enhanced for more hardware devices in the future
 *
 ******************************************************************************/
PAL_Result PAL_sysProbeAndPrep(Uint32 module_version, Uint32 base_addr, Uint32 ptr)
{
  switch ( module_version )
  {
     case HARDWARE_USB_SLAVE:
        if( avalanche_get_titan_variant() != TITAN_TYPE_1050 )
           return -1;
        break;
     case HARDWARE_CPMAC0:
        break;
     case HARDWARE_CPMAC1:
        if( avalanche_get_titan_variant() == TITAN_TYPE_1056 )
           return -1;
        break;
     case HARDWARE_ESWITCH:
        if( avalanche_get_titan_variant() == TITAN_TYPE_1056 )
           return -1;    
        break;
     case HARDWARE_LCD:
        break;
     case HARDWARE_USB_HOST11:
        if( avalanche_get_titan_variant() != TITAN_TYPE_1050 )
           return -1;
		break;
     default:
        return -1;
  }     
  
  return 0;
}

static int avalanche_init_titan_variant( void )
{
    UINT32 new_val;

    /* set GPIO 44 - 47 as input */
    PAL_sysGpioCtrl(44, GPIO_PIN, GPIO_INPUT_PIN);
    PAL_sysGpioCtrl(45, GPIO_PIN, GPIO_INPUT_PIN);
    PAL_sysGpioCtrl(46, GPIO_PIN, GPIO_INPUT_PIN);
    PAL_sysGpioCtrl(47, GPIO_PIN, GPIO_INPUT_PIN);
    
    /* read GPIO to get Titan variant type */
    PAL_sysGpioInValue( &new_val, 1 );

    new_val >>= 12;
    new_val &= 0x0f;
    
    switch ( new_val )
    {
    case TITAN_TYPE_1050:
    case TITAN_TYPE_1055:
    case TITAN_TYPE_1056:
    case TITAN_TYPE_1060:
        return new_val;
        
    default:
        break;
    }       
    
    /* In case we get an invalid value, return the default Titan chip */
    return TITAN_TYPE_1050;
}

/* Get the Titan chip variant installed */
int avalanche_get_titan_variant( void )
{
  static int init = 0;
  static int titan_type = TITAN_TYPE_UNKNOWN;

  if ( !init )
  {
     titan_type = avalanche_init_titan_variant(); 
     init = 1;
  }

  return titan_type;
}
