//-----------------------------------------------------------------------------------------------------
// Copyright 2003, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
//       To drive the monochrome 4 bpp QVGA display.
//
//   THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
//   "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS 
//   OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
//   TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY 
//   PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR 
//   DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, 
//   COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. 
//   See The GNU General Public License for more details.
//
// These changes are covered under version 2 of the GNU General Public License, 
//  dated June 1991.
//-----------------------------------------------------------------------------------------------------

/*
 * File: fb.h
 *
 * This file is the common header file for the arm9 framebuffers.
 *
 * Copyright (C) 2001 RidgeRun, Inc.  
 * Author: Alex McMains <aam@ridgerun.com> 2001/05/10
 *         Greg Lonnon <glonnon@ridgerun.com>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs and problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#ifndef _fb_h_
#define _fb_h_

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/ctype.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/fb.h>
#include <linux/delay.h>
#include <linux/wrapper.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/page.h>
//#include "glcd_hal.h"


/*************************************
 Standard Linux Frame Buffer Interface
**************************************/
int __init tifb_setup(char *);
static int tifb_get_fix(struct fb_fix_screeninfo *fix, int con,
                        struct fb_info *info);
static int tifb_get_var(struct fb_var_screeninfo *var, int con,
                        struct fb_info *info);
static int tifb_set_var(struct fb_var_screeninfo *var, int con,
                        struct fb_info *info);
static int tifb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
                         struct fb_info *info);
static int tifb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
                         struct fb_info *info);
static int tifb_pan_display(struct fb_var_screeninfo *var, int con,
                            struct fb_info *info);
static int __init tifb_init(void);
static int tifbcon_switch(int con, struct fb_info *info);
static void tifbcon_blank(int blank, struct fb_info *info);
static void __exit tifb_cleanup(void);


/*****************
 Internal Routines
 *****************/
static void tifb_encode_fix(struct fb_fix_screeninfo *fix,
                            struct fb_var_screeninfo *var);
static int tifb_getcolreg(u_int regno, u_int *red, u_int *green, u_int *blue,
                          u_int *transp, struct fb_info *info);
static int tifb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
                          u_int transp, struct fb_info *info);
static void tifb_init_palette(void);
static int tifb_map_video_memory(void);


#endif /* _fb_h_ */

/*
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-file-style: "linux"
 * End:
 */

