/******************************************************************************   
 * FILE PURPOSE:     - LED user space Header                                           
 ******************************************************************************   
 * FILE NAME:     led_user.h                                                     
 *                                                                                
 * DESCRIPTION:   Header file defining macros for ioctl commands and user types.                 
 *                                                                                
 *                                                                                
 * (C) Copyright 2004, Texas Instruments, Inc                                     
 *******************************************************************************/ 
#ifndef __LED_USER__
#define __LED_USER__

#include <linux/ioctl.h>

#define LED_MAX_BITS			2   /* Max bits (GPIO pins) for a LED (max=2)*/
#define LED_MAX_COLORS			( ( 1 << LED_MAX_BITS ) - 1 )

/* Return values:
-EINVAL: Invalid argument (Bad handle/GPIO)
-ENOMEM: Memory allocation failure
-EBUSY:  GPIO Pin already assigned */

/****** General types *******/
typedef enum
{
  LED_MODE_OFF,
  LED_MODE_ON,
  LED_MODE_ONE_SHOT_OFF,
  LED_MODE_ONE_SHOT_ON,
  LED_MODE_BLINK,

} LED_MODE_T;

struct led_action
{
  int          led_handle;  /* LED handle */

  LED_MODE_T   mode;        /* Requested mode: On/Off/Blink */
  unsigned int param1;	    /* Parameter 1: Delay (ms) [Blink/OneOn/OneOff] */
  unsigned int param2;	    /* Parameter 2: Delay (ms) [Blink] */

  /* Only for colored LEDs. Use 0 for non colored LEDs */
  unsigned int color_index; /* Color index 0:LED_MAX_COLORS */
};

struct led_gpio_object
{
  int led_handle;                         /* IOCTL returns a handle to the LED */  
  unsigned int num_of_gpios;              /* How many GPIO pins used by the LED (def=1)*/
  unsigned int led_gpios[ LED_MAX_BITS ]; /* GPIO numbers */
};

struct input_gpio_object
{
  int input_handle;         /* IOCTL returns a handle to the LED */
  unsigned int input_gpio;  /* GPIO numbers */
  int invert;               /* 1 : state will be inverted */
};

/******* IOCTL list *******/

#define LED_DRIVER_MAGIC			0xD0

#define LED_CMD_GET_HANDLE              _IOWR(LED_DRIVER_MAGIC, 1, struct led_gpio_object)
#define LED_CMD_ACTION                  _IOW(LED_DRIVER_MAGIC, 2, struct led_action)
#define LED_CMD_RELEASE_HANDLE          _IO(LED_DRIVER_MAGIC, 3)
#define LED_CMD_GET_INPUT_HANDLE		    _IOWR(LED_DRIVER_MAGIC, 4, struct input_gpio_object)
#define LED_CMD_RELEASE_INPUT_HANDLE    _IO(LED_DRIVER_MAGIC, 5)
#define LED_CMD_CONTROL                 _IO(LED_DRIVER_MAGIC, 6)

#endif /*  __LED_USER__ */ 
