/******************************************************************************    
 * FILE PURPOSE:     - LED manager Header                                       
 ******************************************************************************    
 * FILE NAME:     led_manager.h                                                        
 *                                                                                 
 * DESCRIPTION:  Header file for LED manager                                        
 *                                                                                 
 * (C) Copyright 2005, Texas Instruments, Inc                                      
 *******************************************************************************/  
                                                                                   
#ifndef __LED_MANAGER__
#define __LED_MANAGER__

#include <asm/avalanche/generic/led_drv_user.h>
#include <asm/avalanche/generic/led_hal_drv.h>
#include <asm/avalanche/generic/ledcfg.h>

/* Functionality for user space applications */

struct led_manager_user_module		                 /* To be used when registering */
{
	unsigned char *name;											 /* Module name */
	unsigned int instance;	    								 /* Module inctance */
	unsigned int handle;			/* Handle: To be received by the register IOCTL */
};

struct led_manager_user_action			    /* To be used when triggering an action */
{
	unsigned int handle;						  /* Handle from the register IOCTL */
	unsigned int state_id;									     /* Requested state */
};

#define LED_MANAGER_MAGIC			0xD1

#define LED_MANAGER_CONFIG		_IOW(LED_MANAGER_MAGIC, 1, struct led_config)
#define LED_MANAGER_REGISTER	_IOWR(LED_MANAGER_MAGIC, 2, struct led_manager_user_module)
#define LED_MANAGER_UNREGISTER	_IOW(LED_MANAGER_MAGIC, 3, void *)
#define LED_MANAGER_ACTION		_IOW(LED_MANAGER_MAGIC, 4, struct led_manager_user_action)

/* Backward compatibility with old LED driver */
typedef struct led_manager_user_module LED_MODULE_T;
typedef struct led_manager_user_action LED_STATE_T;

#define LED_CONFIG				LED_MANAGER_CONFIG 		
#define LED_GET_HANDLE			LED_MANAGER_REGISTER 	
#define LED_ACTION				LED_MANAGER_ACTION 		
#define LED_RELEASE_HANDLE		LED_MANAGER_UNREGISTER

#define avalanche_led_register		led_manager_register_module
#define avalanche_led_action		led_manager_led_action
#define avalanche_led_unregister	led_manager_unregister_module

/* Functions for Kernel space modules */

/************************************************************************
 *  Function: led_manager_register_module 
 *
 *  Purpose:  Registers a module in the LED manager database
 *                                                                     
 *  Inputs: module_name - A string with the module name
 *		    instance - Instance number
 *
 *  Outputs: Handle to entity on success, or NULL if failed.
 *                                                                     
 ************************************************************************/
void *led_manager_register_module( const char *module_name, int instance );

/************************************************************************
 *  Function: led_manager_unregister_module 
 *
 *  Purpose:  Unregisters a module from the LED manager database
 *                                                                     
 *  Inputs: module_handle - Handle returned by the register function
 *
 *  Outputs: 0 on success.
 *           -EINVAL if module not found  
 *                                                                     
 ************************************************************************/
int led_manager_unregister_module( const void *module_handle );

/************************************************************************
 *  Function: led_manager_install_callbacks 
 *
 *  Purpose:  Install LED callback functions (required for Non-GPIO LEDs)
 *                                                                     
 *  Inputs: module_handle - Handle returned by the register function
 *          led_id - ID of the LED (0,1...)
 *          led_funcs - Structure with all callback functions
 *
 *  Outputs: 0 on success.
 *           -EINVAL on failure  
 *                                                                     
 ************************************************************************/
int led_manager_install_callbacks( void *module_handle, int led_id, struct led_funcs *funcs );

/************************************************************************
 *  Function: led_manager_led_action 
 *
 *  Purpose:  Triggers an action on an LED
 *                                                                     
 *  Inputs: module_handle - Handle returned by the register function
 *          state_id - ID of the state (from configuration file)
 *
 *  Outputs: 0 on success.
 *           -EINVAL on failure  
 *                                                                     
 ************************************************************************/
int led_manager_led_action( void *module_handle, int state_id );

#endif /* __LED_MANAGER__ */
