/*
 * 	NET3	Protocol independent device support routines.
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 *	Derived from the non IP parts of dev.c 1.0.19
 * 		Authors:	Ross Biro, <bir7@leland.Stanford.Edu>
 *				Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *				Mark Evans, <evansmp@uhura.aston.ac.uk>
 *
 *	Additional Authors:
 *		Florian la Roche <rzsfl@rz.uni-sb.de>
 *		Alan Cox <gw4pts@gw4pts.ampr.org>
 *		David Hinds <dhinds@allegro.stanford.edu>
 *		Alexey Kuznetsov <kuznet@ms2.inr.ac.ru>
 *		Adam Sulmicki <adam@cfar.umd.edu>
 *              Pekka Riikonen <priikone@poesidon.pspt.fi>
 *
 *	Changes:
 *              D.J. Barrow     :       Fixed bug where dev->refcnt gets set to 2
 *                                      if register_netdev gets called before
 *                                      net_dev_init & also removed a few lines
 *                                      of code in the process.
 *		Alan Cox	:	device private ioctl copies fields back.
 *		Alan Cox	:	Transmit queue code does relevant stunts to
 *					keep the queue safe.
 *		Alan Cox	:	Fixed double lock.
 *		Alan Cox	:	Fixed promisc NULL pointer trap
 *		????????	:	Support the full private ioctl range
 *		Alan Cox	:	Moved ioctl permission check into drivers
 *		Tim Kordas	:	SIOCADDMULTI/SIOCDELMULTI
 *		Alan Cox	:	100 backlog just doesn't cut it when
 *					you start doing multicast video 8)
 *		Alan Cox	:	Rewrote net_bh and list manager.
 *		Alan Cox	: 	Fix ETH_P_ALL echoback lengths.
 *		Alan Cox	:	Took out transmit every packet pass
 *					Saved a few bytes in the ioctl handler
 *		Alan Cox	:	Network driver sets packet type before calling netif_rx. Saves
 *					a function call a packet.
 *		Alan Cox	:	Hashed net_bh()
 *		Richard Kooijman:	Timestamp fixes.
 *		Alan Cox	:	Wrong field in SIOCGIFDSTADDR
 *		Alan Cox	:	Device lock protection.
 *		Alan Cox	: 	Fixed nasty side effect of device close changes.
 *		Rudi Cilibrasi	:	Pass the right thing to set_mac_address()
 *		Dave Miller	:	32bit quantity for the device lock to make it work out
 *					on a Sparc.
 *		Bjorn Ekwall	:	Added KERNELD hack.
 *		Alan Cox	:	Cleaned up the backlog initialise.
 *		Craig Metz	:	SIOCGIFCONF fix if space for under
 *					1 device.
 *	    Thomas Bogendoerfer :	Return ENODEV for dev_open, if there
 *					is no device open function.
 *		Andi Kleen	:	Fix error reporting for SIOCGIFCONF
 *	    Michael Chastain	:	Fix signed/unsigned for SIOCGIFCONF
 *		Cyrus Durgin	:	Cleaned for KMOD
 *		Adam Sulmicki   :	Bug Fix : Network Device Unload
 *					A network device unload needs to purge
 *					the backlog queue.
 *	Paul Rusty Russell	:	SIOCSIFNAME
 *              Pekka Riikonen  :	Netdev boot-time settings code
 *              Andrew Morton   :       Make unregister_netdevice wait indefinitely on dev->refcnt
 * 		J Hadi Salim	:	- Backlog queue sampling
 *				        - netif_rx() feedback	
 */

//-------------------------------------------------------------------------------------
// Copyright 2004, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
// 
// 1. Performance optimizations.
// 2. Voice-Data coexistence.
// 3. Instrumentation.
//
// THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
// "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS 
// OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY 
// PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR 
// DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS, 
// COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. 
// See The GNU General Public License for more details.
//
// These changes are covered under version 2 of the GNU General Public License, 
// dated June 1991.
//-------------------------------------------------------------------------------------

#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/if_ether.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/notifier.h>
#include <linux/skbuff.h>
#include <linux/brlock.h>
#include <net/sock.h>
#include <linux/rtnetlink.h>
#include <linux/proc_fs.h>
#include <linux/stat.h>
#include <linux/if_bridge.h>
#include <linux/divert.h>
#include <net/dst.h>
#include <net/pkt_sched.h>
#include <net/profile.h>
#include <net/checksum.h>
#include <linux/highmem.h>
#include <linux/init.h>
#include <linux/kmod.h>
#include <linux/module.h>
#if defined(CONFIG_NET_RADIO) || defined(CONFIG_NET_PCMCIA_RADIO)
#include <linux/wireless.h>		/* Note : will define WIRELESS_EXT */
#include <net/iw_handler.h>
#endif	/* CONFIG_NET_RADIO || CONFIG_NET_PCMCIA_RADIO */

#include <linux/trace.h>

#include <linux/if_vlan.h>
#include <linux/ip.h>
#include <linux/udp.h>

#include <linux/ti_qos.h>
#include <net/dsfield.h>

#include <linux/psp_trace.h>

/* PANKAJ: Added the variable which decides the mode in which the voice is
 * running. */
int gIsVoiceEnabled = 0;

static int ti_nqm_netif_rx(struct sk_buff *skb);
static void ti_nqm_net_rx_action(struct softirq_action *h);

#ifdef CONFIG_PLIP
extern int plip_init(void);
#endif

/* This define, if set, will randomly drop a packet when congestion
 * is more than moderate.  It helps fairness in the multi-interface
 * case when one of them is a hog, but it kills performance for the
 * single interface case so it is off now by default.
 */
#undef RAND_LIE

/* Setting this will sample the queue lengths and thus congestion
 * via a timer instead of as each packet is received.
 */
#undef OFFLINE_SAMPLE

NET_PROFILE_DEFINE(dev_queue_xmit)
NET_PROFILE_DEFINE(softnet_process)

const char *if_port_text[] = {
  "unknown",
  "BNC",
  "10baseT",
  "AUI",
  "100baseT",
  "100baseTX",
  "100baseFX"
};

/*
 *	The list of packet types we will receive (as opposed to discard)
 *	and the routines to invoke.
 *
 *	Why 16. Because with 16 the only overlap we get on a hash of the
 *	low nibble of the protocol value is RARP/SNAP/X.25.
 *
 *      NOTE:  That is no longer true with the addition of VLAN tags.  Not
 *             sure which should go first, but I bet it won't make much
 *             difference if we are running VLANs.  The good news is that
 *             this protocol won't be in the list unless compiled in, so
 *             the average user (w/out VLANs) will not be adversly affected.
 *             --BLG
 *
 *		0800	IP
 *		8100    802.1Q VLAN
 *		0001	802.3
 *		0002	AX.25
 *		0004	802.2
 *		8035	RARP
 *		0005	SNAP
 *		0805	X.25
 *		0806	ARP
 *		8137	IPX
 *		0009	Localtalk
 *		86DD	IPv6
 */

static struct packet_type *ptype_base[16];		/* 16 way hashed list */
static struct packet_type *ptype_all = NULL;		/* Taps */

#ifdef OFFLINE_SAMPLE
static void sample_queue(unsigned long dummy);
static struct timer_list samp_timer = { function: sample_queue };
#endif

#ifdef CONFIG_HOTPLUG
static int net_run_sbin_hotplug(struct net_device *dev, char *action);
#else
#define net_run_sbin_hotplug(dev, action) ({ 0; })
#endif

/*
 *	Our notifier list
 */
 
static struct notifier_block *netdev_chain=NULL;

/*
 *	Device drivers call our routines to queue packets here. We empty the
 *	queue in the local softnet handler.
 */
struct softnet_data softnet_data[NR_CPUS] __cacheline_aligned;

#ifdef CONFIG_NET_FASTROUTE
int netdev_fastroute;
int netdev_fastroute_obstacles;
#endif

/* Define this MACRO only for debugging the amount of time each function takes. */
#undef NSP_CORE_MEASUREMENTS

/* NSP Optimizations: This macro controls measuring the number of ticks each function
 * in the IP fast path takes. Only for debugging purposes. */
#ifdef NSP_CORE_MEASUREMENTS

/* Display the warning message so that the user knows that measurements are
 * being compiled into the code. */
#warning "*********************************************************************"
#warning "Net Core Measurements are being compiled into the code"
#warning "*********************************************************************"

/* Measurement: Macro */
#define rdtscl(dest) \
	__asm__ __volatile__("mfc0 %0,$9; nop" : "=r" (dest))

extern int netif_rx_start;
extern int netif_rx_end;
extern int dev_queue_xmit_start;
extern int dev_queue_xmit_end;

#endif  /* NSP_CORE_MEASUREMENTS */

/* TI: Fast Path Mode Static Declarations. */
static void net_set_priority(int high_low);
static int dev_qos_xmit (struct sk_buff *skb, struct net_device* dev);

/**************************************************************************
 ************************* Slow Path: Error Routines **********************
 **************************************************************************/

/**************************************************************************
 * FUNCTION NAME : net_error_handler
 **************************************************************************
 * DESCRIPTION   :
 *  This function is called to clean packet memory on an error condition
 *  while processing a received packet. This was moved to a seperate function
 *  as this is the slow path. 
 ***************************************************************************/
static void net_error_handler(struct sk_buff* skb)
{
    kfree_skb(skb);
    return;
}

/**************************************************************************
 * FUNCTION NAME : net_rx_drop_packet
 **************************************************************************
 * DESCRIPTION   :
 *  The function is called to drop the received packet. This function gets
 *  called only when voice is enabled and on the onset of congestion low 
 *  priority data packets need to be dropped.
 ***************************************************************************/
static void net_rx_drop_packet (struct sk_buff *skb)
{
    /* Release the receive device on which the packet was received. */
    dev_put(skb->dev);

    /* Clean packet memory. */
    kfree_skb(skb);
    return;
}

/**************************************************************************
 * FUNCTION NAME : net_receive_queue_overflow
 **************************************************************************
 * DESCRIPTION   :
 *  This is the error routine called when a packet is enqueued on the NET
 *  layer but the queue is full. In that case, error stats are incremented 
 *  and the packet is dropped.
 *
 * RETURNS       :
 *  NET_RX_DROP - Indicating that the packet was dropped.
 *
 * NOTES         :
 *  This function was called with interrupts disabled. We need to reenable
 *  interrupts here before we return from the function.
 ***************************************************************************/
static int net_receive_queue_overflow (int this_cpu, struct sk_buff* skb, int flags)
{
    /* Queue is being throttled. Drop the packets. The network layer is overflowing.*/
    netdev_rx_stat[this_cpu].dropped++;
    local_irq_restore(flags);

    /* Clean packet memory. */
    net_error_handler(skb);
    return NET_RX_DROP;

}

/******************************************************************************************

		Protocol management and registration routines

*******************************************************************************************/

/*
 *	For efficiency
 */

int netdev_nit=0;

/*
 *	Add a protocol ID to the list. Now that the input handler is
 *	smarter we can dispense with all the messy stuff that used to be
 *	here.
 *
 *	BEWARE!!! Protocol handlers, mangling input packets,
 *	MUST BE last in hash buckets and checking protocol handlers
 *	MUST start from promiscous ptype_all chain in net_bh.
 *	It is true now, do not change it.
 *	Explantion follows: if protocol handler, mangling packet, will
 *	be the first on list, it is not able to sense, that packet
 *	is cloned and should be copied-on-write, so that it will
 *	change it and subsequent readers will get broken packet.
 *							--ANK (980803)
 */

/**
 *	dev_add_pack - add packet handler
 *	@pt: packet type declaration
 * 
 *	Add a protocol handler to the networking stack. The passed &packet_type
 *	is linked into kernel lists and may not be freed until it has been
 *	removed from the kernel lists.
 */
 
void dev_add_pack(struct packet_type *pt)
{
	int hash;

	br_write_lock_bh(BR_NETPROTO_LOCK);

#ifdef CONFIG_NET_FASTROUTE
	/* Hack to detect packet socket */
	if (pt->data) {
		netdev_fastroute_obstacles++;
		dev_clear_fastroute(pt->dev);
	}
#endif
	if (pt->type == htons(ETH_P_ALL)) {
		netdev_nit++;
		pt->next=ptype_all;
		ptype_all=pt;
	} else {
		hash=ntohs(pt->type)&15;
		pt->next = ptype_base[hash];
		ptype_base[hash] = pt;
	}
	br_write_unlock_bh(BR_NETPROTO_LOCK);
}


/**
 *	dev_remove_pack	 - remove packet handler
 *	@pt: packet type declaration
 * 
 *	Remove a protocol handler that was previously added to the kernel
 *	protocol handlers by dev_add_pack(). The passed &packet_type is removed
 *	from the kernel lists and can be freed or reused once this function
 *	returns.
 */
 
void dev_remove_pack(struct packet_type *pt)
{
	struct packet_type **pt1;

	br_write_lock_bh(BR_NETPROTO_LOCK);

	if (pt->type == htons(ETH_P_ALL)) {
		netdev_nit--;
		pt1=&ptype_all;
	} else {
		pt1=&ptype_base[ntohs(pt->type)&15];
	}

	for (; (*pt1) != NULL; pt1 = &((*pt1)->next)) {
		if (pt == (*pt1)) {
			*pt1 = pt->next;
#ifdef CONFIG_NET_FASTROUTE
			if (pt->data)
				netdev_fastroute_obstacles--;
#endif
			br_write_unlock_bh(BR_NETPROTO_LOCK);
			return;
		}
	}
	br_write_unlock_bh(BR_NETPROTO_LOCK);
	printk(KERN_WARNING "dev_remove_pack: %p not found.\n", pt);
}

/******************************************************************************

		      Device Boot-time Settings Routines

*******************************************************************************/

/* Boot time configuration table */
static struct netdev_boot_setup dev_boot_setup[NETDEV_BOOT_SETUP_MAX];

/**
 *	netdev_boot_setup_add	- add new setup entry
 *	@name: name of the device
 *	@map: configured settings for the device
 *
 *	Adds new setup entry to the dev_boot_setup list.  The function
 *	returns 0 on error and 1 on success.  This is a generic routine to
 *	all netdevices.
 */
int netdev_boot_setup_add(char *name, struct ifmap *map)
{
	struct netdev_boot_setup *s;
	int i;

	s = dev_boot_setup;
	for (i = 0; i < NETDEV_BOOT_SETUP_MAX; i++) {
		if (s[i].name[0] == '\0' || s[i].name[0] == ' ') {
			memset(s[i].name, 0, sizeof(s[i].name));
			strcpy(s[i].name, name);
			memcpy(&s[i].map, map, sizeof(s[i].map));
			break;
		}
	}

	if (i >= NETDEV_BOOT_SETUP_MAX)
		return 0;

	return 1;
}

/**
 *	netdev_boot_setup_check	- check boot time settings
 *	@dev: the netdevice
 *
 * 	Check boot time settings for the device.
 *	The found settings are set for the device to be used
 *	later in the device probing.
 *	Returns 0 if no settings found, 1 if they are.
 */
int netdev_boot_setup_check(struct net_device *dev)
{
	struct netdev_boot_setup *s;
	int i;

	s = dev_boot_setup;
	for (i = 0; i < NETDEV_BOOT_SETUP_MAX; i++) {
		if (s[i].name[0] != '\0' && s[i].name[0] != ' ' &&
		    !strncmp(dev->name, s[i].name, strlen(s[i].name))) {
			dev->irq 	= s[i].map.irq;
			dev->base_addr 	= s[i].map.base_addr;
			dev->mem_start 	= s[i].map.mem_start;
			dev->mem_end 	= s[i].map.mem_end;
			return 1;
		}
	}
	return 0;
}

/*
 * Saves at boot time configured settings for any netdevice.
 */
int __init netdev_boot_setup(char *str)
{
	int ints[5];
	struct ifmap map;

	str = get_options(str, ARRAY_SIZE(ints), ints);
	if (!str || !*str)
		return 0;

	/* Save settings */
	memset(&map, 0, sizeof(map));
	if (ints[0] > 0)
		map.irq = ints[1];
	if (ints[0] > 1)
		map.base_addr = ints[2];
	if (ints[0] > 2)
		map.mem_start = ints[3];
	if (ints[0] > 3)
		map.mem_end = ints[4];

	/* Add new entry to the list */	
	return netdev_boot_setup_add(str, &map);
}

__setup("netdev=", netdev_boot_setup);

/*****************************************************************************************

			    Device Interface Subroutines

******************************************************************************************/

/**
 *	__dev_get_by_name	- find a device by its name 
 *	@name: name to find
 *
 *	Find an interface by name. Must be called under RTNL semaphore
 *	or @dev_base_lock. If the name is found a pointer to the device
 *	is returned. If the name is not found then %NULL is returned. The
 *	reference counters are not incremented so the caller must be
 *	careful with locks.
 */
 

struct net_device *__dev_get_by_name(const char *name)
{
	struct net_device *dev;

	for (dev = dev_base; dev != NULL; dev = dev->next) {
		if (strncmp(dev->name, name, IFNAMSIZ) == 0)
			return dev;
	}
	return NULL;
}

/**
 *	dev_get_by_name		- find a device by its name
 *	@name: name to find
 *
 *	Find an interface by name. This can be called from any 
 *	context and does its own locking. The returned handle has
 *	the usage count incremented and the caller must use dev_put() to
 *	release it when it is no longer needed. %NULL is returned if no
 *	matching device is found.
 */

struct net_device *dev_get_by_name(const char *name)
{
	struct net_device *dev;

	read_lock(&dev_base_lock);
	dev = __dev_get_by_name(name);
	if (dev)
		dev_hold(dev);
	read_unlock(&dev_base_lock);
	return dev;
}

/* 
   Return value is changed to int to prevent illegal usage in future.
   It is still legal to use to check for device existance.

   User should understand, that the result returned by this function
   is meaningless, if it was not issued under rtnl semaphore.
 */

/**
 *	dev_get	-	test if a device exists
 *	@name:	name to test for
 *
 *	Test if a name exists. Returns true if the name is found. In order
 *	to be sure the name is not allocated or removed during the test the
 *	caller must hold the rtnl semaphore.
 *
 *	This function primarily exists for back compatibility with older
 *	drivers. 
 */
 
int dev_get(const char *name)
{
	struct net_device *dev;

	read_lock(&dev_base_lock);
	dev = __dev_get_by_name(name);
	read_unlock(&dev_base_lock);
	return dev != NULL;
}

/**
 *	__dev_get_by_index - find a device by its ifindex
 *	@ifindex: index of device
 *
 *	Search for an interface by index. Returns %NULL if the device
 *	is not found or a pointer to the device. The device has not
 *	had its reference counter increased so the caller must be careful
 *	about locking. The caller must hold either the RTNL semaphore
 *	or @dev_base_lock.
 */

struct net_device * __dev_get_by_index(int ifindex)
{
	struct net_device *dev;

	for (dev = dev_base; dev != NULL; dev = dev->next) {
		if (dev->ifindex == ifindex)
			return dev;
	}
	return NULL;
}


/**
 *	dev_get_by_index - find a device by its ifindex
 *	@ifindex: index of device
 *
 *	Search for an interface by index. Returns NULL if the device
 *	is not found or a pointer to the device. The device returned has 
 *	had a reference added and the pointer is safe until the user calls
 *	dev_put to indicate they have finished with it.
 */

struct net_device * dev_get_by_index(int ifindex)
{
	struct net_device *dev;

	read_lock(&dev_base_lock);
	dev = __dev_get_by_index(ifindex);
	if (dev)
		dev_hold(dev);
	read_unlock(&dev_base_lock);
	return dev;
}

/**
 *	dev_getbyhwaddr - find a device by its hardware address
 *	@type: media type of device
 *	@ha: hardware address
 *
 *	Search for an interface by MAC address. Returns NULL if the device
 *	is not found or a pointer to the device. The caller must hold the
 *	rtnl semaphore. The returned device has not had its ref count increased
 *	and the caller must therefore be careful about locking
 *
 *	BUGS:
 *	If the API was consistent this would be __dev_get_by_hwaddr
 */

struct net_device *dev_getbyhwaddr(unsigned short type, char *ha)
{
	struct net_device *dev;

	ASSERT_RTNL();

	for (dev = dev_base; dev != NULL; dev = dev->next) {
		if (dev->type == type &&
		    memcmp(dev->dev_addr, ha, dev->addr_len) == 0)
			return dev;
	}
	return NULL;
}

/**
 *	dev_alloc_name - allocate a name for a device
 *	@dev: device 
 *	@name: name format string
 *
 *	Passed a format string - eg "lt%d" it will try and find a suitable
 *	id. Not efficient for many devices, not called a lot. The caller
 *	must hold the dev_base or rtnl lock while allocating the name and
 *	adding the device in order to avoid duplicates. Returns the number
 *	of the unit assigned or a negative errno code.
 */

int dev_alloc_name(struct net_device *dev, const char *name)
{
	int i;
	char buf[32];
	char *p;

	/*
	 * Verify the string as this thing may have come from
	 * the user.  There must be either one "%d" and no other "%"
	 * characters, or no "%" characters at all.
	 */
	p = strchr(name, '%');
	if (p && (p[1] != 'd' || strchr(p+2, '%')))
		return -EINVAL;

	/*
	 * If you need over 100 please also fix the algorithm...
	 */
	for (i = 0; i < 100; i++) {
		snprintf(buf,sizeof(buf),name,i);
		if (__dev_get_by_name(buf) == NULL) {
			strcpy(dev->name, buf);
			return i;
		}
	}
	return -ENFILE;	/* Over 100 of the things .. bail out! */
}

/**
 *	dev_alloc - allocate a network device and name
 *	@name: name format string
 *	@err: error return pointer
 *
 *	Passed a format string, eg. "lt%d", it will allocate a network device
 *	and space for the name. %NULL is returned if no memory is available.
 *	If the allocation succeeds then the name is assigned and the 
 *	device pointer returned. %NULL is returned if the name allocation
 *	failed. The cause of an error is returned as a negative errno code
 *	in the variable @err points to.
 *
 *	The caller must hold the @dev_base or RTNL locks when doing this in
 *	order to avoid duplicate name allocations.
 */

struct net_device *dev_alloc(const char *name, int *err)
{
	struct net_device *dev=kmalloc(sizeof(struct net_device), GFP_KERNEL);
	if (dev == NULL) {
		*err = -ENOBUFS;
		return NULL;
	}
	memset(dev, 0, sizeof(struct net_device));
	*err = dev_alloc_name(dev, name);
	if (*err < 0) {
		kfree(dev);
		return NULL;
	}
	return dev;
}

/**
 *	netdev_state_change - device changes state
 *	@dev: device to cause notification
 *
 *	Called to indicate a device has changed state. This function calls
 *	the notifier chains for netdev_chain and sends a NEWLINK message
 *	to the routing socket.
 */
 
void netdev_state_change(struct net_device *dev)
{
	if (dev->flags&IFF_UP) {
		notifier_call_chain(&netdev_chain, NETDEV_CHANGE, dev);
		rtmsg_ifinfo(RTM_NEWLINK, dev, 0);
	}
}


#ifdef CONFIG_KMOD

/**
 *	dev_load 	- load a network module
 *	@name: name of interface
 *
 *	If a network interface is not present and the process has suitable
 *	privileges this function loads the module. If module loading is not
 *	available in this kernel then it becomes a nop.
 */

void dev_load(const char *name)
{
	if (!dev_get(name) && capable(CAP_SYS_MODULE))
		request_module(name);
}

#else

extern inline void dev_load(const char *unused){;}

#endif

static int default_rebuild_header(struct sk_buff *skb)
{
	printk(KERN_DEBUG "%s: default_rebuild_header called -- BUG!\n", skb->dev ? skb->dev->name : "NULL!!!");
	kfree_skb(skb);
	return 1;
}

/**
 *	dev_open	- prepare an interface for use. 
 *	@dev:	device to open
 *
 *	Takes a device from down to up state. The device's private open
 *	function is invoked and then the multicast lists are loaded. Finally
 *	the device is moved into the up state and a %NETDEV_UP message is
 *	sent to the netdev notifier chain.
 *
 *	Calling this function on an active interface is a nop. On a failure
 *	a negative errno code is returned.
 */
 
int dev_open(struct net_device *dev)
{
	int ret = 0;

	/*
	 *	Is it already up?
	 */

	if (dev->flags&IFF_UP)
		return 0;

	/*
	 *	Is it even present?
	 */
	if (!netif_device_present(dev))
		return -ENODEV;

	/*
	 *	Call device private open method
	 */
	if (try_inc_mod_count(dev->owner)) {
		if (dev->open) {
			ret = dev->open(dev);
			if (ret != 0 && dev->owner)
				__MOD_DEC_USE_COUNT(dev->owner);
		}
	} else {
		ret = -ENODEV;
	}

	/*
	 *	If it went open OK then:
	 */
	 
	if (ret == 0) 
	{
		/*
		 *	Set the flags.
		 */
		dev->flags |= IFF_UP;

		set_bit(__LINK_STATE_START, &dev->state);

		/*
		 *	Initialize multicasting status 
		 */
		dev_mc_upload(dev);

		/*
		 *	Wakeup transmit queue engine
		 */
		dev_activate(dev);

		/*
		 *	... and announce new interface.
		 */
		notifier_call_chain(&netdev_chain, NETDEV_UP, dev);
	}
	return(ret);
}

#ifdef CONFIG_NET_FASTROUTE

static void dev_do_clear_fastroute(struct net_device *dev)
{
	if (dev->accept_fastpath) {
		int i;

		for (i=0; i<=NETDEV_FASTROUTE_HMASK; i++) {
			struct dst_entry *dst;

			write_lock_irq(&dev->fastpath_lock);
			dst = dev->fastpath[i];
			dev->fastpath[i] = NULL;
			write_unlock_irq(&dev->fastpath_lock);

			dst_release(dst);
		}
	}
}

void dev_clear_fastroute(struct net_device *dev)
{
	if (dev) {
		dev_do_clear_fastroute(dev);
	} else {
		read_lock(&dev_base_lock);
		for (dev = dev_base; dev; dev = dev->next)
			dev_do_clear_fastroute(dev);
		read_unlock(&dev_base_lock);
	}
}
#endif

/**
 *	dev_close - shutdown an interface.
 *	@dev: device to shutdown
 *
 *	This function moves an active device into down state. A 
 *	%NETDEV_GOING_DOWN is sent to the netdev notifier chain. The device
 *	is then deactivated and finally a %NETDEV_DOWN is sent to the notifier
 *	chain.
 */
 
int dev_close(struct net_device *dev)
{
	if (!(dev->flags&IFF_UP))
		return 0;

	/*
	 *	Tell people we are going down, so that they can
	 *	prepare to death, when device is still operating.
	 */
	notifier_call_chain(&netdev_chain, NETDEV_GOING_DOWN, dev);

	dev_deactivate(dev);

	clear_bit(__LINK_STATE_START, &dev->state);

	/*
	 *	Call the device specific close. This cannot fail.
	 *	Only if device is UP
	 *
	 *	We allow it to be called even after a DETACH hot-plug
	 *	event.
	 */
	 
	if (dev->stop)
		dev->stop(dev);

	/*
	 *	Device is now down.
	 */

	dev->flags &= ~IFF_UP;
#ifdef CONFIG_NET_FASTROUTE
	dev_clear_fastroute(dev);
#endif

	/*
	 *	Tell people we are down
	 */
	notifier_call_chain(&netdev_chain, NETDEV_DOWN, dev);

	/*
	 * Drop the module refcount
	 */
	if (dev->owner)
		__MOD_DEC_USE_COUNT(dev->owner);

	return(0);
}


/*
 *	Device change register/unregister. These are not inline or static
 *	as we export them to the world.
 */
 
/**
 *	register_netdevice_notifier - register a network notifier block
 *	@nb: notifier
 *
 *	Register a notifier to be called when network device events occur.
 *	The notifier passed is linked into the kernel structures and must
 *	not be reused until it has been unregistered. A negative errno code
 *	is returned on a failure.
 */

int register_netdevice_notifier(struct notifier_block *nb)
{
	return notifier_chain_register(&netdev_chain, nb);
}

/**
 *	unregister_netdevice_notifier - unregister a network notifier block
 *	@nb: notifier
 *
 *	Unregister a notifier previously registered by
 *	register_netdevice_notifier(). The notifier is unlinked into the
 *	kernel structures and may then be reused. A negative errno code
 *	is returned on a failure.
 */

int unregister_netdevice_notifier(struct notifier_block *nb)
{
	return notifier_chain_unregister(&netdev_chain,nb);
}

/*
 *	Support routine. Sends outgoing frames to any network
 *	taps currently in use.
 */

void dev_queue_xmit_nit(struct sk_buff *skb, struct net_device *dev)
{
	struct packet_type *ptype;
	get_fast_time(&skb->stamp);

	br_read_lock(BR_NETPROTO_LOCK);
	for (ptype = ptype_all; ptype!=NULL; ptype = ptype->next) 
	{
		/* Never send packets back to the socket
		 * they originated from - MvS (miquels@drinkel.ow.org)
		 */
		if ((ptype->dev == dev || !ptype->dev) &&
			((struct sock *)ptype->data != skb->sk))
		{
			struct sk_buff *skb2;
			if ((skb2 = skb_clone(skb, GFP_ATOMIC)) == NULL)
				break;

			/* skb->nh should be correctly
			   set by sender, so that the second statement is
			   just protection against buggy protocols.
			 */
			skb2->mac.raw = skb2->data;

			if (skb2->nh.raw < skb2->data || skb2->nh.raw > skb2->tail) {
				if (net_ratelimit())
					printk(KERN_DEBUG "protocol %04x is buggy, dev %s\n", skb2->protocol, dev->name);
				skb2->nh.raw = skb2->data;
			}

			skb2->h.raw = skb2->nh.raw;
			skb2->pkt_type = PACKET_OUTGOING;
			ptype->func(skb2, skb->dev, ptype);
		}
	}
	br_read_unlock(BR_NETPROTO_LOCK);
}

/* Calculate csum in the case, when packet is misrouted.
 * If it failed by some reason, ignore and send skb with wrong
 * checksum.
 */
struct sk_buff * skb_checksum_help(struct sk_buff *skb)
{
	int offset;
	unsigned int csum;

	offset = skb->h.raw - skb->data;
	if (offset > (int)skb->len)
		BUG();
	csum = skb_checksum(skb, offset, skb->len-offset, 0);

	offset = skb->tail - skb->h.raw;
	if (offset <= 0)
		BUG();
	if (skb->csum+2 > offset)
		BUG();

	*(u16*)(skb->h.raw + skb->csum) = csum_fold(csum);
	skb->ip_summed = CHECKSUM_NONE;
	return skb;
}

#ifdef CONFIG_HIGHMEM
/* Actually, we should eliminate this check as soon as we know, that:
 * 1. IOMMU is present and allows to map all the memory.
 * 2. No high memory really exists on this machine.
 */

static inline int
illegal_highdma(struct net_device *dev, struct sk_buff *skb)
{
	int i;

	if (dev->features&NETIF_F_HIGHDMA)
		return 0;

	for (i=0; i<skb_shinfo(skb)->nr_frags; i++)
		if (skb_shinfo(skb)->frags[i].page >= highmem_start_page)
			return 1;

	return 0;
}
#else
#define illegal_highdma(dev, skb)	(0)
#endif

/**************************************************************************
 *********************** FAST Path: Transmit Routines *********************
 **************************************************************************/

/**************************************************************************
 * FUNCTION NAME : dev_qos_xmit
 **************************************************************************
 * DESCRIPTION   :
 *  The function is called when a packet is transmitted on a device on which
 *  a QOS scheduler is running. The function passes control to the registered
 *  QOS scheduler. 
 *
 * NOTES         :
 *  This function was removed from the dev_queue_xmit to reduce the fast 
 *  path code. 
 ***************************************************************************/
static int dev_qos_xmit (struct sk_buff *skb, struct net_device* dev)
{
    int             ret;
    struct Qdisc*   q = dev->qdisc;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_SHAPER_ENTER);

    spin_lock_bh(&dev->queue_lock);
    ret = q->enqueue(skb, q);
    qdisc_run(dev);
    spin_unlock_bh(&dev->queue_lock);

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_SHAPER_EXIT);

	return ret == NET_XMIT_BYPASS ? NET_XMIT_SUCCESS : ret;
}

/**************************************************************************
 * FUNCTION NAME : dev_queue_xmit
 **************************************************************************
 * DESCRIPTION   :
 *	Queue a buffer for transmission to a network device. The caller must
 *	have set the device and priority and built the buffer before calling this 
 *	function. The function can be called from an interrupt.
 *
 *  The function has been optimized. In the case of the no QOS scheduling 
 *  i.e. tx_queue_len of 0; packets are transmitted directly to the driver 
 *  bypassing the default 
 ***************************************************************************/
int dev_queue_xmit(struct sk_buff *skb)
{
	struct net_device*  dev = skb->dev;
	struct Qdisc*       q = dev->qdisc;

    /* Send the frame only if the device is up. */	
    if (dev->flags & IFF_UP)
    {
        /* The device was UP. Push the packet to the translation unit only if the QoS was enabled. */
        if (gIsQoSEnabled == 1)
            ti_qos_translation_unit(skb, skb->dev);

        /* Link was UP: If there is NO queueing discipline (ex. Loopback device) or 
         * if the driver maintains its own queues and does not want OS support. Send the 
         * packet immediately. */
        if ((dev->tx_queue_len == 0) || (q->enqueue == NULL))
        {
            /* Send the frame immediately. */
		    spin_lock(&dev->xmit_lock);
            if (dev->hard_start_xmit(skb, dev) == 0) 
            {
                /* Packet succesfully transmitted. Unlock and return success. */
                spin_unlock(&dev->xmit_lock);
                return 0;
            }
 
            /* This will be the case when the driver is NOT able to send the packet. Clean
             * up the memory and release the lock. */
            spin_unlock(&dev->xmit_lock);
            net_error_handler(skb);
            return -ENETDOWN;
        }
        else
        {
            /* QOS Enabled or device wants us to handle queueing. Grab device queue */
            return dev_qos_xmit (skb, dev);
        }
    }

    /* The device was NOT up. Drop the packet and indicate that the link was DOWN. */
	net_error_handler(skb);
	return -ENETDOWN;
}

/*=======================================================================
			Receiver routines
  =======================================================================*/

int netdev_max_backlog = 300;
/* These numbers are selected based on intuition and some
 * experimentatiom, if you have more scientific way of doing this
 * please go ahead and fix things.
 */
int no_cong_thresh = 10;
int no_cong = 20;
int lo_cong = 100;
int mod_cong = 290;

struct netif_rx_stats netdev_rx_stat[NR_CPUS];


#ifdef CONFIG_NET_HW_FLOWCONTROL
atomic_t netdev_dropping = ATOMIC_INIT(0);
static unsigned long netdev_fc_mask = 1;
unsigned long netdev_fc_xoff = 0;
spinlock_t netdev_fc_lock = SPIN_LOCK_UNLOCKED;

static struct
{
	void (*stimul)(struct net_device *);
	struct net_device *dev;
} netdev_fc_slots[BITS_PER_LONG];

int netdev_register_fc(struct net_device *dev, void (*stimul)(struct net_device *dev))
{
int bit = 0;
	unsigned long flags;

	spin_lock_irqsave(&netdev_fc_lock, flags);
	if (netdev_fc_mask != ~0UL) {
		bit = ffz(netdev_fc_mask);
		netdev_fc_slots[bit].stimul = stimul;
		netdev_fc_slots[bit].dev = dev;
		set_bit(bit, &netdev_fc_mask);
		clear_bit(bit, &netdev_fc_xoff);
	}
	spin_unlock_irqrestore(&netdev_fc_lock, flags);
	return bit;
}

void netdev_unregister_fc(int bit)
{
	unsigned long flags;

	spin_lock_irqsave(&netdev_fc_lock, flags);
	if (bit > 0) {
		netdev_fc_slots[bit].stimul = NULL;
		netdev_fc_slots[bit].dev = NULL;
		clear_bit(bit, &netdev_fc_mask);
		clear_bit(bit, &netdev_fc_xoff);
	}
	spin_unlock_irqrestore(&netdev_fc_lock, flags);
}

static void netdev_wakeup(void)
{
	unsigned long xoff;

	spin_lock(&netdev_fc_lock);
	xoff = netdev_fc_xoff;
	netdev_fc_xoff = 0;
	while (xoff) {
		int i = ffz(~xoff);
		xoff &= ~(1<<i);
		netdev_fc_slots[i].stimul(netdev_fc_slots[i].dev);
	}
	spin_unlock(&netdev_fc_lock);
}
#endif

#ifdef OFFLINE_SAMPLE
static void sample_queue(unsigned long dummy)
{
/* 10 ms 0r 1ms -- i dont care -- JHS */
/* run from timer interrupt, preemption is always off here */
	int next_tick = 1;
	int cpu = smp_processor_id();

	get_sample_stats(cpu);
	next_tick += jiffies;
	mod_timer(&samp_timer, next_tick);
}
#endif

/* Deliver skb to an old protocol, which is not threaded well
   or which do not understand shared skbs.
 */
static int deliver_to_old_ones(struct packet_type *pt, struct sk_buff *skb, int last)
{
	static spinlock_t net_bh_lock = SPIN_LOCK_UNLOCKED;
	int ret = NET_RX_DROP;


	if (!last) {
		skb = skb_clone(skb, GFP_ATOMIC);
		if (skb == NULL)
			return ret;
	}
	if (skb_is_nonlinear(skb) && skb_linearize(skb, GFP_ATOMIC) != 0) {
		kfree_skb(skb);
		return ret;
	}

	/* The assumption (correct one) is that old protocols
	   did not depened on BHs different of NET_BH and TIMER_BH.
	 */

	/* Emulate NET_BH with special spinlock */
	spin_lock(&net_bh_lock);

	/* Disable timers and wait for all timers completion */
	tasklet_disable(bh_task_vec+TIMER_BH);

	ret = pt->func(skb, skb->dev, pt);

	tasklet_hi_enable(bh_task_vec+TIMER_BH);
	spin_unlock(&net_bh_lock);
	return ret;
}

/* Reparent skb to master device. This function is called
 * only from net_rx_action under BR_NETPROTO_LOCK. It is misuse
 * of BR_NETPROTO_LOCK, but it is OK for now.
 */
static __inline__ void skb_bond(struct sk_buff *skb)
{
	struct net_device *dev = skb->dev;
	
        if (dev->master) {
		dev_hold(dev->master);
		skb->dev = dev->master;
		dev_put(dev);
	}
}

static void net_tx_action(struct softirq_action *h)
{
	int cpu = smp_processor_id();

	if (softnet_data[cpu].completion_queue) {
		struct sk_buff *clist;

		local_irq_disable();
		clist = softnet_data[cpu].completion_queue;
		softnet_data[cpu].completion_queue = NULL;
		local_irq_enable();

		while (clist != NULL) {
			struct sk_buff *skb = clist;
			clist = clist->next;

			BUG_TRAP(atomic_read(&skb->users) == 0);
			__kfree_skb(skb);
		}
	}

	if (softnet_data[cpu].output_queue) {
		struct net_device *head;

		local_irq_disable();
		head = softnet_data[cpu].output_queue;
		softnet_data[cpu].output_queue = NULL;
		local_irq_enable();

		while (head != NULL) {
			struct net_device *dev = head;
			head = head->next_sched;

			smp_mb__before_clear_bit();
			clear_bit(__LINK_STATE_SCHED, &dev->state);

			if (spin_trylock(&dev->queue_lock)) {
				qdisc_run(dev);
				spin_unlock(&dev->queue_lock);
			} else {
				netif_schedule(dev);
			}
		}
	}
}

/**
 *      net_call_rx_atomic
 *      @fn: function to call
 *
 *      Make a function call that is atomic with respect to the protocol
 *      layers.
 */
 
void net_call_rx_atomic(void (*fn)(void))
{
	br_write_lock_bh(BR_NETPROTO_LOCK);
	fn();
	br_write_unlock_bh(BR_NETPROTO_LOCK);
}

#if defined(CONFIG_BRIDGE) || defined(CONFIG_BRIDGE_MODULE)
void (*br_handle_frame_hook)(struct sk_buff *skb) = NULL;
#endif

static __inline__ int handle_bridge(struct sk_buff *skb,
				     struct packet_type *pt_prev)
{
	int ret = NET_RX_DROP;

	if (pt_prev) {
		if (!pt_prev->data)
			ret = deliver_to_old_ones(pt_prev, skb, 0);
		else {
			atomic_inc(&skb->users);
			ret = pt_prev->func(skb, skb->dev, pt_prev);
		}
	}

	br_handle_frame_hook(skb);
	return ret;
}


#ifdef CONFIG_NET_DIVERT
static inline void handle_diverter(struct sk_buff *skb)
{
	/* if diversion is supported on device, then divert */
	if (skb->dev->divert && skb->dev->divert->divert)
		divert_frame(skb);
}
#endif   /* CONFIG_NET_DIVERT */

/**************************************************************************
 *********************** FAST Path: Receive Routines **********************
 **************************************************************************/

int high_priority_preempted = 0;
int norm_priority_preempted = 0;

/**************************************************************************
 * FUNCTION NAME : net_pass_data_handlers
 **************************************************************************
 * DESCRIPTION   :
 *  While optimizing the code, the logic to pass data to other handlers was
 *  removed. This was exposed while running the DHCP Client on the WAN 
 *  interface. The problem arises when the DHCP client opens another protocol
 *  listener through the function 'dev_add_pack'. The orignal code had to be
 *  put back but with some more optimizations so that we can still preserve
 *  the small fast path.
 *
 * NOTES         :
 *  There are 2 things missing from this implementation.
 *   1. No checking on the protocol type if the types fall in the same bucket.
 *   2. The dev passed through ptype is NOT used.
 *
 *  This is done on purpose. Add it when necessary.
***************************************************************************/
static void net_pass_data_other_handlers (struct packet_type* ptype,struct sk_buff* skb)
{
    struct packet_type *pt_prev = NULL;

    /* Cycle through all the registered protocol handlers. */
    while (ptype != NULL) 
    {
        /* Check if we have a previously remembered handler on which we need to 
         * send the packet. */
        if (pt_prev)
        {
            /* YES. Increment usage and pass packet to that handler. */
            atomic_inc(&skb->users);
			pt_prev->func(skb,skb->dev, pt_prev);
        }

        /* Remember the current protocol handler. */
        pt_prev = ptype;

        /* Jump to the next handler. */
        ptype = ptype->next;
    }

    /* Pass packet to the last handler. */
	if (pt_prev)
        pt_prev->func(skb, skb->dev, pt_prev);

    return;
}

/**************************************************************************
 * FUNCTION NAME : get_sample_stats
 **************************************************************************
 * DESCRIPTION   :
 *  The function is used to update the system congestion statistics. The 
 *  algorithm was changed to a very simple queue occupancy algorithm. If the
 *  queue occupancy exceeds the levels configured the approriate congestion
 *  level is reported.
 *
 *      NET_RX_SUCCESS      (no congestion)           
 *      NET_RX_CN_LOW       (low congestion) 
 *      NET_RX_CN_MOD       (moderate congestion)
 *      NET_RX_CN_HIGH      (high congestion) 
 *      NET_RX_DROP         (packet was dropped)
 *      
 * NOTES         :
 *  The function has been modified from the original function to handle 
 *  voice-data coexistence problems.
 ***************************************************************************/
void get_sample_stats(int cpu)
{
    /* PANKAJ: Debugging voice-data coexistence. */
	int blog = softnet_data[cpu].input_pkt_queue.qlen;
	if (blog > mod_cong) 
    {
		/* High Congestion. */
		softnet_data[cpu].cng_level = NET_RX_CN_HIGH;
	} 
    else if (blog > lo_cong) 
    {
        /* Moderate congestion. */
		softnet_data[cpu].cng_level = NET_RX_CN_MOD;
	}
    else if (blog > no_cong)
    {
        /* Low congestion. */
		softnet_data[cpu].cng_level = NET_RX_CN_LOW;
    }
	else
    {  
        /* No congestion */
		softnet_data[cpu].cng_level = NET_RX_SUCCESS;
    }
}

/**************************************************************************
 * FUNCTION NAME : netif_rx_original
 **************************************************************************
 * DESCRIPTION   :
 *  This function receives a packet from a device driver and queues it for
 *  the upper (protocol) levels to process.  It always succeeds. The buffer
 *  may be dropped during processing for congestion control or by the 
 *  protocol layers.
 *
 * RETURNS		 :
 *      NET_RX_SUCCESS      (no congestion)           
 *      NET_RX_CN_LOW       (low congestion) 
 *      NET_RX_CN_MOD       (moderate congestion)
 *      NET_RX_CN_HIGH      (high congestion) 
 *      NET_RX_DROP         (packet was dropped)
 *      
 * NOTES         :
 *  The function has been modified from the original function to handle 
 *  voice-data coexistence problems.
 ***************************************************************************/
int netif_rx_original(struct sk_buff *skb)
{
    int                     this_cpu = smp_processor_id();
    struct softnet_data*    queue;
    unsigned long           flags;

	/* The code is rearranged so that the path is the most
     * short when CPU is congested, but is still operating. */
	queue = &softnet_data[this_cpu];

    /* Disable interrupts. */
	local_irq_save(flags);

    /* Increment the stats. */
	netdev_rx_stat[this_cpu].total++;

    /* Check if we have space in the queue. */
    if (queue->input_pkt_queue.qlen <= netdev_max_backlog) 
    {
        if (queue->input_pkt_queue.qlen) 
        {
            if (queue->throttle)
                return net_receive_queue_overflow (this_cpu, skb, flags);

enqueue:
            /* Hold the device on which the packet was received. This device will be released
             * once the packet is processed. */ 
			dev_hold(skb->dev);

            /* Enqueuing the packet depends on the priority of the received packet. */
            if (skb->cb[ SKB_TI_PRIORITY_IDX ] == 0x0) 
            {
                /* LOW Priority: Queue the packet at the tail of the list. */
                __skb_queue_tail(&queue->input_pkt_queue,skb);
            }
            else
            {
                /* HIGH Priority: Queue the packet at the head of the list and raise the priority
                 * of the softirq daemon. */
                __skb_queue_head(&queue->input_pkt_queue,skb);
                net_set_priority(1);
            }

			/* Runs from irqs or BH's, no need to wake BH */
			cpu_raise_softirq(this_cpu, NET_RX_SOFTIRQ);

            /* Re-enable interrupts. */
            local_irq_restore(flags);

            /* Compute the congestion level reported on the system. */
#ifndef OFFLINE_SAMPLE
			get_sample_stats(this_cpu);
#endif
            /* Return the congestion level. */
			return softnet_data[this_cpu].cng_level;
		}

        if (queue->throttle) 
        {
			queue->throttle = 0;
#ifdef CONFIG_NET_HW_FLOWCONTROL
			if (atomic_dec_and_test(&netdev_dropping))
				netdev_wakeup();
#endif
		}
		goto enqueue;
	}

	if (queue->throttle == 0) {
		queue->throttle = 1;
		netdev_rx_stat[this_cpu].throttled++;
#ifdef CONFIG_NET_HW_FLOWCONTROL
		atomic_inc(&netdev_dropping);
#endif
	}

    /* The control comes here because the input packet queue was greater than
     * the length of the received packet. In this case, we need to drop the packet
     * NOTE: The function is called with interrupts disabled. */
    return net_receive_queue_overflow (this_cpu, skb, flags);
}

/**************************************************************************
 * FUNCTION NAME : netif_rx
 **************************************************************************
 * DESCRIPTION   :
 *  This function receives a packet from a device driver and queues it for
 *  the upper (protocol) levels to process.  It always succeeds. The buffer
 *  may be dropped during processing for congestion control or by the 
 *  protocol layers.
 *
 * RETURNS		 :
 *      NET_RX_SUCCESS      (no congestion)           
 *      NET_RX_CN_LOW       (low congestion) 
 *      NET_RX_CN_MOD       (moderate congestion)
 *      NET_RX_CN_HIGH      (high congestion) 
 *      NET_RX_DROP         (packet was dropped)
 *      
 * NOTES         :
 *  The function has been modified from the original function to handle 
 *  voice-data coexistence problems.
 ***************************************************************************/
int netif_rx(struct sk_buff *skb)
{
    /* Check if the QoS Framework is ENABLED. In that case call the 
     * network queue manager to enqueue the packet into the appropriate
     * queue. if not enabled go to the default code. */
    if (gIsQoSEnabled == 1)
        return ti_nqm_netif_rx(skb);
    
    return netif_rx_original(skb);
}

/**************************************************************************
 * FUNCTION NAME : net_set_priority
 **************************************************************************
 * DESCRIPTION   :
 *  The function was added as part of voice-data coexistence work. The 
 *  function raises the priority of the softirq daemon or sets it to default 
 *  levels. This is required because on detection of a high priority voice 
 *  packet, the softirq daemon should process the packet immediately without
 *  getting deferred by other high priority processes. 
 ***************************************************************************/
static void net_set_priority(int high_low)
{
    static int is_high_prio = 0;
    int this_cpu = smp_processor_id();

    extern void ksoftirqd_high_prio(int cpu);
    extern void ksoftirqd_norm_prio(int cpu);

    /* Check if we need to bump the priority high or move the priority 
     * of the softirq daemon back to the original values. */
    if (high_low)
    {
        /* Raise the priority. Do so only if the priority is already not high. */
        if (!is_high_prio)
        {
            /* Incrememet counter and raise priority. */
            high_priority_preempted++;
            ksoftirqd_high_prio(this_cpu);
            is_high_prio = 1;
        }
    }
    else
    {
        /* Lower to original value. Do so only if the priority is not already at default level. */
        if (is_high_prio)
        {
            /* Increase counter and set priority to default level. */
            norm_priority_preempted++;
            ksoftirqd_norm_prio(this_cpu);
            is_high_prio = 0;
        }
    }
}

/**************************************************************************
 * FUNCTION NAME : net_rx_action
 **************************************************************************
 * DESCRIPTION   :
 *  The network receive bottom half handler. The function dequeues the 
 *  network packets and passes them to the appropriate registered protocol
 *  handlers.
***************************************************************************/
static void net_rx_action(struct softirq_action *h)
{
#if 1
    /* Check if the QoS Framework is enabled. */
    if (gIsQoSEnabled == 1)
    {
        /* Use the Network Queue Manager to dequeue packets. */
        ti_nqm_net_rx_action (h);
        return;
    }
    else
    {
#endif
    register struct sk_buff*    skb;
    int                         this_cpu = smp_processor_id();
	struct softnet_data*        queue = &softnet_data[this_cpu];
	unsigned long               start_time = jiffies;
	int                         bugdet = netdev_max_backlog;    
	struct net_device*          rx_dev;
    struct packet_type*         ptype;
    int                         softnet_break = 0;

	br_read_lock(BR_NETPROTO_LOCK);

	for (;;) 
    {
        /* Disable interrupts and dequeue a packet from the NET queue. */
		local_irq_disable();
		skb = __skb_dequeue(&queue->input_pkt_queue);
		local_irq_enable();

        /* Check if there were any packets in the queue or not ? */
		if (skb != NULL)
     {
            /* YES. Process the dequeued packets. Save the device information on 
             * which the packet was received. */
    		   rx_dev = skb->dev;

            /* Check if voice is enabled. */
            if (gIsVoiceEnabled == 1)
            {
                /* Check the priority of the received packet. */
                if (skb->cb[ SKB_TI_PRIORITY_IDX ] == 0x1)
                {
                    /* HIGH Priority: Increase the priority of the softirq daemon. */
                    net_set_priority(1);
                }
                else
                {
                    /* LOW Priority: Reset the priority to default. */
                    net_set_priority(0);

                    /* Check the congestion level reported by the system. If the system 
                     * is congested, drop the packet. */
                    if (softnet_data[this_cpu].cng_level >= NET_RX_CN_HIGH)
                    {
                        /* Drop Packet and clean memory. */
                        net_rx_drop_packet(skb);
                        continue;
                    }
                }
            }

            /* Initialize the data pointers. */
    		    skb->h.raw = skb->nh.raw = skb->data;

            /* WLAN: Station to Station Handling. */ 
            if ( skb->cb[ SKB_TI_WLAN_IDX ] )
            {
                /* The protocol information is NOT valid. Extract the protocol information 
                 * from the Ethernet header. */
                skb->protocol = *(unsigned short*)(skb->data+12);

                /* Initialize the network header to the correct location. */
                skb->nh.raw = skb->data + sizeof(struct ethhdr);

                /* Check if forwarding is also required */
                if ( skb->cb[ SKB_TI_WLAN_IDX ] == SKB_TI_WLAN_PACKET_LOOP_AND_FORWARD )
                {               
                    struct sk_buff *skb2;

                    /* Clear WLAN marking */
                    skb->cb[ SKB_TI_WLAN_IDX ] = 0;

                    /* Clone the skb */
                    if ( (skb2 = skb_clone(skb, GFP_ATOMIC)) != NULL )  
                    {
                       /* Pass the cloned packet back to the driver. */
                       dev_queue_xmit (skb2);

                       goto rx_action_next;
                    }
                }

                /* Clear WLAN marking */
                skb->cb[ SKB_TI_WLAN_IDX ] = 0;

                /* Pass the packet back to the driver and return. */
                dev_queue_xmit (skb);
            }
            else
            {
               rx_action_next:
               
               /* Check if the interface on which the packet was received lies below the bridge
                * and if so then call the bridging receive function. */
               if ((skb->dev->br_port != NULL) && (br_handle_frame_hook != NULL))
               {
                 /* Remember the physical port on which data came in */
        		   skb->__unused = skb->dev->if_port;
                 br_handle_frame_hook(skb);
               }
               else
               { 
                   /* Determine the protocol handler. */
                   ptype = ptype_base[ntohs(skb->protocol)&15];
                   if (ptype != NULL)
                   {
                       /* Check if there is another registered handler for the same packet. */
                       if (ptype->next == NULL)
                       {
                           /* NO. Pass the packet to the handler */
        	        	   ptype->func(skb, skb->dev, ptype);
                       }
                       else
                       {
                           /* YES. Push data to all registered protocol handlers. */
                           net_pass_data_other_handlers (ptype, skb);
                       }
                   }
                   else
                   {
                       /* Invalid packet as there is no registered handler. Clean memory
                        * and drop the packet. This is not the fast path so call the error 
                        * function to do the needful. Reducing the amount of code in the fast
                        * path. */
                       net_error_handler(skb);
                   }
               }
            }

            /* Release the receive device on which the packet was received. */
        	 dev_put(rx_dev);

            /* Check if the NET RX BH has reached its execution limits. */
    		if ((bugdet-- < 0) || (jiffies - start_time) > 1)
            {
	    		softnet_break = 1;
                break;
            }
        }
        else
        {
            /* No more packets left, we are done. */
            break;
        }
	}

    /* Exit Criticial section. */
	br_read_unlock(BR_NETPROTO_LOCK);

    /* Disable Interrupts. */
    local_irq_disable();
    if (softnet_break == 0)
    {
        /* Check if the queue was being throttled and if so reset the flag. */
	    if (queue->throttle) 
    		queue->throttle = 0;
    }
    else
    {
        /* Net receive bottom half exceeded its time; increment the statistics. 
         * and reschedule the bottom half for execution again. */
        netdev_rx_stat[this_cpu].time_squeeze++;
	    cpu_raise_softirq(this_cpu, NET_RX_SOFTIRQ);
    }

    /* Enable Interrupts. */
    local_irq_enable();

	NET_PROFILE_LEAVE(softnet_process);
	return;
    }
}

static gifconf_func_t * gifconf_list [NPROTO];

/**
 *	register_gifconf	-	register a SIOCGIF handler
 *	@family: Address family
 *	@gifconf: Function handler
 *
 *	Register protocol dependent address dumping routines. The handler
 *	that is passed must not be freed or reused until it has been replaced
 *	by another handler.
 */
 
int register_gifconf(unsigned int family, gifconf_func_t * gifconf)
{
	if (family>=NPROTO)
		return -EINVAL;
	gifconf_list[family] = gifconf;
	return 0;
}


/*
 *	Map an interface index to its name (SIOCGIFNAME)
 */

/*
 *	We need this ioctl for efficient implementation of the
 *	if_indextoname() function required by the IPv6 API.  Without
 *	it, we would have to search all the interfaces to find a
 *	match.  --pb
 */

static int dev_ifname(struct ifreq *arg)
{
	struct net_device *dev;
	struct ifreq ifr;

	/*
	 *	Fetch the caller's info block. 
	 */
	
	if (copy_from_user(&ifr, arg, sizeof(struct ifreq)))
		return -EFAULT;

	read_lock(&dev_base_lock);
	dev = __dev_get_by_index(ifr.ifr_ifindex);
	if (!dev) {
		read_unlock(&dev_base_lock);
		return -ENODEV;
	}

	strcpy(ifr.ifr_name, dev->name);
	read_unlock(&dev_base_lock);

	if (copy_to_user(arg, &ifr, sizeof(struct ifreq)))
		return -EFAULT;
	return 0;
}

/*
 *	Perform a SIOCGIFCONF call. This structure will change
 *	size eventually, and there is nothing I can do about it.
 *	Thus we will need a 'compatibility mode'.
 */

static int dev_ifconf(char *arg)
{
	struct ifconf ifc;
	struct net_device *dev;
	char *pos;
	int len;
	int total;
	int i;

	/*
	 *	Fetch the caller's info block. 
	 */
	
	if (copy_from_user(&ifc, arg, sizeof(struct ifconf)))
		return -EFAULT;

	pos = ifc.ifc_buf;
	len = ifc.ifc_len;

	/*
	 *	Loop over the interfaces, and write an info block for each. 
	 */

	total = 0;
	for (dev = dev_base; dev != NULL; dev = dev->next) {
		for (i=0; i<NPROTO; i++) {
			if (gifconf_list[i]) {
				int done;
				if (pos==NULL) {
					done = gifconf_list[i](dev, NULL, 0);
				} else {
					done = gifconf_list[i](dev, pos+total, len-total);
				}
				if (done<0) {
					return -EFAULT;
				}
				total += done;
			}
		}
  	}

	/*
	 *	All done.  Write the updated control block back to the caller. 
	 */
	ifc.ifc_len = total;

	if (copy_to_user(arg, &ifc, sizeof(struct ifconf)))
		return -EFAULT; 

	/* 
	 * 	Both BSD and Solaris return 0 here, so we do too.
	 */
	return 0;
}

/*
 *	This is invoked by the /proc filesystem handler to display a device
 *	in detail.
 */

#ifdef CONFIG_PROC_FS

static int sprintf_stats(char *buffer, struct net_device *dev)
{
	struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);
	int size;
	
	if (stats)
		size = sprintf(buffer, "%6s:%8lu %7lu %4lu %4lu %4lu %5lu %10lu %9lu %8lu %7lu %4lu %4lu %4lu %5lu %7lu %10lu\n",
 		   dev->name,
		   stats->rx_bytes,
		   stats->rx_packets, stats->rx_errors,
		   stats->rx_dropped + stats->rx_missed_errors,
		   stats->rx_fifo_errors,
		   stats->rx_length_errors + stats->rx_over_errors
		   + stats->rx_crc_errors + stats->rx_frame_errors,
		   stats->rx_compressed, stats->multicast,
		   stats->tx_bytes,
		   stats->tx_packets, stats->tx_errors, stats->tx_dropped,
		   stats->tx_fifo_errors, stats->collisions,
		   stats->tx_carrier_errors + stats->tx_aborted_errors
		   + stats->tx_window_errors + stats->tx_heartbeat_errors,
		   stats->tx_compressed);
	else
		size = sprintf(buffer, "%6s: No statistics available.\n", dev->name);

	return size;
}

/*
 *	Called from the PROCfs module. This now uses the new arbitrary sized /proc/net interface
 *	to create /proc/net/dev
 */
 
static int dev_get_info(char *buffer, char **start, off_t offset, int length)
{
	int len = 0;
	off_t begin = 0;
	off_t pos = 0;
	int size;
	struct net_device *dev;


	size = sprintf(buffer, 
		"Inter-|   Receive                                                |  Transmit\n"
		" face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed\n");
	
	pos += size;
	len += size;
	

	read_lock(&dev_base_lock);
	for (dev = dev_base; dev != NULL; dev = dev->next) {
		size = sprintf_stats(buffer+len, dev);
		len += size;
		pos = begin + len;
				
		if (pos < offset) {
			len = 0;
			begin = pos;
		}
		if (pos > offset + length)
			break;
	}
	read_unlock(&dev_base_lock);

	*start = buffer + (offset - begin);	/* Start of wanted data */
	len -= (offset - begin);		/* Start slop */
	if (len > length)
		len = length;			/* Ending slop */
	if (len < 0)
		len = 0;
	return len;
}

static int dev_proc_stats(char *buffer, char **start, off_t offset,
			  int length, int *eof, void *data)
{
	int i, lcpu;
	int len=0;

	for (lcpu=0; lcpu<smp_num_cpus; lcpu++) {
		i = cpu_logical_map(lcpu);
		len += sprintf(buffer+len, "%08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n",
			       netdev_rx_stat[i].total,
			       netdev_rx_stat[i].dropped,
			       netdev_rx_stat[i].time_squeeze,
			       netdev_rx_stat[i].throttled,
			       netdev_rx_stat[i].fastroute_hit,
			       netdev_rx_stat[i].fastroute_success,
			       netdev_rx_stat[i].fastroute_defer,
			       netdev_rx_stat[i].fastroute_deferred_out,
#if 0
			       netdev_rx_stat[i].fastroute_latency_reduction
#else
			       netdev_rx_stat[i].cpu_collision,
#endif
                   softnet_data[0].input_pkt_queue.qlen
			       );
	}

	len -= offset;

	if (len > length)
		len = length;
	if (len < 0)
		len = 0;

	*start = buffer + offset;
	*eof = 1;

	return len;
}

#endif	/* CONFIG_PROC_FS */


#ifdef WIRELESS_EXT
#ifdef CONFIG_PROC_FS

/*
 * Print one entry of /proc/net/wireless
 * This is a clone of /proc/net/dev (just above)
 */
static int sprintf_wireless_stats(char *buffer, struct net_device *dev)
{
	/* Get stats from the driver */
	struct iw_statistics *stats = (dev->get_wireless_stats ?
				       dev->get_wireless_stats(dev) :
				       (struct iw_statistics *) NULL);
	int size;

	if (stats != (struct iw_statistics *) NULL) {
		size = sprintf(buffer,
			       "%6s: %04x  %3d%c  %3d%c  %3d%c  %6d %6d %6d %6d %6d   %6d\n",
			       dev->name,
			       stats->status,
			       stats->qual.qual,
			       stats->qual.updated & 1 ? '.' : ' ',
			       stats->qual.level,
			       stats->qual.updated & 2 ? '.' : ' ',
			       stats->qual.noise,
			       stats->qual.updated & 4 ? '.' : ' ',
			       stats->discard.nwid,
			       stats->discard.code,
			       stats->discard.fragment,
			       stats->discard.retries,
			       stats->discard.misc,
			       stats->miss.beacon);
		stats->qual.updated = 0;
	}
	else
		size = 0;

	return size;
}

/*
 * Print info for /proc/net/wireless (print all entries)
 * This is a clone of /proc/net/dev (just above)
 */
static int dev_get_wireless_info(char * buffer, char **start, off_t offset,
			  int length)
{
	int		len = 0;
	off_t		begin = 0;
	off_t		pos = 0;
	int		size;
	
	struct net_device *	dev;

	size = sprintf(buffer,
		       "Inter-| sta-|   Quality        |   Discarded packets               | Missed\n"
		       " face | tus | link level noise |  nwid  crypt   frag  retry   misc | beacon\n"
			);
	
	pos += size;
	len += size;

	read_lock(&dev_base_lock);
	for (dev = dev_base; dev != NULL; dev = dev->next) {
		size = sprintf_wireless_stats(buffer + len, dev);
		len += size;
		pos = begin + len;

		if (pos < offset) {
			len = 0;
			begin = pos;
		}
		if (pos > offset + length)
			break;
	}
	read_unlock(&dev_base_lock);

	*start = buffer + (offset - begin);	/* Start of wanted data */
	len -= (offset - begin);		/* Start slop */
	if (len > length)
		len = length;			/* Ending slop */
	if (len < 0)
		len = 0;

	return len;
}
#endif	/* CONFIG_PROC_FS */

/*
 *	Allow programatic access to /proc/net/wireless even if /proc
 *	doesn't exist... Also more efficient...
 */
static inline int dev_iwstats(struct net_device *dev, struct ifreq *ifr)
{
	/* Get stats from the driver */
	struct iw_statistics *stats = (dev->get_wireless_stats ?
				       dev->get_wireless_stats(dev) :
				       (struct iw_statistics *) NULL);

	if (stats != (struct iw_statistics *) NULL) {
		struct iwreq *	wrq = (struct iwreq *)ifr;

		/* Copy statistics to the user buffer */
		if(copy_to_user(wrq->u.data.pointer, stats,
				sizeof(struct iw_statistics)))
			return -EFAULT;

		/* Check if we need to clear the update flag */
		if(wrq->u.data.flags != 0)
			stats->qual.updated = 0;
		return(0);
	} else
		return -EOPNOTSUPP;
}
#endif	/* WIRELESS_EXT */

/**
 *	netdev_set_master	-	set up master/slave pair
 *	@slave: slave device
 *	@master: new master device
 *
 *	Changes the master device of the slave. Pass %NULL to break the
 *	bonding. The caller must hold the RTNL semaphore. On a failure
 *	a negative errno code is returned. On success the reference counts
 *	are adjusted, %RTM_NEWLINK is sent to the routing socket and the
 *	function returns zero.
 */
 
int netdev_set_master(struct net_device *slave, struct net_device *master)
{
	struct net_device *old = slave->master;

	ASSERT_RTNL();

    /* Debug message: NSP does not use the master/slave feature. This is a
     * debug message that will be printed on the console in case the feature
     * is used. In such a case we will need to fix the net_rx_bh as the
     * skb_bond was removed for optimizations. */
    printk ("NSP Error: Setting the device %s as a SLAVE to %s.\n", slave->name, master->name);

	if (master) {
		if (old)
			return -EBUSY;
		dev_hold(master);
	}

	br_write_lock_bh(BR_NETPROTO_LOCK);
	slave->master = master;
	br_write_unlock_bh(BR_NETPROTO_LOCK);

	if (old)
		dev_put(old);

	if (master)
		slave->flags |= IFF_SLAVE;
	else
		slave->flags &= ~IFF_SLAVE;

	rtmsg_ifinfo(RTM_NEWLINK, slave, IFF_SLAVE);
	return 0;
}

/**
 *	dev_set_promiscuity	- update promiscuity count on a device
 *	@dev: device
 *	@inc: modifier
 *
 *	Add or remove promsicuity from a device. While the count in the device
 *	remains above zero the interface remains promiscuous. Once it hits zero
 *	the device reverts back to normal filtering operation. A negative inc
 *	value is used to drop promiscuity on the device.
 */
 
void dev_set_promiscuity(struct net_device *dev, int inc)
{
	unsigned short old_flags = dev->flags;

	dev->flags |= IFF_PROMISC;
	if ((dev->promiscuity += inc) == 0)
		dev->flags &= ~IFF_PROMISC;
	if (dev->flags^old_flags) {
#ifdef CONFIG_NET_FASTROUTE
		if (dev->flags&IFF_PROMISC) {
			netdev_fastroute_obstacles++;
			dev_clear_fastroute(dev);
		} else
			netdev_fastroute_obstacles--;
#endif
		dev_mc_upload(dev);
		printk(KERN_INFO "device %s %s promiscuous mode\n",
		       dev->name, (dev->flags&IFF_PROMISC) ? "entered" : "left");
	}
}

/**
 *	dev_set_allmulti	- update allmulti count on a device
 *	@dev: device
 *	@inc: modifier
 *
 *	Add or remove reception of all multicast frames to a device. While the
 *	count in the device remains above zero the interface remains listening
 *	to all interfaces. Once it hits zero the device reverts back to normal
 *	filtering operation. A negative @inc value is used to drop the counter
 *	when releasing a resource needing all multicasts.
 */

void dev_set_allmulti(struct net_device *dev, int inc)
{
	unsigned short old_flags = dev->flags;

	dev->flags |= IFF_ALLMULTI;
	if ((dev->allmulti += inc) == 0)
		dev->flags &= ~IFF_ALLMULTI;
	if (dev->flags^old_flags)
		dev_mc_upload(dev);
}

int dev_change_flags(struct net_device *dev, unsigned flags)
{
	int ret;
	int old_flags = dev->flags;

	/*
	 *	Set the flags on our device.
	 */

	dev->flags = (flags & (IFF_DEBUG|IFF_NOTRAILERS|IFF_NOARP|IFF_DYNAMIC|
			       IFF_MULTICAST|IFF_PORTSEL|IFF_AUTOMEDIA)) |
				       (dev->flags & (IFF_UP|IFF_VOLATILE|IFF_PROMISC|IFF_ALLMULTI));

	/*
	 *	Load in the correct multicast list now the flags have changed.
	 */				

	dev_mc_upload(dev);

	/*
	 *	Have we downed the interface. We handle IFF_UP ourselves
	 *	according to user attempts to set it, rather than blindly
	 *	setting it.
	 */

	ret = 0;
	if ((old_flags^flags)&IFF_UP)	/* Bit is different  ? */
	{
		ret = ((old_flags & IFF_UP) ? dev_close : dev_open)(dev);

		if (ret == 0) 
			dev_mc_upload(dev);
	}

	if (dev->flags&IFF_UP &&
	    ((old_flags^dev->flags)&~(IFF_UP|IFF_PROMISC|IFF_ALLMULTI|IFF_VOLATILE)))
		notifier_call_chain(&netdev_chain, NETDEV_CHANGE, dev);

	if ((flags^dev->gflags)&IFF_PROMISC) {
		int inc = (flags&IFF_PROMISC) ? +1 : -1;
		dev->gflags ^= IFF_PROMISC;
		dev_set_promiscuity(dev, inc);
	}

	/* NOTE: order of synchronization of IFF_PROMISC and IFF_ALLMULTI
	   is important. Some (broken) drivers set IFF_PROMISC, when
	   IFF_ALLMULTI is requested not asking us and not reporting.
	 */
	if ((flags^dev->gflags)&IFF_ALLMULTI) {
		int inc = (flags&IFF_ALLMULTI) ? +1 : -1;
		dev->gflags ^= IFF_ALLMULTI;
		dev_set_allmulti(dev, inc);
	}

	if (old_flags^dev->flags)
		rtmsg_ifinfo(RTM_NEWLINK, dev, old_flags^dev->flags);

	return ret;
}

/*
 *	Perform the SIOCxIFxxx calls. 
 */
 
static int dev_ifsioc(struct ifreq *ifr, unsigned int cmd)
{
	struct net_device *dev;
	int err;

	if ((dev = __dev_get_by_name(ifr->ifr_name)) == NULL)
		return -ENODEV;

	switch(cmd) 
	{
		case SIOCGIFFLAGS:	/* Get interface flags */
			ifr->ifr_flags = (dev->flags&~(IFF_PROMISC|IFF_ALLMULTI|IFF_RUNNING))
				|(dev->gflags&(IFF_PROMISC|IFF_ALLMULTI));
			if (netif_running(dev) && netif_carrier_ok(dev))
				ifr->ifr_flags |= IFF_RUNNING;
			return 0;

		case SIOCSIFFLAGS:	/* Set interface flags */
			return dev_change_flags(dev, ifr->ifr_flags);
		
		case SIOCGIFMETRIC:	/* Get the metric on the interface (currently unused) */
			ifr->ifr_metric = 0;
			return 0;
			
		case SIOCSIFMETRIC:	/* Set the metric on the interface (currently unused) */
			return -EOPNOTSUPP;
	
		case SIOCGIFMTU:	/* Get the MTU of a device */
			ifr->ifr_mtu = dev->mtu;
			return 0;
	
		case SIOCSIFMTU:	/* Set the MTU of a device */
			if (ifr->ifr_mtu == dev->mtu)
				return 0;

			/*
			 *	MTU must be positive.
			 */
			 
			if (ifr->ifr_mtu<0)
				return -EINVAL;

			if (!netif_device_present(dev))
				return -ENODEV;

			if (dev->change_mtu)
				err = dev->change_mtu(dev, ifr->ifr_mtu);
			else {
				dev->mtu = ifr->ifr_mtu;
				err = 0;
			}
			if (!err && dev->flags&IFF_UP)
				notifier_call_chain(&netdev_chain, NETDEV_CHANGEMTU, dev);
			return err;

		case SIOCGIFHWADDR:
			memcpy(ifr->ifr_hwaddr.sa_data,dev->dev_addr, MAX_ADDR_LEN);
			ifr->ifr_hwaddr.sa_family=dev->type;
			return 0;
				
		case SIOCSIFHWADDR:
			if (dev->set_mac_address == NULL)
				return -EOPNOTSUPP;
			if (ifr->ifr_hwaddr.sa_family!=dev->type)
				return -EINVAL;
			if (!netif_device_present(dev))
				return -ENODEV;
			err = dev->set_mac_address(dev, &ifr->ifr_hwaddr);
			if (!err)
				notifier_call_chain(&netdev_chain, NETDEV_CHANGEADDR, dev);
			return err;
			
		case SIOCSIFHWBROADCAST:
			if (ifr->ifr_hwaddr.sa_family!=dev->type)
				return -EINVAL;
			memcpy(dev->broadcast, ifr->ifr_hwaddr.sa_data, MAX_ADDR_LEN);
			notifier_call_chain(&netdev_chain, NETDEV_CHANGEADDR, dev);
			return 0;

		case SIOCGIFMAP:
			ifr->ifr_map.mem_start=dev->mem_start;
			ifr->ifr_map.mem_end=dev->mem_end;
			ifr->ifr_map.base_addr=dev->base_addr;
			ifr->ifr_map.irq=dev->irq;
			ifr->ifr_map.dma=dev->dma;
			ifr->ifr_map.port=dev->if_port;
			return 0;
			
		case SIOCSIFMAP:
			if (dev->set_config) {
				if (!netif_device_present(dev))
					return -ENODEV;
				return dev->set_config(dev,&ifr->ifr_map);
			}
			return -EOPNOTSUPP;
			
		case SIOCADDMULTI:
			if (dev->set_multicast_list == NULL ||
			    ifr->ifr_hwaddr.sa_family != AF_UNSPEC)
				return -EINVAL;
			if (!netif_device_present(dev))
				return -ENODEV;
			dev_mc_add(dev,ifr->ifr_hwaddr.sa_data, dev->addr_len, 1);
			return 0;

		case SIOCDELMULTI:
			if (dev->set_multicast_list == NULL ||
			    ifr->ifr_hwaddr.sa_family!=AF_UNSPEC)
				return -EINVAL;
			if (!netif_device_present(dev))
				return -ENODEV;
			dev_mc_delete(dev,ifr->ifr_hwaddr.sa_data,dev->addr_len, 1);
			return 0;

		case SIOCGIFINDEX:
			ifr->ifr_ifindex = dev->ifindex;
			return 0;

		case SIOCGIFTXQLEN:
			ifr->ifr_qlen = dev->tx_queue_len;
			return 0;

		case SIOCSIFTXQLEN:
			if (ifr->ifr_qlen<0)
				return -EINVAL;
			dev->tx_queue_len = ifr->ifr_qlen;
			return 0;

		case SIOCSIFNAME:
			if (dev->flags&IFF_UP)
				return -EBUSY;
			if (__dev_get_by_name(ifr->ifr_newname))
				return -EEXIST;
			memcpy(dev->name, ifr->ifr_newname, IFNAMSIZ);
			dev->name[IFNAMSIZ-1] = 0;
			notifier_call_chain(&netdev_chain, NETDEV_CHANGENAME, dev);
			return 0;

#ifdef WIRELESS_EXT
		case SIOCGIWSTATS:
			return dev_iwstats(dev, ifr);
#endif	/* WIRELESS_EXT */

#ifdef CONFIG_NET_SCH_PRIOWRR
		case SIOCGIFASYMMTU:	/* Get the Asymmetric MTU of a device */
			ifr->ifr_mtu = dev->unused_alignment_fixer;
			return 0;
	
		case SIOCSIFASYMMTU:	/* Set the Asymmetric MTU of a device */
			printk("SIOCSIFASYMMTU\n");
			if (ifr->ifr_mtu == dev->unused_alignment_fixer)
				return 0;

			/*	MTU must be positive and should be less than MTU. */
			 
			if ((ifr->ifr_mtu<0) || (ifr->ifr_mtu>dev->mtu))
				return -EINVAL;

			if (!netif_device_present(dev))
				return -ENODEV;

			dev->unused_alignment_fixer = (unsigned short)ifr->ifr_mtu;
			err = 0;
			
			/*
			if (!err && dev->flags&IFF_UP)
				notifier_call_chain(&netdev_chain, NETDEV_CHANGEMTU, dev);
			*/
			return err;
#endif /* CONFIG_NET_SCH_PRIOWRR */

		/*
		 *	Unknown or private ioctl
		 */

		default:
			if ((cmd >= SIOCDEVPRIVATE &&
			    cmd <= SIOCDEVPRIVATE + 15) ||
			    cmd == SIOCBONDENSLAVE ||
			    cmd == SIOCBONDRELEASE ||
			    cmd == SIOCBONDSETHWADDR ||
			    cmd == SIOCBONDSLAVEINFOQUERY ||
			    cmd == SIOCBONDINFOQUERY ||
			    cmd == SIOCBONDCHANGEACTIVE ||
			    cmd == SIOCETHTOOL ||
			    cmd == SIOCGMIIPHY ||
			    cmd == SIOCGMIIREG ||
			    cmd == SIOCSMIIREG) {
				if (dev->do_ioctl) {
					if (!netif_device_present(dev))
						return -ENODEV;
					return dev->do_ioctl(dev, ifr, cmd);
				}
				return -EOPNOTSUPP;
			}

#ifdef WIRELESS_EXT
			if (cmd >= SIOCIWFIRST && cmd <= SIOCIWLAST) {
				if (dev->do_ioctl) {
					if (!netif_device_present(dev))
						return -ENODEV;
					return dev->do_ioctl(dev, ifr, cmd);
				}
				return -EOPNOTSUPP;
			}
#endif	/* WIRELESS_EXT */

	}
	return -EINVAL;
}

/*
 *	This function handles all "interface"-type I/O control requests. The actual
 *	'doing' part of this is dev_ifsioc above.
 */

/**
 *	dev_ioctl	-	network device ioctl
 *	@cmd: command to issue
 *	@arg: pointer to a struct ifreq in user space
 *
 *	Issue ioctl functions to devices. This is normally called by the
 *	user space syscall interfaces but can sometimes be useful for 
 *	other purposes. The return value is the return from the syscall if
 *	positive or a negative errno code on error.
 */

int dev_ioctl(unsigned int cmd, void *arg)
{
	struct ifreq ifr;
	int ret;
	char *colon;

	/* One special case: SIOCGIFCONF takes ifconf argument
	   and requires shared lock, because it sleeps writing
	   to user space.
	 */
	   
	if (cmd == SIOCGIFCONF) {
		rtnl_shlock();
		ret = dev_ifconf((char *) arg);
		rtnl_shunlock();
		return ret;
	}
	if (cmd == SIOCGIFNAME) {
		return dev_ifname((struct ifreq *)arg);
	}

	if (copy_from_user(&ifr, arg, sizeof(struct ifreq)))
		return -EFAULT;

	ifr.ifr_name[IFNAMSIZ-1] = 0;

	colon = strchr(ifr.ifr_name, ':');
	if (colon)
		*colon = 0;

	/*
	 *	See which interface the caller is talking about. 
	 */
	 
	switch(cmd) 
	{
		/*
		 *	These ioctl calls:
		 *	- can be done by all.
		 *	- atomic and do not require locking.
		 *	- return a value
		 */
		 
		case SIOCGIFFLAGS:
		case SIOCGIFMETRIC:
		case SIOCGIFMTU:
		case SIOCGIFHWADDR:
		case SIOCGIFSLAVE:
		case SIOCGIFMAP:
		case SIOCGIFINDEX:
		case SIOCGIFTXQLEN:
#ifdef CONFIG_NET_SCH_PRIOWRR
		case SIOCGIFASYMMTU:
#endif
			dev_load(ifr.ifr_name);
			read_lock(&dev_base_lock);
			ret = dev_ifsioc(&ifr, cmd);
			read_unlock(&dev_base_lock);
			if (!ret) {
				if (colon)
					*colon = ':';
				if (copy_to_user(arg, &ifr, sizeof(struct ifreq)))
					return -EFAULT;
			}
			return ret;

		/*
		 *	These ioctl calls:
		 *	- require superuser power.
		 *	- require strict serialization.
		 *	- return a value
		 */
		 
		case SIOCETHTOOL:
		case SIOCGMIIPHY:
		case SIOCGMIIREG:
			if (!capable(CAP_NET_ADMIN))
				return -EPERM;
			dev_load(ifr.ifr_name);
			dev_probe_lock();
			rtnl_lock();
			ret = dev_ifsioc(&ifr, cmd);
			rtnl_unlock();
			dev_probe_unlock();
			if (!ret) {
				if (colon)
					*colon = ':';
				if (copy_to_user(arg, &ifr, sizeof(struct ifreq)))
					return -EFAULT;
			}
			return ret;

		/*
		 *	These ioctl calls:
		 *	- require superuser power.
		 *	- require strict serialization.
		 *	- do not return a value
		 */
		 
		case SIOCSIFFLAGS:
		case SIOCSIFMETRIC:
		case SIOCSIFMTU:
		case SIOCSIFMAP:
		case SIOCSIFHWADDR:
		case SIOCSIFSLAVE:
		case SIOCADDMULTI:
		case SIOCDELMULTI:
		case SIOCSIFHWBROADCAST:
		case SIOCSIFTXQLEN:
		case SIOCSIFNAME:
		case SIOCSMIIREG:
		case SIOCBONDENSLAVE:
		case SIOCBONDRELEASE:
		case SIOCBONDSETHWADDR:
		case SIOCBONDSLAVEINFOQUERY:
		case SIOCBONDINFOQUERY:
		case SIOCBONDCHANGEACTIVE:
#ifdef CONFIG_NET_SCH_PRIOWRR
		case SIOCSIFASYMMTU:
#endif
			if (!capable(CAP_NET_ADMIN))
				return -EPERM;
			dev_load(ifr.ifr_name);
			dev_probe_lock();
			rtnl_lock();
			ret = dev_ifsioc(&ifr, cmd);
			rtnl_unlock();
			dev_probe_unlock();
			return ret;
	
		case SIOCGIFMEM:
			/* Get the per device memory space. We can add this but currently
			   do not support it */
		case SIOCSIFMEM:
			/* Set the per device memory buffer space. Not applicable in our case */
		case SIOCSIFLINK:
			return -EINVAL;

		/*
		 *	Unknown or private ioctl.
		 */	
		 
		default:
			if (cmd >= SIOCDEVPRIVATE &&
			    cmd <= SIOCDEVPRIVATE + 15) {
				dev_load(ifr.ifr_name);
				dev_probe_lock();
				rtnl_lock();
				ret = dev_ifsioc(&ifr, cmd);
				rtnl_unlock();
				dev_probe_unlock();
				if (!ret && copy_to_user(arg, &ifr, sizeof(struct ifreq)))
					return -EFAULT;
				return ret;
			}
#ifdef WIRELESS_EXT
			/* Take care of Wireless Extensions */
			if (cmd >= SIOCIWFIRST && cmd <= SIOCIWLAST) {
				/* If command is `set a parameter', or
				 * `get the encoding parameters', check if
				 * the user has the right to do it */
				if (IW_IS_SET(cmd) || (cmd == SIOCGIWENCODE)) {
					if(!capable(CAP_NET_ADMIN))
						return -EPERM;
				}
				dev_load(ifr.ifr_name);
				rtnl_lock();
				/* Follow me in net/core/wireless.c */
				ret = wireless_process_ioctl(&ifr, cmd);
				rtnl_unlock();
				if (!ret && IW_IS_GET(cmd) &&
				    copy_to_user(arg, &ifr, sizeof(struct ifreq)))
					return -EFAULT;
				return ret;
			}
#endif	/* WIRELESS_EXT */
			return -EINVAL;
	}
}


/**
 *	dev_new_index	-	allocate an ifindex
 *
 *	Returns a suitable unique value for a new device interface
 *	number.  The caller must hold the rtnl semaphore or the
 *	dev_base_lock to be sure it remains unique.
 */
 
int dev_new_index(void)
{
	static int ifindex;
	for (;;) {
		if (++ifindex <= 0)
			ifindex=1;
		if (__dev_get_by_index(ifindex) == NULL)
			return ifindex;
	}
}

static int dev_boot_phase = 1;

/**
 *	register_netdevice	- register a network device
 *	@dev: device to register
 *	
 *	Take a completed network device structure and add it to the kernel
 *	interfaces. A %NETDEV_REGISTER message is sent to the netdev notifier
 *	chain. 0 is returned on success. A negative errno code is returned
 *	on a failure to set up the device, or if the name is a duplicate.
 *
 *	Callers must hold the rtnl semaphore.  See the comment at the
 *	end of Space.c for details about the locking.  You may want
 *	register_netdev() instead of this.
 *
 *	BUGS:
 *	The locking appears insufficient to guarantee two parallel registers
 *	will not get the same name.
 */

int net_dev_init(void);

int register_netdevice(struct net_device *dev)
{
	struct net_device *d, **dp;
#ifdef CONFIG_NET_DIVERT
	int ret;
#endif

	spin_lock_init(&dev->queue_lock);
	spin_lock_init(&dev->xmit_lock);
	dev->xmit_lock_owner = -1;
#ifdef CONFIG_NET_FASTROUTE
	dev->fastpath_lock=RW_LOCK_UNLOCKED;
#endif

	if (dev_boot_phase)
		net_dev_init();

#ifdef CONFIG_NET_DIVERT
	ret = alloc_divert_blk(dev);
	if (ret)
		return ret;
#endif /* CONFIG_NET_DIVERT */
	
	dev->iflink = -1;

	/* Init, if this function is available */
	if (dev->init && dev->init(dev) != 0) {
#ifdef CONFIG_NET_DIVERT
		free_divert_blk(dev);
#endif
		return -EIO;
	}

#ifdef CONFIG_NET_SCH_PRIOWRR
	dev->unused_alignment_fixer = (unsigned short) dev->mtu;
	printk("Default Asymmetric MTU for %s %d\n", dev->name, dev->unused_alignment_fixer);
#endif /* CONFIG_NET_SCH_PRIOWRR */

	dev->ifindex = dev_new_index();
	if (dev->iflink == -1)
		dev->iflink = dev->ifindex;

	/* Check for existence, and append to tail of chain */
	for (dp=&dev_base; (d=*dp) != NULL; dp=&d->next) {
		if (d == dev || strcmp(d->name, dev->name) == 0) {
#ifdef CONFIG_NET_DIVERT
			free_divert_blk(dev);
#endif
			return -EEXIST;
		}
	}
	/*
	 *	nil rebuild_header routine,
	 *	that should be never called and used as just bug trap.
	 */

	if (dev->rebuild_header == NULL)
		dev->rebuild_header = default_rebuild_header;

	/*
	 *	Default initial state at registry is that the
	 *	device is present.
	 */

	set_bit(__LINK_STATE_PRESENT, &dev->state);

	dev->next = NULL;
	dev_init_scheduler(dev);
	write_lock_bh(&dev_base_lock);
	*dp = dev;
	dev_hold(dev);
	dev->deadbeaf = 0;
	write_unlock_bh(&dev_base_lock);

    /* NSP QoS Framework: Initialize the TCA for the device. */
    if (ti_tca_device_init(dev) < 0)
        return -1;

	/* Notify protocols, that a new device appeared. */
	notifier_call_chain(&netdev_chain, NETDEV_REGISTER, dev);

	net_run_sbin_hotplug(dev, "register");

	return 0;
}

/**
 *	netdev_finish_unregister - complete unregistration
 *	@dev: device
 *
 *	Destroy and free a dead device. A value of zero is returned on
 *	success.
 */
 
int netdev_finish_unregister(struct net_device *dev)
{
	BUG_TRAP(dev->ip_ptr==NULL);
	BUG_TRAP(dev->ip6_ptr==NULL);
	BUG_TRAP(dev->dn_ptr==NULL);

	if (!dev->deadbeaf) {
		printk(KERN_ERR "Freeing alive device %p, %s\n", dev, dev->name);
		return 0;
	}
#ifdef NET_REFCNT_DEBUG
	printk(KERN_DEBUG "netdev_finish_unregister: %s%s.\n", dev->name,
	       (dev->features & NETIF_F_DYNALLOC)?"":", old style");
#endif
	if (dev->destructor)
		dev->destructor(dev);
	if (dev->features & NETIF_F_DYNALLOC)
		kfree(dev);
	return 0;
}

/**
 *	unregister_netdevice - remove device from the kernel
 *	@dev: device
 *
 *	This function shuts down a device interface and removes it
 *	from the kernel tables. On success 0 is returned, on a failure
 *	a negative errno code is returned.
 *
 *	Callers must hold the rtnl semaphore.  See the comment at the
 *	end of Space.c for details about the locking.  You may want
 *	unregister_netdev() instead of this.
 */

int unregister_netdevice(struct net_device *dev)
{
	unsigned long now, warning_time;
	struct net_device *d, **dp;

	/* If device is running, close it first. */
	if (dev->flags & IFF_UP)
		dev_close(dev);

	BUG_TRAP(dev->deadbeaf==0);
	dev->deadbeaf = 1;

	/* And unlink it from device chain. */
	for (dp = &dev_base; (d=*dp) != NULL; dp=&d->next) {
		if (d == dev) {
			write_lock_bh(&dev_base_lock);
			*dp = d->next;
			write_unlock_bh(&dev_base_lock);
			break;
		}
	}
	if (d == NULL) {
		printk(KERN_DEBUG "unregister_netdevice: device %s/%p never was registered\n", dev->name, dev);
		return -ENODEV;
	}

	/* Synchronize to net_rx_action. */
	br_write_lock_bh(BR_NETPROTO_LOCK);
	br_write_unlock_bh(BR_NETPROTO_LOCK);

	if (dev_boot_phase == 0) {
#ifdef CONFIG_NET_FASTROUTE
		dev_clear_fastroute(dev);
#endif

		/* Shutdown queueing discipline. */
		dev_shutdown(dev);

		net_run_sbin_hotplug(dev, "unregister");

		/* Notify protocols, that we are about to destroy
		   this device. They should clean all the things.
		 */
		notifier_call_chain(&netdev_chain, NETDEV_UNREGISTER, dev);

		/*
		 *	Flush the multicast chain
		 */
		dev_mc_discard(dev);
	}

	if (dev->uninit)
		dev->uninit(dev);

	/* Notifier chain MUST detach us from master device. */
	BUG_TRAP(dev->master==NULL);

#ifdef CONFIG_NET_DIVERT
	free_divert_blk(dev);
#endif

    /* NSP QoS Framework: The device is being removed. Clean the TCA */
    if (ti_tca_device_deinit(dev) < 0)
        return -1;

	if (dev->features & NETIF_F_DYNALLOC) {
#ifdef NET_REFCNT_DEBUG
		if (atomic_read(&dev->refcnt) != 1)
			printk(KERN_DEBUG "unregister_netdevice: holding %s refcnt=%d\n", dev->name, atomic_read(&dev->refcnt)-1);
#endif
		dev_put(dev);
		return 0;
	}

	/* Last reference is our one */
	if (atomic_read(&dev->refcnt) == 1) {
		dev_put(dev);
		return 0;
	}

#ifdef NET_REFCNT_DEBUG
	printk("unregister_netdevice: waiting %s refcnt=%d\n", dev->name, atomic_read(&dev->refcnt));
#endif

	/* EXPLANATION. If dev->refcnt is not now 1 (our own reference)
	   it means that someone in the kernel still has a reference
	   to this device and we cannot release it.

	   "New style" devices have destructors, hence we can return from this
	   function and destructor will do all the work later.  As of kernel 2.4.0
	   there are very few "New Style" devices.

	   "Old style" devices expect that the device is free of any references
	   upon exit from this function.
	   We cannot return from this function until all such references have
	   fallen away.  This is because the caller of this function will probably
	   immediately kfree(*dev) and then be unloaded via sys_delete_module.

	   So, we linger until all references fall away.  The duration of the
	   linger is basically unbounded! It is driven by, for example, the
	   current setting of sysctl_ipfrag_time.

	   After 1 second, we start to rebroadcast unregister notifications
	   in hope that careless clients will release the device.

	 */

	now = warning_time = jiffies;
	while (atomic_read(&dev->refcnt) != 1) {
		if ((jiffies - now) > 1*HZ) {
			/* Rebroadcast unregister notification */
			notifier_call_chain(&netdev_chain, NETDEV_UNREGISTER, dev);
		}
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(HZ/4);
		current->state = TASK_RUNNING;
		if ((jiffies - warning_time) > 10*HZ) {
			printk(KERN_EMERG "unregister_netdevice: waiting for %s to "
					"become free. Usage count = %d\n",
					dev->name, atomic_read(&dev->refcnt));
			warning_time = jiffies;
		}
	}
	dev_put(dev);
	return 0;
}


/*
 *	Initialize the DEV module. At boot time this walks the device list and
 *	unhooks any devices that fail to initialise (normally hardware not 
 *	present) and leaves us with a valid list of present and active devices.
 *
 */

extern void net_device_init(void);
extern void ip_auto_config(void);
#ifdef CONFIG_NET_DIVERT
extern void dv_init(void);
#endif /* CONFIG_NET_DIVERT */

static int voice_write_level (struct file *file, const char *buffer, unsigned long count, void *data) 
{
	char voicestate[5];

	/* Validate the length of data passed. */
	if (count > 5)
		count = 5; 
	
	/* Copy from user space. */	
	if (copy_from_user (&voicestate, buffer, count))
		return -EFAULT;

	/* Store the new debug level. */
	gIsVoiceEnabled = (int) simple_strtol(voicestate, NULL, 0);
	return count;
}

static int voice_read_level (char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
	int len = 0;
	len = len + sprintf (buf + len, "Currently Voice Mode is %d.\n",gIsVoiceEnabled); 
	return len;
}

/*
 *       Callers must hold the rtnl semaphore.  See the comment at the
 *       end of Space.c for details about the locking.
 */
int __init net_dev_init(void)
{
	struct net_device *dev, **dp;
	int i;
    struct proc_dir_entry*	ptr_dir_entry;

	if (!dev_boot_phase)
		return 0;

#ifdef CONFIG_NET_DIVERT
	dv_init();
#endif /* CONFIG_NET_DIVERT */
	
	/*
	 *	Initialise the packet receive queues.
	 */

	for (i = 0; i < NR_CPUS; i++) {
		struct softnet_data *queue;

		queue = &softnet_data[i];
		skb_queue_head_init(&queue->input_pkt_queue);
		queue->throttle = 0;
		queue->cng_level = 0;
		queue->avg_blog = 10; /* arbitrary non-zero */
		queue->completion_queue = NULL;
	}
	
#ifdef CONFIG_NET_PROFILE
	net_profile_init();
	NET_PROFILE_REGISTER(dev_queue_xmit);
	NET_PROFILE_REGISTER(softnet_process);
#endif

#ifdef OFFLINE_SAMPLE
	samp_timer.expires = jiffies + (10 * HZ);
	add_timer(&samp_timer);
#endif

	/*
	 *	Add the devices.
	 *	If the call to dev->init fails, the dev is removed
	 *	from the chain disconnecting the device until the
	 *	next reboot.
	 *
	 *	NB At boot phase networking is dead. No locking is required.
	 *	But we still preserve dev_base_lock for sanity.
	 */

	dp = &dev_base;
	while ((dev = *dp) != NULL) {
		spin_lock_init(&dev->queue_lock);
		spin_lock_init(&dev->xmit_lock);
#ifdef CONFIG_NET_FASTROUTE
		dev->fastpath_lock = RW_LOCK_UNLOCKED;
#endif
		dev->xmit_lock_owner = -1;
		dev->iflink = -1;
		dev_hold(dev);

		/*
		 * Allocate name. If the init() fails
		 * the name will be reissued correctly.
		 */
		if (strchr(dev->name, '%'))
			dev_alloc_name(dev, dev->name);

		/* 
		 * Check boot time settings for the device.
		 */
		netdev_boot_setup_check(dev);

		if (dev->init && dev->init(dev)) {
			/*
			 * It failed to come up. It will be unhooked later.
			 * dev_alloc_name can now advance to next suitable
			 * name that is checked next.
			 */
			dev->deadbeaf = 1;
			dp = &dev->next;
		} else {
			dp = &dev->next;
			dev->ifindex = dev_new_index();
			if (dev->iflink == -1)
				dev->iflink = dev->ifindex;
			if (dev->rebuild_header == NULL)
				dev->rebuild_header = default_rebuild_header;
			dev_init_scheduler(dev);
			set_bit(__LINK_STATE_PRESENT, &dev->state);
		}
	}

	/*
	 * Unhook devices that failed to come up
	 */
	dp = &dev_base;
	while ((dev = *dp) != NULL) {
		if (dev->deadbeaf) {
			write_lock_bh(&dev_base_lock);
			*dp = dev->next;
			write_unlock_bh(&dev_base_lock);
			dev_put(dev);
		} else {
			dp = &dev->next;
		}
	}

#ifdef CONFIG_PROC_FS
	proc_net_create("dev", 0, dev_get_info);
	create_proc_read_entry("net/softnet_stat", 0, 0, dev_proc_stats, NULL);
#ifdef WIRELESS_EXT
	/* Available in net/core/wireless.c */
	proc_net_create("wireless", 0, dev_get_wireless_info);
#endif	/* WIRELESS_EXT */
#endif	/* CONFIG_PROC_FS */

	dev_boot_phase = 0;

	open_softirq(NET_TX_SOFTIRQ, net_tx_action, NULL);
	open_softirq(NET_RX_SOFTIRQ, net_rx_action, NULL);

	dst_init();
	dev_mcast_init();

#ifdef CONFIG_NET_SCHED
	pktsched_init();
#endif

    /* PANKAJ: Create a PROC entry for VOICE mode. This could be made board
     * specific. Added here */

	/* Create a proc entry that is used to enable / disable the firewall. */
	ptr_dir_entry = create_proc_entry("net/voice" ,0644, NULL);
	if (ptr_dir_entry == NULL)
	{
		printk ("Error: Unable to create the voice proc entry.\n");
		return -1;
	}
	ptr_dir_entry->data 	  = NULL;
	ptr_dir_entry->read_proc  = voice_read_level;
	ptr_dir_entry->write_proc = voice_write_level;
	ptr_dir_entry->owner 	  = THIS_MODULE;    

    /* NSP QoS Initialization. */
    ti_nsp_qos_initialize();

	/*
	 *	Initialise network devices
	 */
	 
	net_device_init();

	return 0;
}

#ifdef CONFIG_HOTPLUG

/* Notify userspace when a netdevice event occurs,
 * by running '/sbin/hotplug net' with certain
 * environment variables set.
 */

static int net_run_sbin_hotplug(struct net_device *dev, char *action)
{
	char *argv[3], *envp[5], ifname[12 + IFNAMSIZ], action_str[32];
	int i;

	sprintf(ifname, "INTERFACE=%s", dev->name);
	sprintf(action_str, "ACTION=%s", action);

        i = 0;
        argv[i++] = hotplug_path;
        argv[i++] = "net";
        argv[i] = 0;

	i = 0;
	/* minimal command environment */
	envp [i++] = "HOME=/";
	envp [i++] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
	envp [i++] = ifname;
	envp [i++] = action_str;
	envp [i] = 0;
	
	return call_usermodehelper(argv [0], argv, envp);
}
#endif

/****************************************************************************
 ****************************************************************************
 ******************************* NSP QoS Framework **************************
 ****************************************************************************
 ****************************************************************************/

/**************************** INCLUDE FILES *************************/

#include <linux/udp.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>

/**************************** GLOBAL VARIABLES *************************/

/* This is the master control block for the NSP QoS framework. It contains all
 * the information about the individual pieces that comprise the QoS framework.*/
TI_QOS_MCB*             ptr_ti_qos_mcb;

/* Global Varibale; which controls the QoS Framework. */
int gIsQoSEnabled = 1;

/**************************** EXTERN DECLARATIONS *************************/

extern int get_tuple(const struct iphdr *iph, size_t len,
                     struct ip_conntrack_tuple *tuple,
	                 struct ip_conntrack_protocol *protocol);
extern struct ip_conntrack_tuple_hash *ip_conntrack_find_get(const struct ip_conntrack_tuple *tuple,
		                                                     const struct ip_conntrack *ignored_conntrack);
extern struct ip_conntrack_protocol *ip_ct_find_proto(u_int8_t protocol);

/**************************** STATIC FUNCTIONS *************************/

/* The PIL Classifier which handles VLAN Packets. */
static unsigned short ti_pil_handle_vlan_packet (char **ptr_data, 
                                                 TI_QOS_PACKET_INFO* ptr_ti_qos_info,
                                                 TI_QOS_TCA*    ptr_ti_qos_tca);

/**************************** QoS Framework FUNCTIONS ******************/

/**************************************************************************
 * FUNCTION NAME : ti_qos_register_local_port
 **************************************************************************
 * DESCRIPTION   :
 *  The function registers a protocol port with the QoS Local Database.
 *
 * RETURNS       :
 *      0   - Success
 *      <0  - Error
 ***************************************************************************/
int ti_qos_register_local_port (unsigned int port)
{
    struct list_head*   ptr_list_head;
    TI_QOS_LOCAL_PORT*  ptr_local_port;

    printk ("DEBUG: Registering port %d.\n", ntohs(port));

    /* Cycle through the list of voice ports to see if the current port has
     * already been registered. */
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->active_port_list)
    {
        ptr_local_port = (TI_QOS_LOCAL_PORT *)ptr_list_head;

        /* Do we have a match. */
        if (ptr_local_port->port == port)
            return -1;
    }

    /* Allocate a voice port block from the available list. But if the list is
     * empty there are none available.*/
    if (list_empty((struct list_head *)&ptr_ti_qos_mcb->ptr_local_database->available_port_list))
    {
        printk ("DEBUG: No more local port nodes available. \n");
        return -1;
    }

    /* Dequeue the head from the list. */
    ptr_list_head  = ((struct list_head*)&ptr_ti_qos_mcb->ptr_local_database->available_port_list)->next;
    list_del ((struct list_head *)ptr_list_head );

    /* Initialize the port structure. */
    ptr_local_port = (TI_QOS_LOCAL_PORT *)ptr_list_head;
    ptr_local_port->port = port;

    /* Add to the active port list. */
    list_add ((struct list_head *)ptr_local_port, (struct list_head *)&ptr_ti_qos_mcb->ptr_local_database->active_port_list);

#if 0
    /* Debug Work. */
    printk (" -------------------------------------------------------------- \n");
    printk (" Active Port List.\n");
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->active_port_list)
    {
        printk (" 0x%p.\n", ptr_list_head);
    }
    printk (" -------------------------------------------------------------- \n");
    printk (" Available Port List.\n");
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->available_port_list)
    {
        printk (" 0x%p.\n", ptr_list_head);
    }
    printk (" -------------------------------------------------------------- \n");
#endif

    return 0;
}

/**************************************************************************
 * FUNCTION NAME : ti_qos_deregister_local_port
 **************************************************************************
 * DESCRIPTION   :
 *  The function deregisters a local port associated from the QoS local 
 *  database.
 *
 * RETURNS       :
 *      0   - Success
 *      <0  - Error
 ***************************************************************************/
int ti_qos_deregister_local_port (int port)
{
    struct list_head*   ptr_list_head;
    TI_QOS_LOCAL_PORT*  ptr_local_port = NULL;

    printk ("DEBUG: Deregistering port %d.\n", ntohs(port));

    /* Cycle through the list of voice ports to see if the current port has
     * already been registered. */
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->active_port_list)
    {
        ptr_local_port = (TI_QOS_LOCAL_PORT *)ptr_list_head;

        /* Do we have a match. */
        if (ptr_local_port->port == port)
            break;
    }

    /* Did we find a match. */
    if (ptr_local_port == NULL)
    {
        printk ("Error: No voice port information.\n");
        return -1;
    }

    /* Delete the node from the ACTIVE Port List. */
    list_del ((struct list_head *)ptr_local_port);

    /* Initialize the port structure. */
    ptr_local_port->port = 0xFFFFFF;

    /* Add to the available port list. */
    list_add ((struct list_head *)ptr_local_port, (struct list_head *)&ptr_ti_qos_mcb->ptr_local_database->available_port_list);

#if 0
    /* Debug Work. */
    printk (" -------------------------------------------------------------- \n");
    printk (" Active Port List.\n");
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->active_port_list)
    {
        printk (" 0x%p.\n", ptr_list_head);
    }
    printk (" -------------------------------------------------------------- \n");
    printk (" Available Port List.\n");
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->available_port_list)
    {
        printk (" 0x%p.\n", ptr_list_head);
    }
    printk (" -------------------------------------------------------------- \n");
#endif
    return 0;

}

/**************************************************************************
 * FUNCTION NAME : ti_qos_local_port_match
 **************************************************************************
 * DESCRIPTION   :
 *  The function searches the QoS Local Database for a match.
 *
 * RETURNS       :
 *      0   - No match found.
 *      1   - Match found.
 ***************************************************************************/
int ti_qos_local_port_match (unsigned int port)
{
    struct list_head*   ptr_list_head;
    TI_QOS_LOCAL_PORT*  ptr_local_port = NULL;

    /* Cycle through the list of voice ports to see if the current port has
     * already been registered. */
    list_for_each(ptr_list_head, &ptr_ti_qos_mcb->ptr_local_database->active_port_list)
    {
        /* Get the voice port information. */
        ptr_local_port = (TI_QOS_LOCAL_PORT *)ptr_list_head;

        /* Do we have a match. */
        if (ptr_local_port->port == port)
            return 1;
    }

    /* No match found. */
    return 0;
}

/**************************************************************************
 * FUNCTION NAME : ti_qos_lookup_payload_database
 **************************************************************************
 * DESCRIPTION   :
 *  The function is the interface between the QoS Framework and the 
 *  connection tracking database. The function uses the IP header to query
 *  the connection tracking database. It then returns the class of service
 *  associated with the connection tracking entry.
 *
 * RETURNS       :
 *  Class of Service associated with connection
 *  CoS6 if no match is found.
 ***************************************************************************/
TI_NSP_QOS_CLASS ti_qos_lookup_payload_database(struct sk_buff* skb, struct iphdr* ptr_iphdr)
{
#ifdef CONFIG_IP_NF_CONNTRACK
    struct ip_conntrack_tuple       tuple;
    struct ip_conntrack_protocol*   proto;
    TI_QOS_PACKET_INFO*             ptr_ti_qos_info;

    /* Get the pointer to the QoS Information field. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

	/* Check if the packet is FRAGMENTED?  
     *  "Fragmentation is like classful addressing -- an interesting early
     *  architectural error that shows how much experimentation was going
     *  on while IP was being designed."                    
     *          -- Paul Vixie */
	if (ptr_iphdr->frag_off & htons(IP_MF|IP_OFFSET)) 
    {
        /* We do not have enough information for packet classification. We push the
         * packet through irrespective of the congestion state; giving the packet an
         * oppurtunity to survive. */
        ptr_ti_qos_mcb->fragmented_pkts++;
        ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_FRAGMENTED;
		return TI_NSP_QOS_COS6;
    }

    /* The "Local" database is searched only for TCP and UDP packets. */
    if ((ptr_iphdr->protocol == 17) || (ptr_iphdr->protocol == 6))
    {
        /* Get the pointer to the UDP/TCP header. The port information we want to access is at
         * the same location for both protocols. */
        struct udphdr *ptr_udpheader = (struct udphdr *) ((char *)ptr_iphdr + ptr_iphdr->ihl*4);

        /* Check the "LOCAL Database" */
        if (ti_qos_local_port_match(ptr_udpheader->dest) == 1)
        {
            /* YES. Found a hit mark this packet as HIGH Priority. */
            return TI_NSP_QOS_COS1;
        }
    }

    /* Find the protocol. */
    proto = ip_ct_find_proto(ptr_iphdr->protocol);

    /* Get the Tuple. */
	if (!get_tuple(ptr_iphdr, skb->len, &tuple, proto))
    {
        /* Error: Tuple extraction failed. This means that the database lookup will also fail. 
         * since now we can not look into the connection tracking database. */
        ptr_ti_qos_mcb->classification_error++;
		return TI_NSP_QOS_COS6;
    }

    /* If we have reached this stage then we are relying on the connection tracking table to 
     * have the priority. Whatever we put inside the packet now should not override the connection 
     * tracking priority? This is required to SYNC the conntrack database priority with the QoS 
     * Framework. Setting the FLAG will ensure that the priority in the MANGLE is NOT overriden. */
    ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_LIE;

    /* Check the connection tracking database. */
    ptr_ti_qos_info->ti_tuple_hash = ip_conntrack_find_get(&tuple, NULL);
    if (ptr_ti_qos_info->ti_tuple_hash != NULL)
    {
        TI_NSP_QOS_CLASS    class_of_service;

        /* Get the class of service associated with the connection. */
        class_of_service = (TI_NSP_QOS_CLASS)ptr_ti_qos_info->ti_tuple_hash->ctrack->conntrack_priority;

        /* Release the counter that we hold on the connection tracking entry. */
        atomic_dec(&ptr_ti_qos_info->ti_tuple_hash->ctrack->ct_general.use);

        /* Initialize the connection tracking information inside the QoS framework. */
        ptr_ti_qos_info->ti_connection_proto = proto;

        /* Return the class of service. */
        return class_of_service;
    }
#endif
    /* There was no match found. Default to lowest level of service. */
    return TI_NSP_QOS_COS6;
}

/**************************************************************************
 * FUNCTION NAME : ti_qos_layer2_translation_unit
 **************************************************************************
 * DESCRIPTION   :
 *  This is the hook for the Layer 2 NSP QoS Framework "Translation Unit" 
 *  which uses the TCA to determine the layer2 user priority bit to be 
 *  configured in the packet.
 *
 * RETURNS       :
 *  -1  --> If there is no TCA associated / QoS is Disabled.
 *  Else User Priority value to be set in the packet as per the TCA
 ***************************************************************************/
int ti_qos_layer2_translation_unit (struct sk_buff *skb)
{
    TI_QOS_TCA* ptr_ti_qos_tca;

    /* Get the TCA associated with the device. */
    ptr_ti_qos_tca = (TI_QOS_TCA *)skb->dev->ptr_ti_qos_tca;

    /* Check if the QoS is enabled. Then use the TCA to decide what priority to place in the packet. */
    if ((gIsQoSEnabled == 1) && (ptr_ti_qos_tca != NULL) && (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_EGRESS_LAYER2)) 
    {
        TI_QOS_PACKET_INFO* ptr_ti_qos_info;
        int                 user_priority;

        /* Get the pointer to the QoS Packet Information. */
        ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

        /* Check if the packet has been classified. */
        if (!(ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_CLASSIFIED))
        {
            /* NO. The packet was NOT classified. This could be a locally generated packet 
             * or some interface is NOT following the framework requirements.Use the TCA to 
             * default to the well-defined Class of Service. */
            ptr_ti_qos_info->ti_service_class = ptr_ti_qos_tca->default_egress_service_class;
            ptr_ti_qos_mcb->ptr_tu_handle->unclassified_layer2_pkts++;
        }

        /* Get the user priority value from the egress layer2 mappings */
        user_priority = ptr_ti_qos_tca->egress_layer2_mapping[ptr_ti_qos_info->ti_service_class];

        /* Make sure the packet is tagged indicating that the packet has been passed through 
         * the translation unit. */
        ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_TRANSLATED;

        /* Return the user priority that needs to be set in the packet. */
        return user_priority;
    }

    /* There was no TCA or the QoS was disabled. Cant do anything here! */
    return -1;
}

/**************************************************************************
 * FUNCTION NAME : ti_qos_translation_unit
 **************************************************************************
 * DESCRIPTION   :
 *  This is the hook for the Layer 3 NSP QoS Framework "Translation Unit" 
 *  which is responsible for marking TOS byte in the IP packet before passing 
 *  the packet to the QoS Scheduler. It also remarks the Mapped service level 
 *  so that when the packet is passed down to the QoS scheduler it knows what 
 *  category the packet falls into.
 ***************************************************************************/
void ti_qos_translation_unit(struct sk_buff *skb, struct net_device* dev)
{
    TI_QOS_PACKET_INFO* ptr_ti_qos_info;
    TI_QOS_TCA*         ptr_ti_qos_tca;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_TU_ENTER);

    /* Get the pointer to the QoS Information field. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* Get the TCA associated with the device. */
    ptr_ti_qos_tca = (TI_QOS_TCA *)dev->ptr_ti_qos_tca;

    /* Check if there was no TCA associated with the device or if the EGRESS 
     * Layer 3 mappings were NOT configured. These need to specified for the translation 
     * unit to operate. */
    if ((ptr_ti_qos_tca != NULL) && (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_EGRESS_LAYER3))
    {
        struct iphdr* ptr_ipheader;

        /* Get the pointer to the IP header. */
        if (skb->protocol == __constant_htons(ETH_P_IP))
            ptr_ipheader = (struct iphdr *)skb->nh.iph;
        else
            ptr_ipheader = NULL;

        /* Check if the packet is an IP Packet. */
        if (ptr_ipheader)
        {
            unsigned char new_tos;

            /* Check if the packet has been classified. */
            if (!(ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_CLASSIFIED))
            {
                /* NO. The packet was NOT classified. This could be a locally generated packet 
                 * or some interface is NOT following the framework requirements.Use the TCA to 
                 * default to the well-defined Class of Service. */
                ptr_ti_qos_info->ti_service_class = ptr_ti_qos_tca->default_ip_egress_service_class;
                ptr_ti_qos_mcb->ptr_tu_handle->unclassified_ip_pkts++;
            }

            /* Get the new TOS value using the TCA that needs to be configured in the packet. */
            new_tos = ptr_ti_qos_tca->egress_layer3_mapping[ptr_ti_qos_info->ti_service_class];

            /* Change the DS Field inside the packet only if required to do so.
             * This means that the new and old values should be different. */
            if (ptr_ipheader->tos != new_tos)
            {               
                /* Change the DS Field to the new value. */
                ipv4_change_dsfield (ptr_ipheader, 0, new_tos);

                /* Increment the statistics for packets that have been remarked. */
                ptr_ti_qos_mcb->ptr_tu_handle->tos_byte_remarked++;
            }
            else
            {
                /* This implies that the new DSCP was the same as the old one. In this
                 * case we dont need to do anything but increment statistics for the 
                 * number of packets that have gone unchanged. */
                ptr_ti_qos_mcb->ptr_tu_handle->pkts_unchanged++;
            }
        }
        else
        {
            /* Non IP Packet Received. Check if the packet has been classified. */
            if (!(ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_CLASSIFIED))
            {
                /* NO. Use the TCA to default to the service class. */
                ptr_ti_qos_info->ti_service_class = ptr_ti_qos_tca->default_non_ip_egress_service_class;
                ptr_ti_qos_mcb->ptr_tu_handle->unclassified_non_ip_pkts++;
            }
            else
            {
                /* Increment the statistics for NON DSCP Packets that have been translated. */
                ptr_ti_qos_mcb->ptr_tu_handle->non_ip_pkts++;
            }
        }

        /* Make sure the packet is tagged indicating that the packet has been passed through 
         * the translation unit. */
        ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_TRANSLATED;
    }

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_TU_EXIT);
    return;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_rr_update_congestion_state 
 **************************************************************************
 * DESCRIPTION   :
 *  The function is called from the NQM "Enqueue" and "Dequeue" functions
 *  to update the congestion present in the System. The function updates the
 *  internal structures used by the "Round Robin" NQM scheduler. The 
 *  parameter "set_congestion" is passed as 1 indicating that the congestion
 *  was detected and is passed as 0 indicating that there congestion has 
 *  passed.
 *
 * NOTES         :
 *  This function is called with interrupts disabled since shared variables
 *  are being updated.
 ***************************************************************************/
static void ti_nqm_rr_update_congestion_state 
(
    TI_QOS_NQM_RR*      ptr_ti_nqm_rr_mcb, 
    TI_NSP_QOS_CLASS    service_class, 
    int                 set_congestion
)
{
    /* Depending on the flag. Set/Reset the appropriate bit indicating congestion/non-congestion. */
    if (set_congestion == 1)
        ptr_ti_nqm_rr_mcb->nsp_queue_state = ptr_ti_nqm_rr_mcb->nsp_queue_state | (1<<service_class);
    else
        ptr_ti_nqm_rr_mcb->nsp_queue_state = ptr_ti_nqm_rr_mcb->nsp_queue_state & ~(1<<service_class);

    /* The bit set in the NSP queue state decides the congestion level. */
    if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS1))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS1;
    else if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS2))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS2;
    else if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS3))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS3;
    else if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS4))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS4;
    else if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS5))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS5;
    else if (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<TI_NSP_QOS_COS6))
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_DROP_COS6;
    else
        ptr_ti_nqm_rr_mcb->nsp_congestion_level = TI_QOS_NQM_NOT_CONGESTED;

    /* The work is completed. */
    return;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_rr_congestion_check
 **************************************************************************
 * DESCRIPTION   :
 *  The function checks the congestion present in the System. This is 
 *  the Round-Robin implementation of the algorithm.
 *
 * RETURNS       :
 *  1   - Accept the packet
 *  0   - Drop the packet.
 ***************************************************************************/
int ti_nqm_rr_congestion_check (struct sk_buff* skb, TI_NSP_QOS_CLASS service_class)
{
    int             congestion_level;
    unsigned long   flags;
    int             ret_val;
    TI_QOS_NQM_RR*  ptr_ti_nqm_rr_mcb;
    TI_QOS_PACKET_INFO*     ptr_ti_qos_info;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_CONGESTION_ENTER);

    /* Get the pointer to the Private Information. */
    ptr_ti_nqm_rr_mcb = (TI_QOS_NQM_RR *)ptr_ti_qos_mcb->ptr_nqm_handle->ptr_nqm_private_info;

    /* Get the TI QoS information related to the packet. */ 
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* If the packet is fragmented; that implies there was not enough information for the 
     * classification to fully succeed. This might be a high priority packet allow it 
     * through the NQM and give it a chance to survive. */
    if (ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_FRAGMENTED)
        return 1;

    /* By default: We accept all packets until told do otherwise. */
    ret_val = 1;

    /* Critical Section Start: Disable Interrupts. */    
    local_irq_save(flags);

    /* Get the congestion level from the Network Queue Manager and copy it to
     * a local variable. This needs to be protected. */
    congestion_level = ptr_ti_nqm_rr_mcb->nsp_congestion_level;

    /* Critical Section End: Enable Interrupts. */
    local_irq_restore(flags);

    /* The Packet is processed depending on the congestion level. */
    switch (congestion_level)
    {
        case TI_QOS_NQM_NOT_CONGESTED:
        {
            /* The System is NOT congested. Accept all packets. */
            break;
        }
        case TI_QOS_NQM_DROP_COS6:
        {
            /* Drop all COS6 packets. */
            if (service_class >= TI_NSP_QOS_COS6)
                ret_val = 0;
            break;
        }
        case TI_QOS_NQM_DROP_COS5:
        {
            /* Drop all CoS5 and CoS6 packets. */
            if (service_class >= TI_NSP_QOS_COS5)
                ret_val = 0;
            break;
        }
        case TI_QOS_NQM_DROP_COS4:
        {
            /* Drop all CoS4, CoS5 and CoS6 packets. */
            if (service_class >= TI_NSP_QOS_COS4)
                ret_val = 0;
            break;
        }
        case TI_QOS_NQM_DROP_COS3:
        {
            /* Drop all CoS3, CoS4, CoS5 and CoS6 packets. */
            if (service_class >= TI_NSP_QOS_COS3)
                ret_val = 0;
            break;
        }
        case TI_QOS_NQM_DROP_COS2:
        {
            /* Drop all CoS2, CoS3, CoS4, CoS5 and CoS6 packets. */
            if (service_class >= TI_NSP_QOS_COS2)
                ret_val = 0;
            break;
        }
        default:
        {
            /* System is so congested that even CoS1 packets need to be
             * dropped. This implies that the System is deadlocked. Nothing can be done
             * now. So drop all other packets. */
            ret_val = 0;
        }
    }

    /* Check if the packet will be dropped and if so increment the statistics. */
    if (ret_val == 0)
        ptr_ti_nqm_rr_mcb->nsp_dropped_packets[service_class]++;

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_CONGESTION_EXIT);

    /* Return the packet acceptance level. */
    return ret_val;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_rr_enqueue_packet
 **************************************************************************
 * DESCRIPTION   :
 *  The function enqueues a packet to the Network Queue Manager. This is the
 *  registered routine for the Strict Round Robin Scheduler.
 *
 * RETURNS       :
 *  Congestion Level in the System.
 ***************************************************************************/
int ti_nqm_rr_enqueue_packet (struct sk_buff* skb)
{
    TI_QOS_PACKET_INFO*     ptr_ti_qos_info;
    struct sk_buff_head*    queue;
    unsigned long           flags;
    TI_QOS_NQM_RR*          ptr_ti_nqm_rr_mcb;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_ENQUEUE_ENTER);

    /* Get the pointer to the Private Information. */
    ptr_ti_nqm_rr_mcb = (TI_QOS_NQM_RR *)ptr_ti_qos_mcb->ptr_nqm_handle->ptr_nqm_private_info;

    /* Get the TI QoS information related to the packet. */ 
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* Sanity Check: Before we enqueue the packet to the Network Queue Manager. Verify that the
     * packet was processed by the PIL layer and appropriately marked. */
    if ((ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_CLASSIFIED) == 0)
    {
        /* This should NOT happen and implies that the drivers have not called the
         * protocol inspection. Default all such packets to the best-effort queue. */
        ptr_ti_qos_mcb->unclassified_pkts++;
        ptr_ti_qos_info->ti_service_class = TI_NSP_QOS_COS6;
        ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
    }

    /* Critical Section Start: Disable Interrupts. */    
    local_irq_save(flags);

    /* Select the correct queue from the NSP Service level. */
    queue = &ptr_ti_nqm_rr_mcb->nsp_pkt_queue[ptr_ti_qos_info->ti_service_class];

    /* Check the length of the queue and enqueue the packet only if allowed to do so. */
    if (queue->qlen > ptr_ti_nqm_rr_mcb->nsp_threshold[ptr_ti_qos_info->ti_service_class])
    {
        /* One of the queues in the System has exceeded the threshold. Its
         * time to update the congestion levels. */
        ti_nqm_rr_update_congestion_state (ptr_ti_nqm_rr_mcb, ptr_ti_qos_info->ti_service_class, 1);

        /* This packet has been dropped. Increment the statistics and clean the memory. */
        ptr_ti_nqm_rr_mcb->nsp_dropped_packets[ptr_ti_qos_info->ti_service_class]++;
        net_error_handler(skb);
    }
    else
    {
        /* The queue has space enqueue the packet into the appropriate queue. 
         * Hold the device counter so that the device cannot be deleted. */
        dev_hold(skb->dev);
    	__skb_queue_tail(queue, skb);

        /* Increment the statistics block in the Network Queue Manager. */
        ptr_ti_nqm_rr_mcb->nsp_receive_stats[ptr_ti_qos_info->ti_service_class]++;
    }

    /* Wake up the network BH for execution. */
    cpu_raise_softirq(smp_processor_id(), NET_RX_SOFTIRQ);

    /* Critical Section End: Enable Interrupts. */
    local_irq_restore(flags);

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_ENQUEUE_EXIT);

    /* Return the congestion level back to the drivers. */
    return ptr_ti_nqm_rr_mcb->nsp_congestion_level;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_netif_rx
 **************************************************************************
 * DESCRIPTION   :
 *	This function receives a packet from a device driver and queues it for
 *	the upper (protocol) levels to process.  It always succeeds. The buffer
 *	may be dropped during processing for congestion control or by the 
 *	protocol layers.
 *
 * RETURNS       :
 *  Always returns 0
 ***************************************************************************/
static int ti_nqm_netif_rx(struct sk_buff *skb)
{
    /* PANKAJ TODO: Removing the bridge to run in the context of the callee. This 
     * will not work with the QoS Framework hooks because this way the bridged
     * interfaces indirectly have highest priority. */
#if 0
    /* Check if the packet is meant for the bridge and if so pass the packet
     * to the bridge directly. There is no need to enqueue and then dequeue
     * the packet. */ 
    if ((skb->dev->br_port != NULL) && (br_handle_frame_hook != NULL))
    {
        /* Remember the physical port on which data came in */
		skb->__unused = skb->dev->if_port;
        br_handle_frame_hook(skb);

        /* PANKAJ: Debugging the voice-data coexistence. Return a snap shot of
         * the congestion level reported by the kernel. This can be used by
         * the drivers to stop pushing data up the stack. */
        return 0;
    }
#endif

    /* Enqueue the packet into the Network Queue Manager. */
    netdev_rx_stat[smp_processor_id()].total++;

    /* Pass the Packet to the QoS Framework Network Queue Manager. */
    ptr_ti_qos_mcb->ptr_nqm_handle->enqueue(skb);

    return 0;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_rr_dequeue_packet
 **************************************************************************
 * DESCRIPTION   :
 *  The function dequeus a packet from the Network Queue Manager. This is the
 *  registered routine for the Strict Round Robin Scheduler.
 *
 * RETURNS       :
 *  Packet from the highest priority queue
 *  NULL            - If all queues are empty
 ***************************************************************************/
struct sk_buff* ti_nqm_rr_dequeue_packet (void)
{
    int                     queue_index;
    struct sk_buff*         skb = NULL;
    unsigned long           flags;
    TI_QOS_NQM_RR*          ptr_ti_nqm_rr_mcb;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_DEQUEUE_ENTER);

    /* Get the pointer to the Private Information. */
    ptr_ti_nqm_rr_mcb = (TI_QOS_NQM_RR *)ptr_ti_qos_mcb->ptr_nqm_handle->ptr_nqm_private_info;

    /* Start from the Class of Service 1. Top Most Queue.... */
    queue_index = TI_NSP_QOS_COS1;

    /* Critical Section Start: Disable Interrupts. */    
    local_irq_save(flags);

    /* Cycle through all the queues till the work is done. */
    while (queue_index < TI_NSP_SERVICE_NUMBER)
    {
        /* Select the queue. */
        struct sk_buff_head* queue = &ptr_ti_nqm_rr_mcb->nsp_pkt_queue[queue_index];

        /* Check if there was anything in the queue or not. */
        if (queue->qlen != 0)
        {
            /* Dequeue from this queue. */
            skb = __skb_dequeue(queue);

            /* Check if the queue length has fallen below the threshold and if the queue
             * was previously congested. In that case we need to update the status else
             * there is no change is congestion state.  */
            if ((queue->qlen <= ptr_ti_nqm_rr_mcb->nsp_threshold[queue_index]) && 
                (ptr_ti_nqm_rr_mcb->nsp_queue_state & (1<<queue_index)))
            {
                /* There was a change in the queue state so update the status. */
                ti_nqm_rr_update_congestion_state(ptr_ti_nqm_rr_mcb,(TI_NSP_QOS_CLASS)queue_index, 0); 
            }
            else
            {
                /* No change in state detected... So do nothing! */
            }

            /* The work has been completed. Return the dequeued packet. */
            break;
        }

        /* This queue was empty jump to the next queue. */
        queue_index = queue_index + 1;
    }

    /* Critical Section End: Enable Interrupts. */
    local_irq_restore(flags);

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_DEQUEUE_EXIT);

    return skb;
}

/**************************************************************************
 * FUNCTION NAME : ti_nqm_net_rx_action
 **************************************************************************
 * DESCRIPTION   :
 *  The network receive bottom half handler. The function dequeues the 
 *  network packets and passes them to the appropriate registered protocol
 *  handlers.
***************************************************************************/
static void ti_nqm_net_rx_action(struct softirq_action *h)
{
    register struct sk_buff*    skb;
	unsigned long               start_time = jiffies;
	int                         bugdet = netdev_max_backlog;    
	struct net_device*          rx_dev;
    struct packet_type*         ptype;
    int                         softnet_break = 0;

    /* Start Critical section. */
	br_read_lock(BR_NETPROTO_LOCK);

	for (;;) 
    {        
        /* Dequeue a packet from the Network Queue Manager. */
        skb = ptr_ti_qos_mcb->ptr_nqm_handle->dequeue();

        /* Check if there were any packets in the queue or not ? */
        if (skb != NULL)
        {
            /* YES. Process the dequeued packets. Save the device information on 
             * which the packet was received. */
            rx_dev = skb->dev;

            /* Initialize the data pointers. */
            skb->h.raw = skb->nh.raw = skb->data;
       
            /* Check if the Voice-Data Coexistence Patch is on or not? */
            if (gIsVoiceEnabled == 1)
            {
                /* Check the priority of the packet? */
                if (skb->qos_pkt_info.ti_service_class == TI_NSP_QOS_COS1)
                {
                    /* Highest priority packet: Increase the priority of the softirq daemon. 
                     * Only CoS1 packets are given this treatment. */
                    net_set_priority(1);
                }
                else
                {
                    /* All other priorities run at the normal priority level. */
                    net_set_priority(0);
                }
            }
       
            /* WLAN: Station to Station Handling. */ 
            if ( skb->cb[ SKB_TI_WLAN_IDX ] )
            {
                /* The protocol information is NOT valid. Extract the protocol information 
                 * from the Ethernet header. */
                skb->protocol = *(unsigned short*)(skb->data+12);

                /* Initialize the network header to the correct location. */
                skb->nh.raw = skb->data + sizeof(struct ethhdr);
 
                /* Check if forwarding is also required */
                if ( skb->cb[ SKB_TI_WLAN_IDX ] == SKB_TI_WLAN_PACKET_LOOP_AND_FORWARD )
                {               
                    struct sk_buff *skb2;

                    /* Clear WLAN marking */
                    skb->cb[ SKB_TI_WLAN_IDX ] = 0;

                    /* Clone the skb */
                    if ( (skb2 = skb_clone(skb, GFP_ATOMIC)) != NULL )  
                    {
                       /* Pass the cloned packet back to the driver. */
                       dev_queue_xmit (skb2);

                       goto ti_nqm_net_rx_action_next;
                    }
                }

                /* Clear WLAN marking */
                skb->cb[ SKB_TI_WLAN_IDX ] = 0;
                
                /* Pass the packet back to the driver and return. */
                dev_queue_xmit (skb);
            }
            else
            {
                ti_nqm_net_rx_action_next:
                
                /* Check if the packet is meant for the bridge and if so pass the packet
                 * to the bridge directly. */
                if ((skb->dev->br_port != NULL) && (br_handle_frame_hook != NULL))
                {
                    /* Remember the physical interface on which the packet arrived. */
            		skb->__unused = skb->dev->if_port;

                    /* Since the drivers do NOT call the "eth_type_trans" there is no information
                     * about the protocol currently. */
                    skb->protocol = 0xFFFF;

                    /* Push the packet to the bridge. */
                    br_handle_frame_hook(skb);
                }
                else
                {
                    /* NO. This was NOT meant for the bridge. Determine the protocol handler. */
                    ptype = ptype_base[ntohs(skb->protocol)&15];
                    if (ptype != NULL)
                    {
                        /* Check if there is another registered handler for the same packet. */
                        if (ptype->next == NULL)
                        {
                            /* NO. Pass the packet to the handler */
                            ptype->func(skb, skb->dev, ptype);
                        }
                        else
                        {
                            /* YES. Push data to all registered protocol handlers. */
                            net_pass_data_other_handlers (ptype, skb);
                        }
                    }
                    else
                    {
                        /* Invalid packet as there is no registered handler. Clean memory
                         * and drop the packet. This is not the fast path so call the error 
                         * function to do the needful. Reducing the amount of code in the fast
                         * path. */
                        net_error_handler(skb);
                    }
                }
            }

            /* Release the receive device on which the packet was received. */
        	dev_put(rx_dev);

            /* Check if the NET RX BH has reached its execution limits. */
        	if ((bugdet-- < 0) || (jiffies - start_time) > 1)
            {
	            softnet_break = 1;
                break;
            }
        }
        else
        {
            /* No more packets left, we are done. */
            break;
        }
	}

    /* Exit Criticial section. */
	br_read_unlock(BR_NETPROTO_LOCK);

    /* Disable Interrupts. */
    local_irq_disable();
    if (softnet_break == 1)
    {
        /* Net receive bottom half exceeded its time; increment the statistics. 
         * and reschedule the bottom half for execution again. */
	    netdev_rx_stat[smp_processor_id()].time_squeeze++;
	    cpu_raise_softirq(smp_processor_id(), NET_RX_SOFTIRQ);
    }

    /* Enable Interrupts. */
    local_irq_enable();

	NET_PROFILE_LEAVE(softnet_process);
	return;
}

#ifdef CONFIG_MIPS_AVALANCHE_PROTOCOL_INSPECT

/**************************************************************************
 * FUNCTION NAME : ti_pil_classify_ip_packet
 **************************************************************************
 * DESCRIPTION   :
 *  This function is called to classify an IP Packet. This is typically done
 *  using the IP header TOS byte. This could be called after the Ethernet Type
 *  classifications are completed or could also be called from the drivers 
 *  directly. Typically PPPoA and Routed 2684 are examples where there is no
 *  Ethernet header in the packet so those drivers directly call these 
 *  functions. The function does the classification using the rules specified
 *  in the TCA.
 *
 * RETURNS       :
 *  1   - Accept the packet
 *  0   - Drop the packet.
 *  
 * NOTES        : 
 *  If the ptr_iphdr passed to the function is NULL. It is treated
 *  as a CONTROL packet i.e. ARP or PPP control packets. These packets are then
 *  given a priority as per the TCA. 
 ***************************************************************************/
int ti_pil_classify_ip_packet (struct sk_buff* skb, struct iphdr* ptr_iphdr)
{
    TI_QOS_PACKET_INFO* ptr_ti_qos_info;
    TI_QOS_TCA*         ptr_ti_qos_tca;
    
    /* Check if the QoS is enabled? If QoS is disabled then we always accept the packet. */
    if (gIsQoSEnabled == 0)
        return 1;
    
    /* Trace Point: Start */
    psp_trace (NSP_QOS_PIL_IP_ENTER);
 
    /* Get the pointer to the QoS Classification. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* Get the TCA associated with the device. */
    ptr_ti_qos_tca = (TI_QOS_TCA *)skb->dev->ptr_ti_qos_tca;

    /* Check if there is a TCA associated with the device. If not then treat those packets
     * to have the lowest priority. */
    if (ptr_ti_qos_tca == NULL)
    {
        /* NO TCA associated with the device. Mark this packet to CoS6 */
        ptr_ti_qos_info->ti_service_class  = TI_NSP_QOS_COS6;
    }
    else
    {
        /* YES. TCA was present. Check if the TCA is running in STATIC mode or not? */
        if (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_STATIC)
        {
            /* YES. Use the default mapping in the TCA to return the class of service. */
            ptr_ti_qos_info->ti_service_class  = ptr_ti_qos_tca->src_interface_service_class;
        }
        else
        {
            /* NO. The TCA is configured to operate in DYNAMIC mode. Check if this is a valid
             * IP Packet. If NOT then this has to be a control packet. This is because this 
             * function need not necessarily be called after the Ethernet Classification. 
             * For PPPoA and Routed 2684 Encapsulations this function is called directly. */
            if (ptr_iphdr == NULL)
            {
                /* Mark the service class with the default value. */
                ptr_ti_qos_info->ti_service_class  = ptr_ti_qos_tca->default_non_ip_ingress_service_class;
            }
            else
            {
                /* Valid IP Packet... Check if the TCA is TRUSTED or not? */
                if (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_LAYER3_TRUSTED)
                {                   
                    /* YES. This is a TRUSTED Interface. Honor Layer 3 Priorities if any. 
                     * Check if the TOS byte is present to decide the QoS Service Level. */
                    if (ptr_iphdr->tos != 0)
                    {
                        /* YES TOS Byte Prevails. */
                        ptr_ti_qos_info->ti_service_class = ptr_ti_qos_tca->ingress_layer3_mapping[ptr_iphdr->tos];
                    }
                    else
                    {
                        /* NO. The TOS Byte is NOT present. Use the Payload database now. */
                        ptr_ti_qos_info->ti_service_class = ti_qos_lookup_payload_database (skb, ptr_iphdr);
                    }
                }
                else
                {
                    /* NO. This is NOT a trusted interface. Dont honor the TOS byte. Look at the Rule Database. */
                    ptr_ti_qos_info->ti_service_class = ti_qos_lookup_payload_database (skb, ptr_iphdr);
                }
            }
        }
    }

    /* Fill up the NSP QoS Information. Set the bit indicating that the packet has been classified. */
    ptr_ti_qos_info->ti_qos_flags       = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_PIL_IP_EXIT);

    /* Return the congestion control on the basis of the class of service of the packet. */
    return ptr_ti_qos_mcb->ptr_nqm_handle->congestion_check(skb, ptr_ti_qos_info->ti_service_class);
}

/**************************************************************************
 * FUNCTION NAME : ti_pil_classify_ethernet_frame
 **************************************************************************
 * DESCRIPTION   :
 *  The function is called from the drivers to classify a packet on the 
 *  basis of the "Layer2". This function can be called from the drivers 
 *  only if the received frame is an ETHERNET Frame.
 *
 *  Certain protocols like PPPoA and 2684 Routed encapsulation DO NOT 
 *  call this routine.
 *  
 * RETURNS       :
 *      1   - Accept the packet
 *      0   - Drop the packet.
 *      
 *  It also marks the SKB priority field to indicate the new priority of the
 *  received packet.
 ***************************************************************************/
int ti_pil_classify_ethernet_frame (struct sk_buff* skb, int protocol_offset)
{
    struct iphdr*       ptr_iphdr = NULL;
    TI_QOS_PACKET_INFO* ptr_ti_qos_info;
    TI_QOS_TCA*         ptr_ti_qos_tca;

    /* Check if the QoS is enabled? If QoS is disabled then we always accept the packet. */
    if (gIsQoSEnabled == 0)
        return 1;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_PIL_ETH_ENTER);

    /* Get the TCA associated with the device. */
    ptr_ti_qos_tca = (TI_QOS_TCA *)skb->dev->ptr_ti_qos_tca;

    /* Get the pointer to the QoS Classification. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* Check if there is a TCA associated with the device. If not then treat those packets
     * to have the lowest priority. */
    if (ptr_ti_qos_tca == NULL)
    {
        /* NO TCA associated with the device. Mark this packet to CoS6 */
        ptr_ti_qos_info->ti_qos_flags   = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
        ptr_ti_qos_info->ti_service_class  = TI_NSP_QOS_COS6;
    }
    else
    {
        /* YES. TCA was present. Check if the TCA is running in STATIC mode or not? */
        if (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_STATIC)
        {
            /* YES. Use the default mapping in the TCA to return the class of service. */
            ptr_ti_qos_info->ti_qos_flags   = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
            ptr_ti_qos_info->ti_service_class  = ptr_ti_qos_tca->src_interface_service_class;
        }
        else
        {
            /* NO. The TCA is running in Dynamic Mode. */
            struct ethhdr*  ptr_ethhdr;
            char*           ptr_data;
            unsigned short  protocol_type;

            /* Skip the protocol offset as indicated by the drivers. This is to skip any 2684 
             * Bridged/Routed LLC/VC Mux encapsulations. In the case of the Ethernet driver for 
             * example the protocol offset is 0. */
            ptr_data = skb->data + protocol_offset;

            /* Initialize the Ethernet Header. */
            ptr_ethhdr = (struct ethhdr *)ptr_data;

            /* Skip the Ethernet Header. */
            ptr_data        = ptr_data + sizeof(struct ethhdr);
            protocol_type   = ptr_ethhdr->h_proto;

            /* Possible values of the frame type are as follows:- 
             *  a) IP 
             *  b) ARP
             *  c) PPP Session and Discovery
             *  d) VLAN 
             *      VLAN Tags could in turn have any of the above mentioned tags integrated. 
             *      So for VLAN Tags we have a special case; look into the encapsulated header.*/
            if (protocol_type == __constant_htons(ETH_P_8021Q))
            {
                /* Pass the control to the PIL VLAN Protocol Handler. This will fill up the 
                 * mapped DSCP which will be marked later into the packet. */
                protocol_type = ti_pil_handle_vlan_packet (&ptr_data, ptr_ti_qos_info, ptr_ti_qos_tca);
            }

            /* Processing is now based on the protocol type that has been calculated. */
            switch (protocol_type)
            {
                case __constant_htons(ETH_P_IP):
                {
                    /* Standard Ethernet IP Frame. */
                    ptr_iphdr = (struct iphdr*)ptr_data;
                    break;
                }
                case __constant_htons(ETH_P_PPP_DISC):
                case __constant_htons(ETH_P_PPP_SES):
                {
                    /* PPP Packet: Skip the PPP header. */
                    ptr_data = ptr_data + 6;
    
                    /* Get the PPP Protocol Information*/
                    protocol_type = *((unsigned short *)ptr_data);
                    if (protocol_type == 0x2100)
                    {
                        /* IP Packet. */
                        ptr_iphdr = (struct iphdr *)(ptr_data + 2);
                    }
                    else
                    {
                        /* PPP Control Packets. */
                    }
                    break;
                }
                case __constant_htons(ETH_P_ARP):
                {
                    /* ARP Packets. These should be treated as per the TCA.  */
                    break;
                }
                default:
                {
                    /* Unknown packets: Mark them as LOW priority. */
                    ptr_ti_qos_info->ti_qos_flags   = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
                    ptr_ti_qos_info->ti_service_class  = TI_NSP_QOS_COS6;
                    break;
                }
            }
        }
    }

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_PIL_ETH_EXIT);

    /* Has the packet already been classified? */
    if ((ptr_ti_qos_info->ti_qos_flags & TI_QOS_PACKET_CLASSIFIED) == 0)
    {
        /* NO. Layer 2 was NOT sufficient to do the needful. Move on to the next step. */
        return ti_pil_classify_ip_packet(skb, ptr_iphdr);
    }
    else
    {
        /* YES. The packet has been classified. Return the Congestion control on the basis 
         * of the class of service of the packet. */
        return ptr_ti_qos_mcb->ptr_nqm_handle->congestion_check(skb, ptr_ti_qos_info->ti_service_class);
    }
}

/**************************************************************************
 * FUNCTION NAME : ti_pil_handle_vlan_packet
 **************************************************************************
 * DESCRIPTION   :
 *  The function is the generic handler for all VLAN tagged packets. The
 *  function parses the packets and extracts user priorities. It then 
 *  uses the TCA for the interface to decide on the NSP service level
 *  and ToS byte that is supported.
 *  
 * RETURNS       :
 *  Encapsulated Protocol Field residing inside the VLAN header.
 ***************************************************************************/
static unsigned short ti_pil_handle_vlan_packet 
(
    char**              ptr_data,
    TI_QOS_PACKET_INFO* ptr_ti_qos_info,
    TI_QOS_TCA*         ptr_ti_qos_tca
)
{
    struct vlan_hdr*    ptr_vlanheader;

    /* Get the VLAN header. */
    ptr_vlanheader = (struct vlan_hdr *)(*ptr_data);

    /* Skip the VLAN Header. */
    *ptr_data = *ptr_data + sizeof(struct vlan_hdr);

    /* Check if we are allowed to prioritize the packet i.e. The interface is a Layer 2 Trusted Interface. */
    if (ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_LAYER2_TRUSTED)
    {
        /* YES. There was a valid TCA. Get the user priority information from the VLAN header. */
        unsigned short user_priority = ((ntohs(ptr_vlanheader->h_vlan_TCI)) & 0xE000) >> 13;

        /* Fill up the QOS Packet Information structure. */
        ptr_ti_qos_info->ti_qos_flags       = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
        ptr_ti_qos_info->ti_service_class   = ptr_ti_qos_tca->ingress_layer2_mapping[user_priority];
    }

    /* Return the encapsulated protocol. */
    return ptr_vlanheader->h_vlan_encapsulated_proto;
}

/**************************************************************************
 * FUNCTION NAME : ti_pil_classify_wlan_frame
 **************************************************************************
 * DESCRIPTION   :
 *  The function is to be called only for the WLAN driver. The function 
 *  classifies the packet on the basis of only the skb->priority. The 
 *  function does NOT assume any of the other fields in skb are valid.
 *  
 * RETURNS       :
 *      1   - Accept the packet
 *      0   - Drop the packet.
 *
 * NOTES         :
 *  The function does NOT do any sanity check of the range of the "priority"
 *  field. It is assumed that the WLAN driver fills the "priority" field
 *  correctly.
 ***************************************************************************/
int ti_pil_classify_wlan_frame (struct sk_buff* skb, struct net_device* dev)
{
    TI_QOS_PACKET_INFO* ptr_ti_qos_info;
    TI_QOS_TCA*         ptr_ti_qos_tca;

    /* Check if the QoS is enabled? If QoS is disabled then we always accept the packet. */
    if (gIsQoSEnabled == 0)
        return 1;

    /* Trace Point: Start */
    psp_trace (NSP_QOS_PIL_WLAN_ENTER);

    /* Get the TCA associated with the device. */
    ptr_ti_qos_tca = (TI_QOS_TCA *)dev->ptr_ti_qos_tca;

    /* Get the pointer to the QoS Classification. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;

    /* Check if there is a TCA associated with the WLAN device and if so it is configured 
     * to run in Layer 2 TRUSTED Mode. */
    if ((ptr_ti_qos_tca == NULL) || ((ptr_ti_qos_tca->tca_mode & TI_NSP_QOS_LAYER2_TRUSTED) == 0))
    {        
        /* There was no TCA or it was not configured for trusted mode. Treat all packets as CoS6. */
        ptr_ti_qos_info->ti_service_class = TI_NSP_QOS_COS6;
    }
    else
    {
        /* Use the Ingress mappings to determine the Service class. */
        ptr_ti_qos_info->ti_service_class = ptr_ti_qos_tca->ingress_layer2_mapping[skb->priority];
    }

    /* Mark the packet as been classifed. */
    ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;

    /* Trace Point: Stop */
    psp_trace (NSP_QOS_PIL_WLAN_EXIT);

    /* Return the congestion control on the basis of the service class. */
    return ptr_ti_qos_mcb->ptr_nqm_handle->congestion_check(skb, ptr_ti_qos_info->ti_service_class);
}

/**************************************************************************
 * FUNCTION NAME : ti_pil_classify_wlan_eapol_frame
 **************************************************************************
 * DESCRIPTION   :
 *  The function is to be called only for the WLAN driver. This should be 
 *  called only for EAPOL packets and it marks the packet as CoS1 
 *  irrespective of the mode.
 *  
 * RETURNS       :
 *      1   - Accept the packet
 *      0   - Drop the packet.
 ***************************************************************************/
int ti_pil_classify_wlan_eapol_frame (struct sk_buff* skb)
{
    TI_QOS_PACKET_INFO* ptr_ti_qos_info;

    /* Check if the QoS is enabled? If QoS is disabled then we always accept the packet. */
    if (gIsQoSEnabled == 0)
        return 1;
 
    /* Get the pointer to the QoS Classification. */
    ptr_ti_qos_info = (TI_QOS_PACKET_INFO *)&skb->qos_pkt_info;
 
    /* Mark the packet as CoS1. */
    ptr_ti_qos_info->ti_service_class = TI_NSP_QOS_COS1;
 
    /* Mark the packet as been classifed. */
    ptr_ti_qos_info->ti_qos_flags = ptr_ti_qos_info->ti_qos_flags | TI_QOS_PACKET_CLASSIFIED;
 
    /* Return the congestion control on the basis of the service class. */
    return ptr_ti_qos_mcb->ptr_nqm_handle->congestion_check(skb, ptr_ti_qos_info->ti_service_class);
}

EXPORT_SYMBOL_NOVERS(ti_pil_classify_ip_packet); 
EXPORT_SYMBOL_NOVERS(ti_pil_classify_ethernet_frame);
EXPORT_SYMBOL_NOVERS(ti_pil_handle_vlan_packet);
EXPORT_SYMBOL_NOVERS(ti_pil_classify_wlan_frame);
EXPORT_SYMBOL_NOVERS(ti_pil_classify_wlan_eapol_frame);

#endif
