/* $Id: getifstats.c,v 1.5 2007/12/20 11:53:05 nanard Exp $ */
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006,2007 Thomas Bernard 
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <time.h>

#include "../getifstats.h"
#include "../config.h"

int
getifstats(const char * ifname, struct ifdata * data)
{
	FILE *f;
	char line[512];
	char * p;
	int i;
	int r = -1;
#ifdef ENABLE_GETIFSTATS_CACHING
	static time_t cache_timestamp = 0;
	static struct ifdata cache_data;
	time_t current_time;
#endif
#ifdef ROUTERTECH
#include <string.h>
#include <unistd.h>
    unsigned int UpstreamMaxBitRate=0;
    unsigned int DownstreamMaxBitRate=0;
    int uptime=1;
    int retval;
#else // ROUTERTECH
	data->baudrate = 4200000;
	data->opackets = 0;
	data->ipackets = 0;
	data->obytes = 0;
	data->ibytes = 0;
#endif //  ROUTERTECH
#ifdef ENABLE_GETIFSTATS_CACHING
	current_time = time(NULL);
	if(current_time == ((time_t)-1)) {
		syslog(LOG_ERR, "getifstats() : time() error : %m");
	} else {
		if(current_time < cache_timestamp + GETIFSTATS_CACHING_DURATION) {
			memcpy(data, &cache_data, sizeof(struct ifdata));
#ifdef ROUTERTECH
            /* pause a fraction of a second to keep CPU load down */
            usleep(100000);
#endif
			return 0;
		}
	}
#endif

#ifdef ROUTERTECH
	data->opackets = 0;
	data->ipackets = 0;
	data->obytes = 0;
	data->ibytes = 0;
    f = fopen("/proc/uptime", "r");
    if(!f) {
        syslog(LOG_ERR, "getifstats() : cannot open /proc/uptime : %m");
        return -1;
    }
	memset(line, 0, sizeof(line));
    retval = read (fileno(f), (void *)line, sizeof(line) - 1);
    fclose(f);
    uptime = atoi(line);
#ifdef DEBUG
    printf("line: %s\n", line);
    printf("system uptime is %d seconds\n", uptime);
#endif

    /* Get rates from proc entry */
    f = fopen("/proc/avalanche/avsar_modem_stats", "r");
	if(!f) {
		syslog(LOG_ERR, "getifstats() : cannot open /proc/avalanche/avsar_modem_stats : %m");
		return -1;
    }
    fread(line, sizeof(char), sizeof(line), f);
    fclose(f);

    p=strstr(line, "US Connection Rate:");
    p += strlen("US Connection Rate:");
    if(p)
    {
        i = sscanf(p," %u", &UpstreamMaxBitRate);
#ifdef DEBUG
        printf("retval = %d, upstream = %u\n", i, UpstreamMaxBitRate );
#endif
    }

    p=strstr(line, "DS Connection Rate:");
    p += strlen("DS Connection Rate:");
    if(p)
    {
        i = sscanf(p," %u", &DownstreamMaxBitRate );
#ifdef DEBUG
        printf("retval = %d, downstream = %u\n", i, DownstreamMaxBitRate );
#endif
    }

    data->baudrate = (long unsigned int) (DownstreamMaxBitRate*1000);
    data->uprate = (long unsigned int) (UpstreamMaxBitRate*1000);
    data->uptime = (time_t) uptime;

#endif

	f = fopen("/proc/net/dev", "r");
	if(!f) {
		syslog(LOG_ERR, "getifstats() : cannot open /proc/net/dev : %m");
		return -1;
	}
	/* discard the two header lines */
	fgets(line, sizeof(line), f);
	fgets(line, sizeof(line), f);
	while(fgets(line, sizeof(line), f)) {
		p = line;
		while(*p==' ') p++;
		i = 0;
		while(ifname[i] == *p) {
			p++; i++;
		}
		/* TODO : how to handle aliases ? */
		if(ifname[i] || *p != ':')
			continue;
		p++;
		while(*p==' ') p++;
		data->ibytes = strtoul(p, &p, 0);
		while(*p==' ') p++;
		data->ipackets = strtoul(p, &p, 0);
		/* skip 6 columns */
		for(i=6; i>0 && *p!='\0'; i--) {
			while(*p==' ') p++;
			while(*p!=' ' && *p) p++;
		}
		while(*p==' ') p++;
		data->obytes = strtoul(p, &p, 0);
		while(*p==' ') p++;
		data->opackets = strtoul(p, &p, 0);
		r = 0;
		break;
	}
	fclose(f);
#ifdef ENABLE_GETIFSTATS_CACHING
	if(r==0 && current_time!=((time_t)-1)) {
		cache_timestamp = current_time;
		memcpy(&cache_data, data, sizeof(struct ifdata));
	}
#endif
	return r;
}

