#*******************************************************************************
#*+--------------------------------------------------------------------------+**
#*|                            ****                                          |**
#*|                            ****                                          |**
#*|                            ******o***                                    |**
#*|                      ********_///_****                                   |**
#*|                      ***** /_//_/ ****                                   |**
#*|                       ** ** (__/ ****                                    |**
#*|                           *********                                      |**
#*|                            ****                                          |**
#*|                            ***                                           |**
#*|                                                                          |**
#*|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
#*|                        ALL RIGHTS RESERVED                               |**
#*|                                                                          |**
#*| Permission is hereby granted to licensees of Texas Instruments           |**
#*| Incorporated (TI) products to use this computer program for the sole     |**
#*| purpose of implementing a licensee product based on TI products.         |**
#*| No other rights to reproduce, use, or disseminate this computer          |**
#*| program, whether in part or in whole, are granted.                       |**
#*|                                                                          |**
#*| TI makes no representation or warranties with respect to the             |**
#*| performance of this computer program, and specifically disclaims         |**
#*| any responsibility for any damages, special or consequential,            |**
#*| connected with the use of this program.                                  |**
#*|                                                                          |**
#*+--------------------------------------------------------------------------+**
#*******************************************************************************

# ******************************************************************************
# * FILE PURPOSE:    PSPBoot build configuration include
# ******************************************************************************
# * FILE NAME:       bootcfg.mak
# *
# * DESCRIPTION:     Skeleton Makefile include for PSPBoot build.
# *
# * (C) Copyright 2005, Texas Instruments, Inc
# *****************************************************************************/

# This file is meant for building targets that are not built automatically
# during release. For each target, there is a board configuration makefile
# which needs to be included. In order to compile one of the follwing targets,
# use the follwing example:
# make BOARD_CONFIG=bin/builds/inc/hw_tnetc401b.mak

# Use this to specify the endianess of the board. 
# Valid values are LE (Little Endian)and BE (Big Endian)
ENDIAN=BE

# Use this to select the preferred network vehicle.
# Valid options are FTP (for FTP client), TFTP (for TFTP client) 
# and NONE (for neither).
NETWORK_VEHICLE=TFTP

# Use this to configure DHCP based network settings
# Valid options are YES or NO.
DHCP=YES

# Use this to select the PC APP Support.
# Valid options are YES or NO. When Selecting this option NETWORK_VEHICHLE
# should be chosen as NONE.
PC_APP=NO

# Use this to select whether Flash File system support is needed or not.
# Valid options are YES or NO.
FFS=YES

# Use this to select whether Secondary Flash File system support is needed or not.
# Valid options are YES or NO. Option ignored if FFS is disabled.
SECONDARY_FFS=YES

# Use this to select whether permanent variables support is needed or not.
# Valid options are YES or NO.
PERMANENT_VARIABLES=YES

# Use this to select your build platform. Valid options are
# WINDOWS 		: Use the Tornado Toolchain. (running on Windows)
# LINUX			: Use the monta-vista tool chain (running on Linux)
PLATFORM=WINDOWS

# GZip support. Use this to select GNU zip file format support
# Valid options are YES or NO
CONF_OMOD_GZIP=YES

# LZMA support. Use this to select LZMA file format support
# Valid options are YES or NO
CONF_OMOD_LZMA=YES

# 7-Zip support. Use this to select 7-Zip file format support.
# Compressed image must by in flat binary format.
# Valid options are YES or NO
CONF_OMOD_7ZIP=YES

# ELF support. Use this to select ELF file format support
# Valid options are YES or NO
CONF_OMOD_ELF=YES

# TIBINARY support. Use this to select TI proprietary file format support
# Valid options are YES or NO
CONF_OMOD_TIBINARY=YES

# NSP Single image support (Image with header, instead of mtd).
# Valid options are YES or NO. 
CONF_OMOD_NSP_SINGLE_IMAGE=NO

# Dual boot image support. Enables best image selection out of two single images. 
# Available only with NSP single image support. 
# Without this option, Dual option is ignored. Valid options are YES or NO. 
CONF_OMOD_DUAL=NO

# TFTP failover support. Download an image from TFTP if failed to boot.
# Valid options are YES or NO. Available only with TFTP support. Without TFTP
# support this option is ignored.
CONF_OMOD_TFTP_FAILOVER=YES

# Reboot command.
# Valid options are YES or NO
CONF_OCMD_REBOOT=YES

# Version command.
# Valid options are YES or NO
CONF_OCMD_VERSION=YES

# Info command.
# Valid options are YES or NO
CONF_OCMD_INFO=YES

# Fa command.
# Valid options are YES or NO
CONF_OCMD_FA=YES

# Printenv command.
# Valid options are YES or NO
CONF_OCMD_PRINTENV=YES

# Setenv command.
# Valid options are YES or NO
CONF_OCMD_SETENV=YES

# Setperenv command.
# Valid options are YES or NO
CONF_OCMD_SETPERENV=YES

# Unsetenv command.
# Valid options are YES or NO
CONF_OCMD_UNSETENV=YES

# Defragenv command.
# Valid options are YES or NO
CONF_OCMD_DEFRAGENV=YES

# Fmt command.
# Valid options are YES or NO
CONF_OCMD_FMT=YES

# Boot command.
# Valid options are YES or NO
CONF_OCMD_BOOT=YES

# Dm command.
# Valid options are YES or NO
CONF_OCMD_DM=YES

# Oclk command
# Valid options are YES or NO
CONF_OCMD_OCLK=YES

# Help command
# Valid options are YES or NO
CONF_OCMD_HELP=YES

# Ls command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_LS=YES

# Df command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_DF=YES

# Cp command.
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_CP=YES

# Cat command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_CAT=YES

# Rm command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_RM=YES

# TFTP command. 
# Valid options are YES or NO. Available only with TFTP support. Without TFTP
# support (NETWORK_VEHICLE != TFTP) this option is taken as NO. When NETWORK_VEHICLE is TFTP and this
# option is NO, it means that TFTP download support (for network bootup) is 
# available, but the 'tftp' command is not available from shell. 
CONF_OCMD_TFTP=YES

# FTP command. 
# Valid options are YES or NO. Available only with FTP support. Without FTP
# support (NETWORK_VEHICLE != FTP) this option is taken as NO. When NETWORK_VEHICLE is FTP and this
# option is NO, it means that FTP engine is present but the 'ftp' command is 
# not available. 
CONF_OCMD_FTP=NO

# Flash memory configuration: 
# Select the source file for flash driver from psbl/kernel/flash directory, or use default configuration. 
# SRCS_FLASH_C= 

# Board hardware configuration: 
# Use BOARD_CONFIG=<filename> to generate configuration header file.
 
include $(BOARD_CONFIG)

# Miscellaneous configuration

# Maximum size of environment variables block (or DEFAULT)
ENV_SPACE_SIZE = DEFAULT

# Approximate size of secondary file system in bytes. Ignored if secondary FFS not enabled
SECONDARY_FFS_SIZE = ( 512 * 1024 )
